// ********************************************************************
//
//  Routine:      acGlobalAlignmentTransformation.C
//  Author:       T. Oest
//
//  Description:  applies the global alignment transformation
//                on GEDE tables
//
//  Modification:
//   -- update gede->xfw[0] for VDS                  27/09/99 T.Oest
//   -- routine reactivated                          07/01/00 T.Oest
//   -- routine activated, 
//      global transformation only if internal subsystem
//      constants changed                            10/03/00 T.Oest
//   -- include ITR/OTR transformations              26/05/00 T.Oest
//
// 08/12/00 (SKAG) - ACGA/GEDE matrix multiplication rewritten
//                 - Added RICH tx/ty transformation
//                 - OTR divided into 4 parts +PC/MC, -PC/MC, +TC and -TC
//                 - Included scaling in z
//                 - ECAL transformations added 
//                 - ITR transformations included (ITR divided in 4 quadrants:
//                   +x,+y; +x,-y; -x,+y; -x,-y; 
// 17/01/01 (SKAG) - RICH tx/ty transformation made clearer
// 19/01/01 (SKAG) - bug fixed (in scaling in zO)
// ********************************************************************

#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>

#include "arte/arte.hh"
#include "component.h"



unsigned int GEDEcmpToComponent(int) ;
int OTcmp ( ArtePointer<GEDE> gede ) ;
int ITcmp ( ArtePointer<GEDE> gede ) ;

extern "C"
{
 void rinv_(const int    & n,
            const float  * wMeas,
            const int    & idim,
            const float  * r,
            const int    & ifail);
}

// const int indexARTE[3][3] = {{0, 3,  6},
//                              {1, 4,  7},
//                              {2, 5,  8}};

//****************************************************************
// function
//****************************************************************
void acGlobalAlignmentForGEDE(unsigned int alcaUpdate) {

//----------------------------------------------------------------
// loop over ACGA table
//----------------------------------------------------------------
  for( ArteTable<ACGA>::iterator acga = ArteTable<ACGA>::begin();
       acga != ArteTable<ACGA>::end(); ++acga ) {

    if (GEDEcmpToComponent((*acga)->cmp & 0xFFFF) & alcaUpdate) {
      
      switch ( (*acga)->cmp & 0xFFFF ) {
	
      case 5: {//RICH
	cout << "updating RICH\n" ;
	float sx=sin((*acga)->dtx) ;
	float sy=sin((*acga)->dty) ;
	float cx=cos((*acga)->dtx) ;
	float cy=cos((*acga)->dty) ;
	
	int upper = ((*acga)->cmp>>16) ; // 0x1 upper RICH
	// 0x0 lower RICH
	for ( ArteTable<CSSM>::iterator shmi=ArteTable<CSSM>::begin() ;
	      shmi!=ArteTable<CSSM>::end(); ++shmi ) {
	  cout << "in the loop\n" ;
	  if ( ( (*shmi)->orig[1]>0. && upper ) ||
	       ( (*shmi)->orig[1]<0. && (!upper) ) ) {
	    // ty
	    (*shmi)->norm[1] =  cy*(*shmi)->norm[1] + sy*(*shmi)->norm[2] ;
	    (*shmi)->norm[2] = -sy*(*shmi)->norm[1] + cy*(*shmi)->norm[2] ;
	    
	    // tx
	    (*shmi)->norm[0] = cx*(*shmi)->norm[0] - sx*(*shmi)->norm[2] ;
	    (*shmi)->norm[2] = sx*(*shmi)->norm[0] + cx*(*shmi)->norm[2] ;
	    
	  }
	}
	break ;
      }

      case 7: {//ECAL
  	for ( ArteTable<GESL>::iterator sl=ArteTable<GESL>::begin() ;
  	      sl!=ArteTable<GESL>::end(); ++sl ) {
  	  if ( ((*acga)->cmp & 0xFFFF) == (*sl)->cmp ) {
  	    (*sl)->x1 += (*acga)->dx ;
  	    (*sl)->x2 += (*acga)->dx ;
  	    (*sl)->y1 += (*acga)->dy ;
  	    (*sl)->y2 += (*acga)->dy ;
  	    (*sl)->z1 += (*acga)->dz ;
  	    (*sl)->z2 += (*acga)->dz ;
  	  }
  	}
//  	for ( ArteTable<GCAL>::iterator gc=ArteTable<GCAL>::begin() ;
//  	      gc!=ArteTable<GCAL>::end(); gc++ ) {
//  	  if ( ((*acga)->cmp & 0xFFFF) == (*gc)->cmp ) {
//  	    (*gc)->xmin += (*acga)->dx ;
//  	    (*gc)->xmax += (*acga)->dx ;
//  	    (*gc)->ymin += (*acga)->dy ;
//  	    (*gc)->ymax += (*acga)->dy ;
//  	    (*gc)->zmin += (*acga)->dz ;
//  	    (*gc)->zmax += (*acga)->dz ;
//  	  }
//  	}
	
	break ;
      }
      case 1: //SVD
      case 2: //OTR
      case 3: //ITR
      case 4: //HiPt
      case 6: //TRD
      case 8: //MUON OUTER
      case 9:{//MUON CENTRAL
 
//----------------------------------------------------------------
// acga rotation matrix:
// e.g. z-rotation:
//      ( 0 1 2 )    ( cos -sin 0 )
//      ( 3 4 5 )  = ( sin  cos 0 )
//      ( 6 7 8 )    (  0    0  1 )
//----------------------------------------------------------------

	float ACGArot[3][3];

	int l=0 ;
	for (int i=0; i<3; i++){for (int j=0; j<3; j++){
	  ACGArot[i][j]  = (*acga)->rot[l++];
	};};

	
//  	cout << "  " << endl;
//  	cout << " acga matrix " << endl;
//  	cout << GEDElikeMatrix[0][0] << " " 
//  	     << GEDElikeMatrix[0][1] << " " 
//  	     << GEDElikeMatrix[0][2] << endl;
//  	cout << GEDElikeMatrix[1][0] << " " 
//  	     << GEDElikeMatrix[1][1] << " " 
//  	     << GEDElikeMatrix[1][2] << endl;
//  	cout << GEDElikeMatrix[2][0] << " "  
//  	     << GEDElikeMatrix[2][1] << " "  
//  	     << GEDElikeMatrix[2][2] << endl;


//----------------------------------------------------------------
// loop over GEDE tables
//----------------------------------------------------------------
	for( ArteTable<GEDE>::iterator gede = ArteTable<GEDE>::begin();
	     gede != ArteTable<GEDE>::end(); ++gede ) {
    
	  if ( ((*acga)->cmp & 0xFFFF) == (*gede)->cmp) {

	    if ( ((*gede)->cmp==2) && //OTR
		 (OTcmp(gede) != (((*acga)->cmp)>>16)) ) continue ;

	    if ( ((*gede)->cmp==3) && //ITR
		 (ITcmp(gede) != (((*acga)->cmp)>>16)) ) continue ;

	    
	    
	    float GEDErot[3][3], GEDErotnew[3][3]; 

	    l=0 ;
	    for (int i=0; i<3; i++){
	      for (int j=0; j<3; j++){
		GEDErot[i][j] = (*gede)->rot[l++];
	      }
	    }

//----------------------------------------------------------------
// transformation
//     new GEDE rot matrix:   Gnew = A * G
//----------------------------------------------------------------

	    l=0 ;
	    for (int i=0; i<3; i++){
	      for (int j=0; j<3; j++){
		GEDErotnew[i][j] = 0.0;
		for (int k=0; k<3; k++){
		  GEDErotnew[i][j] += ACGArot[i][k] * GEDErot[k][j];
		}
		(*gede)->rot[l++] = GEDErotnew[i][j];
	      }
	    }

//----------------------------------------------------------------
// transformation
//     new position       :   Rnew = R + Gnew**-1 * dr
//----------------------------------------------------------------

	    float dz0, dr[3], drRot[3];

	    dz0 = 0.5*((*gede)->zmin+(*gede)->zmax)
	      + (*acga)->dz - (*acga)->z;

	    dr[0] = (*acga)->dx + (*acga)->dtx * dz0;
	    dr[1] = (*acga)->dy + (*acga)->dty * dz0;
	    dr[2] = (*acga)->dz ;

	    for (int i=0; i<3; i++){
	      drRot[i] = 0.0;
	      for (int j=0; j<3; j++){
		drRot[i] = drRot[i] + GEDErotnew[j][i] * dr[j];
	      }
	    }

	    (*gede)->xmin = (*gede)->xmin + drRot[0];
	    (*gede)->xmax = (*gede)->xmax + drRot[0];
	    (*gede)->ymin = (*gede)->ymin + drRot[1];
	    (*gede)->ymax = (*gede)->ymax + drRot[1];
	    (*gede)->zmin = ((*gede)->zmin + drRot[2]) * ((*acga)->fz)+
	      ((*acga)->z) * (1.-(*acga)->fz) ;
	    (*gede)->zmax = ((*gede)->zmax + drRot[2]) * ((*acga)->fz)+
	      ((*acga)->z) * (1.-(*acga)->fz) ;
      
	    (*gede)->sxmn = (*gede)->sxmn + drRot[0];
	    (*gede)->sxmx = (*gede)->sxmx + drRot[0];
	    (*gede)->symn = (*gede)->symn + drRot[1];
	    (*gede)->symx = (*gede)->symx + drRot[1];

    
//----------------------------------------------------------------
// detector specific updates
//----------------------------------------------------------------

	    if ((*gede)->cmp == 1){  //VDS
	      (*gede)->xfw[0] = (*gede)->cos * (*gede)->sxmn
		-(*gede)->sin * (*gede)->symn 
		+ (*gede)->xfw[1];
	    }

	    if ((*gede)->cmp == 2 || (*gede)->cmp == 3){  //OTR/ITR
	      for (int i = 0 ; i < (*gede)->nsl ; i++) {
		(*gede)->xfw[i] = (*gede)->xfw[i] + drRot[0];
		(*gede)->zfw[i] = ((*gede)->zfw[i] + drRot[2]) * ((*acga)->fz)+
		  ((*acga)->z) * (1.-(*acga)->fz) ;
		(*gede)->y1w[i] = (*gede)->y1w[i] + drRot[1];
		(*gede)->y2w[i] = (*gede)->y2w[i] + drRot[1];
	      }
	
	    }       
	    
	  }

	}

      }
      default: break ;
      

      } 

    }

  }

}

//****************************************************************
// function
//****************************************************************
unsigned int GEDEcmpToComponent(int cmp) {

 if (cmp == 1) return COMP_M_SVD;
 if (cmp == 2) return COMP_M_OTR;
 if (cmp == 3) return COMP_M_ITR;
 if (cmp == 4) return COMP_M_HIPT;
 if (cmp == 5) return COMP_M_RICH;
 if (cmp == 6) return COMP_M_MUON;
 if (cmp == 7) return COMP_M_ECAL;
 if (cmp == 8) return COMP_M_MUON;
 if (cmp == 9) return COMP_M_MUON;

 return 0;
}



int OTcmp ( ArtePointer<GEDE> gede ) {
  // OTcmp = 0x0 + PC/MC
  //         0x1 - PC/MC
  //         0x2 + TC
  //         0x3 - TC

  int rc=0 ;
  if (gede->hwc[0]=='-') rc|=0x1 ;
  if (gede->hwc[1]=='t') rc|=0x2 ;
  
  return rc ;
}



int ITcmp ( ArtePointer<GEDE> gede ) {
  // ITcmp = 0x0 +x+y PC/MC
  //         0x1 -x+y PC/MC
  //         0x2 +x-y PC/MC
  //         0x3 -x-y PC/MC
  //         0x4 +x+y TC
  //         0x5 -x+y TC
  //         0x6 +x-y TC
  //         0x7 -x-y TC

  int rc=0 ;
  if ((gede->xmin+gede->xmax)<0.) rc|=0x1 ;
  if ((gede->ymin+gede->ymax)<0.) rc|=0x2 ;  

  // chambers in TC area
  //  if ( (strstr ( gede->hwc[1], "14" ) != NULL) ||   
  //       (strstr ( gede->hwc[1], "15" ) != NULL) )   rc|=0x4 ; 
  
  return rc ;
}



