
// ********************************************************************
//
// Routine:      acReadOTR
// Author:       T.Oest
//
// Description:  read OTR alignment constants
//                
//
// Changes:
// 13 Jun 2000 (AG) add prototype int OtrDbWasUpdated( char * )
// 13 Nov 2000 (UU) change default Otr hit preparation to OtrI
// 15 Nov 2000 (AG) introduced ARTE_mark()
// 07 Dec 2000 (SK) introduced OtrIDbError()
// 14 Dec 2000 (DG) call OtrDbUpdate(tahd.dbmajor) in TAXX mode
// 15 Jan 2001 (DG) use actaxx_Command for more safety
// 26 Jan 2001 (SK) changes for compatibility with SLT
//
// ********************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>
#include <limits.h>

// ONLINE includes

// ARTE includes
#include "Otr/OtrDb.h"
#include "arte/arte.hh"
#include "database/table.h"
#include "farm/farm.hh"
#include "Otr/OtrI.h"
#include "Otr/OtrEvent.h"

#define ARTE_KEY 1

// prototypes
void TAReadFile(char* filename="/afs/desy.de/user/k/kruecker/public/align/TAAlign.txt", int prtflg=0);

int OtrDbWasUpdated( char *text );

// globals
static int hitprepflag = -1;


//==================================================================
// get file name, and read constants
//==================================================================

int oldalign() {

 char * OTRdb;

 if (getenv("DB_OTR_ALIGNMENT") != NULL){

   OTRdb = getenv("DB_OTR_ALIGNMENT");

   TAReadFile(OTRdb);
   return 1;
   }
 return -1;
 }

//*****************************************************************
// function
//*****************************************************************

int acReadOTR(int KTstatus) {
  ARTE_mark( "acReadOTR 0", 0 );
  if (hitprepflag==-1) {
    char *f;
    
/*                    hit preparation for OTR  */
    if ((f=getenv("OtrHitPreparation"))) hitprepflag = atoi(f);
    else hitprepflag=0;

    if ((f=getenv("OtrDqSwitch"))) OtrDqSwitch(atoi(f));
    else OtrDqSwitch(OTRDQSelectOccupOr);

    
    if ((f=getenv("OtrDebugSwitch"))) OtrIDebugSwitch(atoi(f));
    else OtrIDebugSwitch(OTRIDebugDefault);//Offline); 

/*  enables all massages with the exception of debug  massages */       
    OtrIDebugSwitch(OTRIDebugToAll|(OTRIAll&~OTRIDebug));
//    OtrIDebugSwitch(OTRIDebugToAll|OTRIAll);

/*  enables to work with both database and files    */       
    OtrISwitch(OTRIUseDb|OTRIUseEnvironment); 

    OtrIInitialize();
    if ((f=getenv("OtrEventSwitch"))) OtrEventSwitch(atoi(f));
      
    if (hitprepflag!=0) {
      if (hitprepflag==2) {
        OtrEventInfoEventGet(ARTE_get_event());
        OtrDbUpdate(OtrEventInfoDbKey());
        }
      ARTE_mark( "acReadOTR 1", 0 );
      return oldalign();
      }
    }

  if (hitprepflag==0) {
    OtrDbUpdate(ARTE_KEY);
    ARTE_mark( "acReadOTR 1", 0 );
    if(OtrIDbError()){
      ARTE_exit( GRR_SFATAL, "acReadOTR: Reading of DB failed!", 0 );
      }
    return OtrDbWasUpdated("OTR_Align");
    }
  else {
    ARTE_mark( "acReadOTR 1", 0 );
    return 0;
  }
} 
