// ********************************************************************
//
//  Routine:	alca/actaxxPrep.C
//  Author:		Dmitry Golubkov
//
//  Description:	Restores TAHB --> HITB,
//				called from a user routine
//
//  Modifications:
//   -- none yet					06/12/00 DG
//   -- added ARTE_mark					14/12/00 DG
//   -- restore ITR hits				19/12/00 TZ
//   -- fix precision loss of ITR resolution		18/01/01 DG
//
// ********************************************************************

#include <iostream.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <fstream.h>
#include <iomanip.h>
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "rang/zfextse.hh"
#include "rang/zfparext.hh"
#include "rang/Fit.hh"
#include "Otr/OtrI.h"

#include "Otr/OtrEvent.h"


// ONLINE includes
#include "farm/farm.hh"
#include "table.h"
#include "esp.h"

#include <sys/time.h>

#include "alca/actaxxComm.hh"
#include "alca/actaxxFLG.hh"





// Make hit preparation input for OtrEventUpdate() from TAHB bank

int actaxxPrep( int jTaTr =-1 )
{
	ARTE_mark( "actaxxPrep 0", GRR_OK );

	taTATRSeedTATRGetIndex(1);

	// We_should_avoid_possibility_of_counting_the_same_hit_twice !
	// Check if any hit preparation has been done already

	for (	ArteTableIterator<HITB> hitb = ArteTable<HITB>::begin();
		hitb != ArteTable<HITB>::end(); ++hitb )
	{
		if ( (hitb->gede->cmp != 2) && (hitb->gede->cmp != 3) ) continue;

		cout << "actaxxPrep: *** Error, found an OTR/ITR hit "
			<< "(#"<< hitb.index()<< "),\n"
			<< "\t\tTAHB --> HITB preparation skipped.\n";

		taTATRSeedTATRGetNumberOf(0);

		ARTE_mark( "actaxxPrep 1", GRR_OK );
		return 0;
	}


	// Remove everything that can remind about previous OTR hit preparation
	for ( ArteTableIterator<GEDE> gede = ArteTable<GEDE>::begin();
		gede != ArteTable<GEDE>::end(); ++gede )
	{
		if ( (gede->cmp != 2) && (gede->cmp != 3) ) continue;

		gede->hit1 = 0;
		gede->hitl =-1;
	}

	// Clear tables before next event (usually done in rc99AStandAlone)
	ArteTable<RREL>::clear();
	ArteTable<RPNT>::clear();
	ArteTable<RVER>::clear();
	ArteTable<RHIT>::clear();
	ArteTable<RSEG>::clear();
	ArteTable<RTRA>::clear();
	ArteTable<RCCL>::clear();


	taTATRSeedTATRGetNumberOf(ArteTable<TATR>::size());

	if ( !ArteTable<TAHB>::size() )
	{
		ARTE_mark( "actaxxPrep 1", GRR_OK );
		return 0;
	}


	if ( jTaTr == 0 )
		taTATRSeedTATRGetNumberOf(0); // No seeding
	else
		if ( jTaTr > 0 )
		{
			taTATRSeedTATRGetNumberOf(1);
			taTATRSeedTATRGetIndex(jTaTr);
		}




	int nHits = 0;
	HITB hitb;

	for (	ArteTableIterator<TAHB> tahb = ArteTable<TAHB>::begin();
		tahb != ArteTable<TAHB>::end(); ++tahb )
	{
		// FITONLY flagging if hit was used to fit the track
		if (	(actaxx_Command.flg & TACMD::fitonly)
			&& !(tahb->flg&TAFLG::fit)
		   ) continue;

		if ( jTaTr >= 0 )
		{
			// For TA event split into single tracks
			if ( (int)(tahb->flg&TAFLG::tatr) != jTaTr ) continue;
		}
		else
		{
			// Do not count the same hit twice
			if ( tahb.index() > 1 )
			{
				ArtePointer<TAHB> tahbOld(tahb.index()-1);

				if ( (tahbOld->flg & tahb->flg) & (TAFLG::itr|TAFLG::otr) )
				{
					if	( tahb->flg & TAFLG::otr )
					{
						if ( tahb->channel == tahbOld->channel ) continue;
					}
					else if	( tahb->flg & TAFLG::itr )
					{
						if ( tahb->dt == tahbOld->dt ) continue;
					}
				}
			}
		}

		// Restore HITB
		if ( tahb->flg&TAFLG::itr )
		{
		  // Inner tracker, Torsten Zeuner 18/12/2000
                  
                  if ( tahb->gede->cmp!=3 ) {
                    cout << "ERROR : NOT AN ITR HIT " << endl;
                    continue;
                  }

                  ArtePointer<GEDE> gede, gede1;
                   
                   if ( tahb->gede->sec == 2 || tahb->gede->sec == 4 ) {
                     gede1 = tahb->gede;
                     gede  = ArtePointer<GEDE>(gede1.index()+1);
                   }
                   if ( tahb->gede->sec == 5 || tahb->gede->sec == 7 ) {
                     gede = tahb->gede;
                     gede1  = ArtePointer<GEDE>(gede.index()+1);
                   }
                   if ( tahb->gede->sec == 1 || tahb->gede->sec == 3 ) {
                     gede = tahb->gede;
                     gede1 = ArtePointer<GEDE>(gede.index()-1);
                   }
                   if ( tahb->gede->sec == 6 || tahb->gede->sec == 8 ) {
                     gede1 = tahb->gede;
                     gede = ArtePointer<GEDE>(gede1.index()-1);
                   }
                   
                   float relpos=0;
                   float gasgap = gede->zmax-gede->zmin;

                   hitb.z    = gede->zfw[0]+gasgap/2;

                   if ( gede->gesl->nsl>8 && gede->sec > 4 ) {
                     // -x side Typ III 
                     relpos = gede->nwi[0]+gede1->nwi[0]+4-tahb->dt;
                     hitb.x    = relpos*gede->pit + gede->xfw[0];
                   }

                   if ( gede->gesl->nsl>8 && gede->sec < 5 ) {
                     // +x side Typ III
                     relpos = tahb->dt-5;
                     hitb.x    = relpos*gede->pit + gede->xfw[0];
                   }

                   if ( gede->gesl->nsl<9 && gede->sec > 4 ) {
                     // -x side Typ I+II
                     relpos = gede->nwi[0]+gede1->nwi[0]+7-tahb->dt;
                     hitb.x    = relpos*gede->pit + gede->xfw[0];
                   }

                   if ( gede->gesl->nsl<9 && gede->sec < 5 ) {
                     // +x side Typ I+II
                     relpos = tahb->dt-8;
                     hitb.x    = relpos*gede->pit + gede->xfw[0];
                   }

                   hitb.r = float(tahb->channel)/1000000.;

                   hitb.gede = tahb->gede;


		}
		else if ( tahb->flg & TAFLG::otr )
		{
			// Outer tracker
			unsigned char d = (unsigned char) (int) tahb->dt;

			int ci		= OtrIFromId(tahb->channel);
			if ( ci == OTRIInvalid )
			{
				printf("actaxxPrep: *** problems with tahb->channel\n");
				continue;
			}

			// Note, OtrEventUpdate will think, Wax_Was_Used (as in MC)
			// In Wax hitb->chid = wireId + 128*(chId + 16*asdId),
			// where wireId = wire+32*mono

			int mono	= OtrIMonoLayer(ci);

			if ( taCommOTR.flg & TACommOTR::OtrI_st )
			{
				int wire	= OtrIWire(ci);
				int ch		= OtrIAsdChannel(ci);
				int asd		= OtrIAsdIndex(ci);
				int waxid	= wire + (mono<<5) + ((ch + (asd<<4))<<7);
				hitb.chid	= waxid;
			}
			else
				hitb.chid	= tahb->channel;



			hitb.y		= (int) d;

			if ( taCommOTR.flg & TACommOTR::OtrI_gd )
				hitb.gede	= ArtePointer<GEDE>
						  (
							OtrGeoGet( OtrIGeoIndex(ci) ).index()
						  );
			else
			{
				if ( tahb->gede.index() == 0 )
				{
					cout<<"acatxxPrep: *** OTR gede = 0 found,"
						<<" try /AC/TAXX/OTR/OPT gede=n";

					ARTE_mark( "actaxxPrep 1", GRR_FISHY );
					return -1;
				}

				hitb.gede	= tahb->gede;
			}


			hitb.x		= hitb.gede->xfw[mono]
						+ OtrIWire(ci)*OtrIWirePitch(ci);
			hitb.z		= hitb.gede->zfw[mono];

			float t		= d - OtrITdcT0(ci);
			hitb.q		= OtrRtRadius(ci,t);
			hitb.r		= OtrRtRadiusSigma(ci,t);
		}

		ArteTable<HITB>::push_back(hitb);




		int hitbInd		= ArteTable<HITB>::size();
		ArtePointer<GEDE> gede	= hitb.gede;

		// TAHBs are sorted already at write time
		// struct GEDE { ... ArteTableIterator<HITB> hit1; ... }

		if ( !(*gede->hit1).index() )
		{
			gede->hit1 = hitbInd;
			gede->hitl = hitbInd;
		}
		else
			gede->hitl = hitbInd;




		nHits++;
	}

	if ( !nHits ) taTATRSeedTATRGetNumberOf(0);

	ARTE_mark( "actaxxPrep 1", GRR_OK );
	return nHits;
}





// Store/(by default return) index of first TATR for ranger seed
int taTATRSeedTATRGetNumberOf(int a)
{
	static int b;
	if ( a !=-1 ) b = a;

	return b;
}





// Store/(by default return) number of TATRs for ranger to seed with
int taTATRSeedTATRGetIndex(int a) 
{
	static int b;
	if ( a !=-1 ) b = a;

	return b;
}
