// ********************************************************************
//
//  Routine:	alca/actaxxevnt.C
//  Author:		Dmitry Golubkov
//
//  Description:	Calls detector-specific TAXX selection functions,
//				called from mmwevt.F
//
//  Modifications:
//   -- none yet					06/12/00 DG
//   -- added ARTE_mark					14/12/00 DG
//   -- introduced actaxxOut				18/12/00 DG
//   -- add TAXX reselection & selective extraction	13/02/00 DG
//
// ********************************************************************

#include "arte/arte.hh"
#include <sys/time.h>

#include "arte/ArteUtil.h"
#include "farm/farm.hh"





#ifdef THE_TATEST_MODE

#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"

extern "C" {
	void hbook1_(const int &,const char *,const int &,
			const float &, const float &, const float &, const int);
	void hf1_(const int &, const float &, const float &);
}
void taCheckUp(int);

#endif





#include "alca/actaxxComm.hh"
#include "alca/actaxxFLG.hh"




extern "C" void actaxxevnt_()
{
	ARTE_mark( "actaxxevnt_ 0", GRR_OK );

	if ( actaxx_Command.cmd == TACMD::extract )
	{
		// Selective extraction (mainly for ITR) to skip OTR-only events
		int found = 0;
		/* Somehow I managed to produce memory corruption here if not checking ArteTable<TAHB>::size() (DG) */
		if ( ArteTable<TAHB>::size() )
			for ( ArteTableIterator<TAHB> tahb = ArteTable<TAHB>::begin(); tahb != ArteTable<TAHB>::end(); ++tahb )
				if (
					( (tahb->flg & TAFLG::itrsel) && (actaxx_Command.flg & TACMD::ta_itr) )
					||( (tahb->flg & TAFLG::otrsel) && (actaxx_Command.flg & TACMD::ta_otr) )
				   )
				{
					found = 1;
					break;
				}

		if ( found ) actaxxOut();

		ARTE_mark( "actaxxevnt_ 1", 0 );
		return;
	}




	if ( actaxx_Command.cmd != TACMD::fill )
	{
		ARTE_mark( "actaxxevnt_ 1", 0 );
		return;
	}


	if ( actaxx_Command.flg & TACMD::ta_itr )
	{
		ARTE_mark( "actaxxSelectITR 0", GRR_OK );
		actaxxSelectITR();		// called only from here
		ARTE_mark( "actaxxSelectITR 1", GRR_OK );
	}

	if ( actaxx_Command.flg & TACMD::ta_otr )
	{
		ARTE_mark( "actaxxSelectOTR 0", GRR_OK );
		actaxxSelectOTR();		// called only from here
		ARTE_mark( "actaxxSelectOTR 1", GRR_OK );
	}

	// Clear old TAXX in case of new selection (TACMD::fill bit checked)
	ArteTable<TAHD>::clear();
	ArteTable<TAHB>::clear();
	ArteTable<TATR>::clear();

	int tmp = actaxxFill();		// Final filling of TAXX tables

	if ( tmp ==-1 )
	{
		ARTE_mark( "actaxxevnt_ 1", GRR_FISHY );
		return;					// Big problems
	}

	if ( tmp != (int)ArteTable<TAHB>::size() )	// Something too odd
	{
		cout<<"actaxxevnt_: *** Finally we have "
			<<ArteTable<TAHB>::size()<<" TAHBs, "
			<<"while taFill() created "<<tmp<<"\n";

		ARTE_mark( "actaxxevnt_ 1", GRR_FISHY );
		return;
	}


	actaxxOut();				// Write out TAXX event


#ifdef THE_TATEST_MODE
	taCheckUp(-2);
	taCheckUp( 0);
#endif


	ARTE_mark( "actaxxevnt_ 1", GRR_OK );	// If no problems occured

	return;
}
