#ifndef __TACOMM__
#define __TACOMM__

// ********************************************************************
//
//  Header:	alca/actaxxComm.hh
//  Author:		Dirk Kruecker
//
//  Description:	TAXX command and selection - related stuff
//
//  Modifications:
//   -- creation					08/11/00 DK
//   -- remake						06/12/00 DG
//   -- add "extract" comand and actaxxOut()		19/12/00 DG
//
// ********************************************************************

#include "arte/arte.hh"

struct TaHitStruct
{
	int hb; 		/* the HITB to remember */
	int rs;			/* associated RSEG index */
	unsigned int who;	/* who made selection etc. */
};


// Parameters for detector-specific TAXX selection

#include "alca/actaxxCommITR.hh"
#include "alca/actaxxCommOTR.hh"


// General commands for TAXX
extern struct TACMD
{
	// Some general switches definitions
	enum {
		fill	=	0x00, /* fill TAXX */
		event	=	0x01, /* restore the whole TAXX event */
		track	=	0x02, /* restore TAXX track */
		extract	=	0x03, /* extract TAXX from dst */

		off	=	0x80, /* TAXX is off */


		fitonly	=      0x100, /* restore only hits with TAFLG::fit */

		bit9    =      0x200, /* not assigned yet */
		bit10   =      0x400, /* not assigned yet */
		bit11   =      0x800, /* not assigned yet */
		bit12   =     0x1000, /* not assigned yet */
		bit13   =     0x2000, /* not assigned yet */
		bit14   =     0x4000, /* not assigned yet */
		bit15   =     0x8000, /* not assigned yet */


		taout	=    0x10000, /* use TAOUT also */
		rhp	=    0x20000, /* send TAXX via rhp */

		bit18   =    0x40000, /* not assigned yet */
		bit19   =    0x80000, /* not assigned yet */
		bit20	=   0x100000, /* not assigned yet */
		bit21   =   0x200000, /* not assigned yet */
		bit22   =   0x400000, /* not assigned yet */
		bit23   =   0x800000, /* not assigned yet */


		ta_otr	=  0x1000000, /* Outer Tracker selectin */
		ta_itr	=  0x2000000, /* Inner Tracker selection */

		bit26   =  0x4000000, /* not used yet */
		bit27   =  0x8000000, /* not used yet */
		bit28   = 0x10000000, /* not used yet */
		bit29   = 0x20000000, /* not used yet */
		bit30   = 0x40000000, /* not used yet */
		bit31	= 0x80000000  /* not used yet */
	};

	// Command number according to TACMD::taxx
	char cmd;

	// Some switches according to TACMD
	unsigned int flg;

} actaxx_Command;



// TAXX functions

extern "C" void actaxxevnt_();	// calls selections and TAXX filling
void actaxxSelectITR();		// ITR-specific hit selection
void actaxxSelectOTR();		// OTR-specific hit selection


int actaxxFill();		// TAHB filling (called after selection)

int actaxxPrep(int =-1);	// Hit preparation

int actaxxAdd(	ArtePointer<HITB> ,
		ArtePointer<RSEG> = ArtePointer<RSEG>(0),
		unsigned int = 0
	     );			// Add a hit to the list of selected ones

int actaxxOut();		// Write TAXX out to taout or rhp


// Store/(by default return) relevant TATR ranger seeding parameters
int taTATRSeedTATRGetIndex(int =-1 );		// index of first TATR to seed
int taTATRSeedTATRGetNumberOf(int =-1);		// number of TATRs to seed with

#endif
