#include <arte/arte.hh>

bool anl_IsMuonCandidate(ArtePointer<RTRA> rtra, int level)
{
  const int   minNVdsHits       = 6 ;
  const int   minNPatHits       = 15 ;
  const float maxEcalEnergy     = 5 ;
  const float maxMomentum       = 200 ;
  const float minRichLikelyHood = 0.05 ;
  const float minMuonLikelyHood = 0.1 ;
  
  bool rc = true ;
  
  // first some track selection criteria:

  // check number of vds hits
  rc = rc && rtra->hsi >= minNVdsHits ;
  
  // check number of main tracker hits
  rc = rc && (rtra->hou+rtra->hin) >= minNPatHits ;
  
  // check momentum
  rc = rc && fabs(1/rtra->pf) < maxMomentum ;

  // now we get the real muon identification stuff. it would be better
  // to combine these into one likelyhood, but that is maybe long term
  // strategy.

  // check ecal energy (of cluseter with highest energy)
  for(ArteRelationIterator<RCCL> rcclIt = rtra->rccl.begin() ;
      rcclIt != rtra->rccl.end() ; rcclIt++)
    rc = rc && (*rcclIt)->esum < maxEcalEnergy ;

  // check RICH likelyhood
  if(rtra->flr > 0) {
    float lrmu = rtra->lrmu/(rtra->lrmu+rtra->lrk+rtra->lrp+rtra->lrpi) ;
    rc = rc && lrmu > minRichLikelyHood ;
  }

  // check MUON likelyhood
  if(rtra->flm > 10) rc = rc && rtra->lmmu > minMuonLikelyHood ;


  // return
  return rc ;
}




