
// *****************************************************************************
//
// source:    anl_TrSelect.C
//
// created by: Michael Smelling.
// Modified by: P.Conde
//
// purpose:   select a vector of RTRA's.
//
// *****************************************************************************
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <map.h>

// ARTE include
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "CLHEP/Hist/HBookTuple.h"
#include "CLHEP/Vector/ThreeVector.h"
#include "marple/marple.hh"
#include "arte/ArteUtil.h"
#include "arte/RSEG_cmp.hh"
int  
anl_TrSelect( vector< ArtePointer<RTRA> > &rtra )
{ 
 //-------------------------------------------------------
  // return a vector with ArtePointers into RTRA
  //-------------------------------------------------------
   
  static int MINHITS_VDS  = 6;   // required minimum number of VDS hits
  static int MINHITS_MAIN = 20;  // required minimum number of Maintracker hits

  map< float, ArtePointer<RTRA> > mrtra;
  //------------------------------------
  
  // put all VDS-OTR tracks into a sorted map

  for(ArteTable<RTRA>::iterator itrtra = ArteTable<RTRA>::begin();
      itrtra != ArteTable<RTRA>::end(); ++itrtra) {
    if( (*itrtra)->fit !=  0 ) continue;
    int nvds  = (*itrtra)->hsi;
    int nmain = (*itrtra)->hou + (*itrtra)->hin;
    if( nvds  < MINHITS_VDS ) continue;
    if( nmain < MINHITS_MAIN) continue;
    float fom = float(nvds+nmain) + 1./(1.+(*itrtra)->chi);
    mrtra.insert(pair< float, ArtePointer<RTRA> >(-fom,*itrtra));
  }

  // copy the tracks into the return vector, omitting all clones

  for(map< float, ArtePointer<RTRA> >::iterator it=mrtra.begin();
      it != mrtra.end(); ++it) {
    int OK = 1;
    for(unsigned int i=0; i!=rtra.size(); ++i) {
      if(rtra[i]->csvd  == it->second->csvd ) OK=0;
      if(rtra[i]->cmain == it->second->cmain) OK=0;
      if(!OK) break;
    }
    if(OK)  rtra.push_back(it->second);
  }
 
  // Now that we selected the VDS-OTR, let's select the VDS-ECAL-RICH:
  //------------------------------------------------------------------
 
 
  map< float, ArtePointer<RTRA> > mapVdsEcalRich;
  for(ArteTable<RTRA>::iterator itrtra = ArteTable<RTRA>::begin();
      itrtra != ArteTable<RTRA>::end(); ++itrtra) {
    if( (*itrtra)->fit !=  0 ) continue;
    if( (*itrtra)->hin + (*itrtra)->hou > 0 ) continue;
    if( (*itrtra)->csvd <= 0 ) continue;
    if( (*itrtra)->cecal <= 0 ) continue;
    if( (*itrtra)->crich <= 0 ) continue;

    // Take the clusters:

    int goodCluster = 0;
    float Xcl=0, Ycl=0, Zcl=0;
    for(ArteRelationIterator<RCCL> rccl = (*itrtra)->rccl.begin();
	rccl != (*itrtra)->rccl.end(); rccl++){

      // See if this ecal cluster was used in one of the previous tracks:

      int index = (*rccl).index();
      int used = 0;
      for(unsigned int i=0; i!=rtra.size(); ++i) {
	if( rtra[i]->cecal == index) used++;
      }
      if(used == 0 ) goodCluster++;  
      Xcl = (*rccl)->x;
      Ycl = (*rccl)->y;
      Zcl = (*rccl)->z;
    }
    if(goodCluster == 0 ) continue;

    //Create another sorted map, using the distance between the cluster and the track:

    RcTraPoint TrackF((*itrtra));
    TrackF.prolongate(Zcl);
	
    float Dist = pow(pow(TrackF.x()-Xcl,2)+pow(TrackF.y()-Ycl,2),0.5);
      
    mapVdsEcalRich.insert(pair< float, ArtePointer<RTRA> >(Dist,*itrtra));

  } // End of loop in VDS-ECAL tracks.

 // copy the tracks into the return vector, omitting all clones

  for(map< float, ArtePointer<RTRA> >::iterator it=mapVdsEcalRich.begin();
      it != mapVdsEcalRich.end(); ++it) {
    int OK = 1;
    for(unsigned int i=0; i!=rtra.size(); ++i) {
      if(rtra[i]->csvd  == it->second->csvd ) OK=0;
      if(rtra[i]->cecal == it->second->cecal) OK=0;
      if(!OK) break;
    }
    if(OK)  rtra.push_back(it->second);
  }

  // Now take VDS-ECAL:
  //------------------------------------------------------------------
  
  map< float, ArtePointer<RTRA> > mapVdsEcal;
  for(ArteTable<RTRA>::iterator itrtra = ArteTable<RTRA>::begin();
      itrtra != ArteTable<RTRA>::end(); ++itrtra) {
    if( (*itrtra)->fit !=  0 ) continue;
    if( (*itrtra)->hin + (*itrtra)->hou > 0 ) continue;
    if( (*itrtra)->csvd <= 0 ) continue;
    if( (*itrtra)->cecal <= 0 ) continue;
    if( (*itrtra)->crich > 0 ) continue;

    // Get Clusters:

    int goodCluster = 0;
    float Xcl=0, Ycl=0, Zcl=0;
    for(ArteRelationIterator<RCCL> rccl = (*itrtra)->rccl.begin();
	rccl != (*itrtra)->rccl.end(); rccl++){

      // See if this ecal cluster was used in one of the previous tracks:

      int index = (*rccl).index();
      int used = 0;
      for(unsigned int i=0; i!=rtra.size(); ++i) {
	if( rtra[i]->cecal == index) used++;
      }
      if(used == 0 ) goodCluster++;  
      Xcl = (*rccl)->x;
      Ycl = (*rccl)->y;
      Zcl = (*rccl)->z;
    }
    if(goodCluster == 0 ) continue;

   // Create another sorted map, using the distance between the cluster and the track:

    RcTraPoint TrackF((*itrtra));
    TrackF.prolongate(Zcl);
	
    float Dist = pow(pow(TrackF.x()-Xcl,2)+pow(TrackF.y()-Ycl,2),0.5);
      
    mapVdsEcal.insert(pair< float, ArtePointer<RTRA> >(Dist,*itrtra));

  } // End of loop in VDS-ECAL tracks.

 // copy the tracks into the return vector, omitting all clones

  for(map< float, ArtePointer<RTRA> >::iterator it=mapVdsEcal.begin();
      it != mapVdsEcal.end(); ++it) {
    int OK = 1;
    for(unsigned int i=0; i!=rtra.size(); ++i) {
      if(rtra[i]->csvd  == it->second->csvd ) OK=0;
      if(rtra[i]->cecal == it->second->cecal) OK=0;
      if(!OK) break;
    }
    if(OK)  rtra.push_back(it->second);
  }

  return rtra.size();
}                              
