*-- Author : T. Lohse   

      SUBROUTINE ALDY (IFLAG)

************************************************************************
*
* Description: Selection of PYTHIA Drell-Yan muons
*              as preparation of generation of underlying event (FRITIOF)
* 
*-- Authors :  T. Lohse    (ALONIA)
*              W. Gradl    (Adapted for Drell-Yan)
*
* Modified by W. Gradl (29-nov-00):  
* - remove all generated particles for events with no light lepton pair
* - flag successful generation of DY-lepton pair to caller via IFLAG
*   
* Called from: ALPYTH
*
* External references: LUND, PYTHIA
*
* Arguments: IFLAG 
*   on exit, IFLAG =
*            1  if e+e- or mu+mu- pair generated
*            0  no such pair
* 
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "jets/LUDAT1.fi"
#include "jets/LUDAT3.fi"
#include "jets/LUJETS.fi"
#include "pyth/PYSUBS.fi"
#include "pyth/PYPARS.fi"
#include "arte/luopts.fi"

      real evwei
      common / weight / evwei

      INTEGER IFLAG
      integer i, j, j1, j2, im, imu1, imu2, mtyp 
      integer mess_id_1, mess_id_2
      save mess_id_1, mess_id_2

      real xF, pT
      real*8 beta, gamma, sqrts, sum1, sum2, x, term
      real*8 facups, facpsi
      real*8 a1,a2,a3,a4,a5,a6,b1,b2,b3,pthr
      real*8 p1,p2,p3,p4,q1
      real*8 e789pt, e789xf
      save a1,a2,a3,a4,a5,a6,b1,b2,b3,pthr
      save p1,p2,p3,p4,q1
      save e789pt,e789xf
      save facups, facpsi

#include "arte/Arte.ff"

      IFLAG = 1

C   find outgoing particles/partons from 2->1->2 scattering process
      imu1 = MSTI(7)
      imu2 = MSTI(8)
      
C   keep only events with e+e- or mu+mu- from hard subprocess
      if (imu1.ne.0.and.imu2.ne.0) then
         if ( (k(imu1,2).eq.13.and.k(imu2,2).eq.-13) .or.
     $        (k(imu2,2).eq.13.and.k(imu1,2).eq.-13) .or.
     $        (k(imu1,2).eq.11.and.k(imu2,2).eq.-11) .or.
     $        (k(imu2,2).eq.11.and.k(imu1,2).eq.-11)) then
            continue            ! DO nothing
         else                   ! no light leptons generated
            imu1 = -1
            imu2 = -1
            IFLAG = 0           ! signal this to caller
         endif
      endif

C..      call MEERRO(mess_id_1,'ALDY: No ONIUM state found')
C      goto 900


 10   continue

c discard all particles but the drell-yan leptons
      do  j = 1, N
        if (K(j,1).ne.0.and.K(j,1).lt.21) then
          if (j.ne.imu1.and.j.ne.imu2) then
            im = j
 2          continue
            im = K(im,3)
            if (im.eq.imu1.or.im.eq.imu2) goto 3
            if (im.ne.0)                  goto 2
            K(j,1)=0
 3          continue
          endif
        endif
      enddo

c     remove lines with status flag =0. Update event history information.
      call luedit(12)      

c     remove comment lines. Update event history information.
      call luedit(15)      

 900  END

************************************************************************
* End of ALDY
************************************************************************


