//                              -*- Mode: C++ -*-
//: EDBoundCellModifier.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 2/5/97
// Last Update ........
//
#undef HIST

#ifdef HIST
#  include "ana/Ntuple.hh"
#endif
#include "care/EMClusterTree.hh"
#include "care/EDBoundCellModifier.hh"
#include "care/ECBuilder.hh"
#include "care/ELNearHitCellFrac.hh"
#include "care/ELNearClusterFrac.hh"
#include "care/EMHitCell.hh"
#include "arte/ArteMessage.hh"

/* 
CLASS
  EDBoundCellModifier

*/


//////////////// EDBoundCellModifier Static data Definitions ////////////////



/////////  EDBoundCellModifier Static member function Definitions /////////////
//--


//////////////////// EDBoundCellModifier Class Definitions ////////////////

EDBoundCellModifier::EDBoundCellModifier(ECBuilder* pbuilder) : 
  builder(pbuilder), cells_to_split(0) {
  //--
}

// No functions
EDBoundCellModifier::~EDBoundCellModifier(){
  // remember to delete the Clus-To-Clus strategy

}


//--
//  Base function to be called with the TopCluster List
//
void EDBoundCellModifier::analyzeTopList(EMClusterTree & topList){
  doModify(&topList);
}



//--
// Cluster Modifier
void EDBoundCellModifier::doModify(EMCluster * clus){
  //
  //
  if( clus->getHypotheses()>1 ){
    for(EMClusterMember* child = clus->rbegin(); 
	child!=clus->rend();  child = child->previous()){
      child->modifyMe(*this);
    }
  }
  //
}


//--
// EMHitCell Analyzer
void EDBoundCellModifier::doModify(EMHitCell * hitT){
  cells_to_split.push_back(hitT);
}

//--
// EMClusterMember Analyzer
void EDBoundCellModifier::doModify(EMClusterMember *){
}

//--
// TrackOnEcal Analyzer
void EDBoundCellModifier::doModify(EMTrackOnEcal * ){
}

//--
// ClusterTree Analyzer
void EDBoundCellModifier::doModify(EMClusterTree * tree){
  //
  for(EMClusterMember* child = tree->begin(); child!=tree->end(); 
      child = child->next()){
    if( !child->isALeaf()){
      child->modifyMe(*this);
    }
  }
  // now in the cell_to split array there is the list of
  // all the cells to split in some way.
  ELNearHitCellFrac nearCells;
  ELNearClusterFrac nearClus;
  EMCluster* cluster = 0;

  for( std::vector<EMHitCell *>::iterator itercell = cells_to_split.begin();
       itercell !=  cells_to_split.end(); itercell++){
    // find the neighboring cells and their fractions
    //std::cout<<"Splitting cell: ";
    //(*itercell)->dump();
    nearCells.of(*itercell,*tree);
    // find the neighboring clusters and their fractions
    nearClus.of(nearCells);
    //std::cout<<"on "<<nearClus.howMany()<<" clusters "<<std::endl;
    nearClus.minimize();
    //std::cout<<"Remaining "<<nearClus.howMany()<<" clusters "<<std::endl;
    for(nearClus.begin(); nearClus.more(); nearClus.next()){
      // now get the value 
      if(nearClus.value() > 0.0){
	// fill a fraction of cell and store it in the tree
	cluster = nearClus.operator*();
	builder->cellFraction(*itercell, cluster, nearClus.value());
      } 
    }
  }
}
