//                              -*- Mode: C++ -*-
//: EDClusHypotModifier.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 2/5/97
// Last Update ........
//

#ifndef EDCLUSHYPOTMODIFIER_HH_
#define EDCLUSHYPOTMODIFIER_HH_

#include "care/EDClusterModifier.hh"
class ESCTCDisStrat;

/**
class EDClusHypotModifier.
  Class for finding the number of hypothetical particles gave origin to
  a cluster.

  The algorithm choseen make use of a constant CutOff level to define an
  hypotetical cluster.

*/

class EDClusHypotModifier : public EDClusterModifier {
public:
  //-- Constructors
  /// Default constructor
  EDClusHypotModifier();
  /// copy constructor
  EDClusHypotModifier(const EDClusHypotModifier & clhyp);
  /// assignment operator
  const EDClusHypotModifier & operator = (const EDClusHypotModifier & clhyp);
  /// The virtual default destructor
  virtual ~EDClusHypotModifier();
  //-- 
  // ClusterMember Analyzer
  //
  /**@name Beheavioral methods */
  //@{ 
  /// EMClusterMember Modifier
  virtual void doModify(EMClusterMember * clsMem);
  //--
  /// EMHitCell Modifier
  virtual void doModify(EMHitCell * hitT) ;
  //--
  /// EMHitFraction Modifier
  inline
  virtual void doModify(EMCellFraction * hitT) ;
  //--
  /// EMCluster Modifier
  virtual void doModify(EMCluster * clus);
  //--
  /// EMClusterTree Modifier
  virtual void doModify(EMClusterTree * tree);
  //--
  /// TrackOnEcal Modifier
  virtual void doModify(EMTrackOnEcal * track);

  //--
  ///  Base function to be called with the TopCluster List
  void analyzeTopList(EMClusterTree & topList);
  //@}

private:
 
  //--
  /// Number of already found Hypotheses in this cluster
  Hyp_t numberOfHypotheses;
  //--
  ///  Flag to be set when a cell is above threshold
  bool newHypot;
  // A Clus-To-Clus distance meter
  ESCTCDisStrat * ctcstrat;
};
 
/// EMCellFraction Analyzer
void  EDClusHypotModifier::doModify(EMCellFraction * hitT){
  doModify((EMHitCell*) hitT);
};

#endif // EDCLUSHYPOTMODIFIER_HH_
