using namespace std;

#include <arte/arte.hh>
#include <arte/artefunc.h>
#include <map>
#include <vector>
#include <algorithm>
#include "cats/OtrCluster.hh"

extern vector<HIT > vhit;              // all hitbs are stored here
extern map<int, int> hit1map, hitlmap; // "pointers" to the first and the last hits of a gede

void OtrClusters(vector<CLUSTER>& vcluster, const ArtePointer<GEDE> gede){

  vcluster.clear();

  int gede_index = gede.index();

  int hit1 = hit1map[gede_index]; int hitl = hitlmap[gede_index];
  if (hitl < hit1) return;           // no hits on the gede

  // collect clusters

  cout.setf(ios::fixed);
  cout.setf(ios::showpoint);
  cout.precision(4);

  CLUSTER  cluster;
  cluster.used = 0;
  cluster.sina = gede->sin*gede->rot[4] - gede->cos*gede->rot[3];
  cluster.cosa = gede->cos*gede->rot[0] - gede->sin*gede->rot[1];
  cluster.gede = gede;
  cluster.lay  = gede->lay; 

  float pit = gede->pit;

  // create the first cluster
  pHIT lastphit = vhit.begin()+hit1;
  cluster.vphit.push_back(lastphit);

  vector<HIT>::iterator phit_begin = vhit.begin()+hit1; phit_begin++; // first is stored
  vector<HIT>::iterator phit_end   = vhit.begin()+hitl; phit_end++;   // next to the last

  if (phit_begin != phit_end){
    for (vector<HIT>::iterator phitIt = phit_begin; phitIt != phit_end; ++phitIt ){
      pHIT thisphit = phitIt;

      if (((thisphit->x - lastphit->x) < 0.6*pit) // 0.6add hitb to the cluster
	  && (distance(cluster.vphit.begin(), cluster.vphit.end()) < gede->nsl) 
	  ){ 

	cluster.vphit.push_back(thisphit);
	lastphit = thisphit;
      }
      else{ 
	float xest  = 0.0; float zest  = 0.0; int   nhits = 0; float whits = 0.0;
	float q_min = 1000.0;
	for (vector<pHIT>::iterator phIt = cluster.vphit.begin();
	    phIt != cluster.vphit.end(); phIt++){
	  float w = 1.0/(fabs((*phIt)->q)+(*phIt)->r);
	  w *= w; // !!!
	  xest += w*(*phIt)->x; zest += w*(*phIt)->z; 
	  nhits++; whits += w;
	  if (q_min > fabs((*phIt)->q)+(*phIt)->r) q_min = fabs((*phIt)->q)+(*phIt)->r;
	}
	cluster.x = xest/whits; cluster.z = zest/whits; cluster.length = nhits;
	if (nhits > 2) whits = (whits/nhits)*2;// simulate 2 hits maximum in a cluster
	cluster.weight = whits;
        //cluster.sigma = sqrt((pit*pit)/(4.0*9.0*nhits));
	cluster.sigma = /*1.0/whits;*/ 1.0/sqrt(whits);
	//cluster.sigma = q_min;

	vcluster.push_back(cluster);

	// create new cluster
	cluster.vphit.clear();
	lastphit = thisphit;
	cluster.vphit.push_back(lastphit);
      }
    }
  }  

  // store the last cluster

  float xest  = 0.0; float zest  = 0.0; int   nhits = 0; float whits = 0.0;
  float q_min = 1000.0;
  for(vector<pHIT>::iterator phitIt = cluster.vphit.begin();
    phitIt != cluster.vphit.end(); phitIt++){
    float w = 1.0/(fabs((*phitIt)->q)+(*phitIt)->r);
    w *= w;
    xest += w*(*phitIt)->x; zest += w*(*phitIt)->z; 
    nhits++; whits += w;
    if (q_min > fabs((*phitIt)->q)+(*phitIt)->r) q_min = fabs((*phitIt)->q)+(*phitIt)->r;
  }

  cluster.x = xest/whits; cluster.z = zest/whits; cluster.length = nhits;
  if (nhits > 2) whits = (whits/nhits)*2;// simulate 2 hits maximum in a cluster
  cluster.weight = whits;
  //cluster.sigma = sqrt((pit*pit)/(4.0*9.0*nhits));
  cluster.sigma = /*1.0/whits;*/ 1.0/sqrt(whits);
  //cluster.sigma = q_min;

  vcluster.push_back(cluster);
  
  return;
}


