// ******************************************************************
// 
// M. Braeuer / D. Samtleben
// Filling of histograms for VDS
//
// changes:
// 28 Oct 1999 (AG)  use defines from DQ.h
// 01 Nov 1999 (AG)  get rid of dq_hid.h
// 03 Nov 1999 (AG)  repair hist id
// 10 Dez 1999 (MBr) update/major revision
// 19 Apr 2000 (AG)  polish
// 16 May 2000 (GS)  vertexing added
// 25 May 2000 (GS)  bug fix in filling of residual histograms
// 29 May 2000 (GS)  target spots added
// 27 Jun 2000 (GS)  bug fix in residual calculation
// 28 Jun 2000 (GS)  signal / noise added
// 19 Jul 2000 (GS)  wrong histo id (203) fixed
// 26 Sep 2000 (GS)  bug fix in filling of VDS Hits/plane tracks histogram
// 19 Oct 2000 (GS)  unbiased residuals added
// 08 Nov 2000 (GS)  bug fix in unbiased residual calculation
// 28 Nov 2000 (GS)  Arte_mark included / polish printout
// 29 Nov 2000 (SM)  new histograms for primary vertices, all based on RVER
// 06 Dec 2000 (GS)  filling of some histograms commented out
// 07 Dec 2000 (GS)  normalisation to target rate introduced
// 08 Dec 2000 (GS)  trigger flag included
// 11 Dec 2000 (SM)  trigger flag included also for vertexing
// 22 Dec 2000 (GS)  martins histograms compacted in 3 2-D histograms
// 18 Jan 2001 (SM)  Histograms added for the J/psi vertices
// 19 Jan 2001 (SM)  Add J/psi-wire
// 01 Mar 2001 (SM)  more safety in selecting primaries only
// 06 Mar 2001 (GS)  handling target rates < 1
//  ******************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"
#include "event.h"

// Arte include
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "cats/hbook.h"
#include "dq/DQ.h"
#include "dq/DQ_SVD.h"

extern "C" {
#include "sltinfo.h"
}

// from Alignment stuff:
inline int fUnsin(int sl,int sec,int lay) { 
  int i = (lay-1)+(4*(sec-1))+(16*(sl-1));
  return(i); 
}

//extern int RoverHidden;
//extern int RoverNver;
//extern int rver_per_wnr[10];
//extern int ntr_per_rver[10];

extern "C"{
  //void prima_love(void);
  float prob_(const float & ccchi2, const int & ndeg); 
}
extern "C"{
  void dinv_(int*, double*, int*, double*, int*);
}

int DQ_SVD_vertex_evnt(int,int);
int DQ_SVD_unbiasres_evnt(int, int, float*);
//
// function code
//
int
DQ_SVD_rec_evnt()
{
  // show up
  ARTE_mark( "DQ_SVD_rec_evnt 0", 0 );

  int VDShitsontrack=0;      
  int nrSVDtracks=0;
  int nrVDStrackhits=0; // no of track associated hits/event

//====================================================================
//     target information
//====================================================================
          
  float targetz= 0.;
  int ntargets = 0;
  int usedwire;
        
  for( ArteTable<GTAR>::iterator target = ArteTable<GTAR>::begin();
     target!=ArteTable<GTAR>::end(); ++target ) {
 
     if((target->x*target->x+target->y*target->y)<1.0){
        ntargets++;
        usedwire = target->wnr;
        targetz = targetz + target->z;
      }
  }
           
  if(ntargets!=0){
    if(ntargets>1){
      targetz = targetz/ntargets;
    } 
  }
  float IR = faDTARia();
  float BG = faDTARbg();
            
  //no scaling if no target
  if (IR<=1)
    IR = 1;

//====================================================================
//     trigger flag   
//====================================================================

  int tflag = -100; 
  if (faDDAQFCS_tcode()==6
      || (faDDAQFCS_tcode()>=8 && faDDAQFCS_tcode()<=15)) {
    //cout << "This is a random trigger " << endl;
    tflag = 0;
  }
  else {
    if (SLTReadDSLT()==SLT_OK) {
      SLT_STATUS stat;
      if (SLTStatus(&stat)==SLT_OK) {
        int sltcode=stat.status;
        if (sltcode==SLTEVENT_ACCEPT) {
          // Event was accepted by the SLT
          //cout << "Event accepted by SLT " << endl;
          tflag = 1;
        }
      }
//        else {
          // Event is a bypass event
          //cout << "Event bypassed by SLT " << endl;
//          tflag = 600;
//        }
     }
    }

//====================================================================
//     information about segments
//====================================================================

  // loop over all tracks, track=pointer to RSEG information
  for(ArteTableIterator<RSEG> track;track.valid();track.next()){
    float tx=track->txf;  // x-slope at first hit (smallest z)
    float ty=track->tyf;
    
    /* VDS Modified !!================================== */
    // VDS segments only 65537=CATS  1=HOLMES
    if(track->cmp ==1||track->cmp==65537){ 
      VDShitsontrack = 0;

      float VDSxp1,VDSyp1,VDSxp2,VDSyp2,VDSres, VDSunbiasres, VDSocc, VDSusid, VDSnoise;
      VDSxp1 = VDSyp1 = VDSxp2 = VDSyp2 = VDSres = VDSunbiasres = VDSocc = VDSusid = VDSnoise = 0;
      float VDSx = track->xf;
      float VDSy = track->yf;
      float VDSz = track->zf;
            
//      // calculate point at target plane: 
//      float xx = VDSx - tx*(VDSz-targetz);
//      float yy = VDSy - ty*(VDSz-targetz);
//         
//      rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,211),xx,yy,1.);
//      rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,212),xx,1.);
//      rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,213),yy,1.);   

//      int sect = 0;
//      int prevsect = 0;
//      int first = 0;
	
      // loop over the hits of track      
      int hit=0;	//hit counter
      for ( ArteRelationIterator<HITB> hitbIt = track->hitb.begin();
	    hitbIt != track->hitb.end(); ++hitbIt ) {
	hit += 1;
	ArtePointer<HITB> hitb(hitbIt);     // hitb = pointer to HITB 
	ArtePointer<GEDE> gede(hitb->gede); // gede = pointer to GEDE
	
//	// all hits in the same sector?
//        if(sect >= 0) {
//        sect = gede->sec;
//	  if(!first) {		//first hit of track?
//	    prevsect = sect;
//	    first = 1;
//	  }
//	  if(sect != prevsect)
//	     sect = -1;		//flag that NOT all hits in same sector
//	}	

 	// calculate the plane ID:
	VDSusid = float(fUnsin(gede->gesl->nsl,gede->sec,gede->lay) ); 

	++VDShitsontrack;		// increment number of hits per track:
	++nrVDStrackhits;		// increment total number of hits

	// book residuals: VDSxp1 = extrapolated track position at 
	// the plane of hit, VDSxp2 taking rotation from alignment into account
	VDSxp1 = VDSx + tx*( (gede->zmax+gede->zmin)/2 - track->zf ); 
	VDSyp1 = VDSy + ty*( (gede->zmax+gede->zmin)/2 - track->zf );
	VDSxp2 = VDSxp1*gede->rot[0] + VDSyp1*gede->rot[3];
	VDSyp2 = VDSxp1*gede->rot[1] + VDSyp1*gede->rot[4];
	VDSres = 10000 * (hitb->x - ((VDSxp2*gede->cos) - (VDSyp2*gede->sin) ));

	rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,204),VDSres,1); // to total residual
//	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,203),VDSusid,VDSres,1); // to residual
	hfill(DQ_ID(COMP_SVD,LVL_REC,209),VDSusid,VDSres,1); // to residual (profile)
	
	double             x, y, txx, tyy,um,u,utrack;
	double             da[16], db[4]; 
	double             proj_tr[4], proj_vv[16], proj_trConf[4];
	double             dIntWork[16];
	int iFail;
	int n,k;

	um = -1.;
	// fill weight matrix, constraint vector, projectors
	for(int i=0; i<16; i++) da[i] = 0.0;
	for(int i=0; i<4;  i++) db[i] = 0.0;
	for(int i=0; i<16; i++) proj_vv[i] = 0.0;
	for(int i=0; i<4;  i++) proj_tr[i] = 0.0;

	//store projector to confirming plane (with stereo and global rotations)
	proj_trConf[0] =  gede->cos*gede->rot[0] - gede->sin*gede->rot[1];
	proj_trConf[1] =  gede->cos*gede->rot[3] - gede->sin*gede->rot[4];
	proj_trConf[2] = (gede->cos*gede->rot[0] - gede->sin*gede->rot[1])*((gede->zmax+gede->zmin)/2);
	proj_trConf[3] = (gede->cos*gede->rot[3] - gede->sin*gede->rot[4])*((gede->zmax+gede->zmin)/2);

	//loop over hits of track and refit track excluding current hit (confirming plane) 
	k = 0;	//hit counter
	for ( ArteRelationIterator<HITB> hitbIter = track->hitb.begin();
            hitbIter != track->hitb.end(); ++hitbIter ) {

	    ArtePointer<HITB> hitbP(hitbIter);		// hitbP = pointer to HITB

	    if (hitbP->gede->cmp == 1) {       //VDS hit?
	      u = hitbP->x;
	      k += 1;

	      // exclude confirming plane from tracking
	      if (hit == k) {
	        um = hitbP->x;	//track coordinate of confirming plan
	        continue;		// exclude
	      }
	      proj_tr[0] =  hitbP->gede->cos*hitbP->gede->rot[0] - hitbP->gede->sin*hitbP->gede->rot[1];
	      proj_tr[1] =  hitbP->gede->cos*hitbP->gede->rot[3] - hitbP->gede->sin*hitbP->gede->rot[4];
	      proj_tr[2] = (hitbP->gede->cos*hitbP->gede->rot[0] - hitbP->gede->sin*hitbP->gede->rot[1])*((hitbP->gede->zmax+hitbP->gede->zmin)/2);
	      proj_tr[3] = (hitbP->gede->cos*hitbP->gede->rot[3] - hitbP->gede->sin*hitbP->gede->rot[4])*((hitbP->gede->zmax+hitbP->gede->zmin)/2);

	      // : vv is vector_v dot Transpose(vector_v) == sym. matrix!
	      for(int i=0; i<4; i++)
	       for(int j=0; j<4; j++)
	        proj_vv[4*i+j] = proj_tr[i]*proj_tr[j];

	      for(int j=0; j<4;  j++) db[j] += proj_tr[j] * u;
	      for(int j=0; j<16; j++) da[j] += proj_vv[j];

	    } // if VDS hit
	} //end of loop over hits

	x = y = txx = tyy = utrack = 0.;
	// get the covariance matrix
	n=4;

	// CALL to Cern-Lib:
	dinv_(&n, da, &n, dIntWork, &iFail);
	if(iFail) {
          ARTE_error( GRR_SWARNING, "refit inversion failed", 0 );

//        printf("DQ_SVD_rec_evnt warning: refit inversion failed \n");
	  return(-1); // Do NOT contribute with shit to any reasonable measurement!
	}

	// determine the fit parameters
	for(int i=0; i<4; i++) {
	  x  += da[4*0+i]*db[i];
	  y  += da[4*1+i]*db[i];
	  txx+= da[4*2+i]*db[i]; 
	  tyy+= da[4*3+i]*db[i];
	}

	utrack =	x*(proj_trConf[0])	+	//
			y*(proj_trConf[1])	+	//
			txx*(proj_trConf[2])	+	//
			tyy*(proj_trConf[3])  ;		//

	VDSunbiasres = (um - utrack)*10000;

	//unbiased residuals	
	rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,208),VDSunbiasres,1); // to total residual
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,207),VDSusid,VDSunbiasres,1); // to residual
	hfill(DQ_ID(COMP_SVD,LVL_REC,210),VDSusid,VDSunbiasres,1); // to residual (profile)

	// the occupancy:
	VDSocc = (hitb->x - gede->xfw[0]) / gede->pit;
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,202),VDSusid,VDSocc,(1/IR)); // to occupancy
        rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,201),VDSusid,(1/IR));

	// S/N, signal, noise
        VDSnoise = (hitb->q)/(hitb->y);
        if(hitb->q > 0) {    // N-sides
          rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,220), VDSusid,VDSocc, (hitb->y/IR));
          rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,221), VDSusid,VDSocc, (hitb->q/IR));
          rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,222), VDSusid,VDSocc, (VDSnoise/IR));
        }
        else {                // P-sides 
          rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,220), VDSusid,VDSocc, (-hitb->y/IR));
          rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,221), VDSusid,VDSocc, (-hitb->q/IR));
          rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,222), VDSusid,VDSocc, (VDSnoise/IR));
        }

      } // end of loop over hits of track
  

//      //fill target spots only if all hits in same sector
//      if(sect!=-1) {	
//        if(sect==1){
//          rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,214),xx,yy,1.);   
//        }
//        else if(sect==2){
//          rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,215),xx,yy,1.);
//        }
//        else if(sect==3){
//          rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,216),xx,yy,1.);
//        }
//        else if(sect==4){
//          rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,217),xx,yy,1.);
//        }
//        else{
//          cout << "DQ_SVD_rec_evnt WARNING: not used sector: " << sect << endl;
//        }
//      }			//end of fill target spots
//
//      VDSxp1 = VDSx + (tx * (100 - track->zf));
//      VDSyp1 = VDSy + (ty * (100 - track->zf));
//      rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,205),10*VDSxp1,10*VDSyp1,1); // to refplane
      nrSVDtracks++;

      if ( VDShitsontrack > 0 )
	rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,206),VDShitsontrack,1);

    } // if VDS-track (rseg->cmp==1)
    
    /* End of VDS modified ============================= */
  }
  rhp_hf1(DQ_ID(COMP_SVD,LVL_REC,199+tflag),(nrSVDtracks/IR),1);
  
//====================================================================
//     vertexing
//====================================================================

  DQ_SVD_vertex_evnt(nrSVDtracks,tflag);

  // show up
  ARTE_mark( "DQ_SVD_rec_evnt 1", 0 );

  return 0; // return error code (0 = okay)
}

int
DQ_SVD_vertex_evnt(int vds_rseg, int tflag)
{ 

  int nwires = 0;   
  int nvtx = 0;
  int nvtx_wire[9] = {0,0,0,0,0,0,0,0,0};
  int vds_used = 0; float frac_used = 0.;
  
  ////*******************   rate    *************************////

  float IR = faDTARia();
  //no scaling if no target
  if (IR<=1)
    IR = 1;
  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,328),faDTARia(), 0.0, 1.0);

  ////*******************   GTAR    *************************////

  for( ArteTable<GTAR>::iterator target = ArteTable<GTAR>::begin();
       target!=ArteTable<GTAR>::end();
       ++target ) {
    if((target->x*target->x+target->y*target->y)<1.0){
      rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,329),target->wnr , 0.0, 1.0);
      nwires++;
    }
  }
  ////*******************   RTAR    *************************////

  if (ArteTable<RTAR>::size() !=0){
    for( ArteTable<RTAR>::iterator rtarget = ArteTable<RTAR>::begin();
	 rtarget!=ArteTable<RTAR>::end();
	 ++rtarget ) {
      if((rtarget->x*rtarget->x+rtarget->y*rtarget->y)<1.0){
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,330),rtarget->wnr , 0.0, 1.0);
	// Comparison RTAR - GTAR: out for the moment
	//ArtePointer<GTAR> rgtar(rtarget->wnr);
	//float tdx = rtarget->x - rgtar->x;
	//float tdy = rtarget->y - rgtar->y;
	//float tdz = rtarget->z - rgtar->z;
	//int index = (rtarget->wnr-1)*10;
	// ATTENTION: hi numbers do not match any more!!!
	//rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,330+index),tdx , 0.0, 1.0);
	//rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,331+index),tdy , 0.0, 1.0);
	//rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,332+index),tdz , 0.0, 1.0);
      }
    }
  }
  ////*******************   RVER    *************************////

  if (ArteTable<RVER>::size() !=0){
    for( ArteTable<RVER>::iterator rverIt = ArteTable<RVER>::begin();
         rverIt != ArteTable<RVER>::end() ; ++rverIt){

      if( ((*rverIt)->btra).index() == 0 && (*rverIt)->type == 1){ // consider primaries only
        nvtx++;
	nvtx_wire[(*rverIt)->wire]++;
	int it = (*rverIt)->wire;

	// see below: fill them only if there is 1 vertex in the event
	//rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,310), float((*rverIt)->nt), 0.0, 1.0);
	//rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,310+it), float((*rverIt)->nt), 0.0, 1.0);
	vds_used += (*rverIt)->nt;

	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,319), (*rverIt)->chi2, 0.0, 1.0);
        int ndf = 2*rverIt->nt - 3;
        float mychi2 = rverIt->chi2*ndf;
        float myprob = prob_(mychi2,ndf);
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,320), myprob , 0.0, 1.0);
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,321), (*rverIt)->xv, 0.0, 1.0);
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,322), (*rverIt)->yv, 0.0, 1.0);
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,323), (*rverIt)->zv, 0.0, 1.0);
	float dx = ((*rverIt)->cv[0] > 0) ? sqrt((*rverIt)->cv[0]) : (*rverIt)->cv[0];
	float dy = ((*rverIt)->cv[2] > 0) ? sqrt((*rverIt)->cv[2]) : (*rverIt)->cv[2];
	float dz = ((*rverIt)->cv[5] > 0) ? sqrt((*rverIt)->cv[5]) : (*rverIt)->cv[5];
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,324),dx , dy , 1.0);
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,325),dz , 0.0, 1.0);

	// histos for Martin
        rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,500), it, (*rverIt)->xv, 1.0);
        rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,500), it+8, (*rverIt)->yv, 1.0);
        rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,501), it, (*rverIt)->zv, 1.0);
        rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,502), it, dx, 1.0);
        rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,502), it+8, dy, 1.0);
        rhp_hfill(DQ_ID(COMP_SVD, LVL_REC,502), it+16, dz, 1.0);

	/* rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,(500+((it-1)*10))), (*rverIt)->xv, 0.0, 1.0);
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,(501+((it-1)*10))), (*rverIt)->yv, 0.0, 1.0);
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,(502+((it-1)*10))), (*rverIt)->zv, 0.0, 1.0);
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,(503+((it-1)*10))),dx , 0.0, 1.0);
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,(504+((it-1)*10))),dy , 0.0, 1.0);
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,(505+((it-1)*10))),dz , 0.0, 1.0);
	*/
	// end histos for Martin

      }
      // Histograms for the J/psi->mumu
      else if( (*rverIt)->btra->topp.index()==295 &&
	       (*rverIt)->btra->fit==2 ){
	rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,600),(*rverIt)->btra->fid , 0.0, 1.0);
	float probchi2 = prob_((*rverIt)->chi2,1);
	// ----------->   calculate deltaz = z_J/psi - z_wire
	// pick the wire which is closest in z
	float mindist = 1e10; float dist = 1e9;
	ArtePointer<RTAR> closest = 0;
	for( ArteTable<RTAR>::iterator rtarget = ArteTable<RTAR>::begin();
	     rtarget!=ArteTable<RTAR>::end();
	     ++rtarget ) {
	  // consider only wires close to beam
	  if(sqrt(rtarget->x*rtarget->x+rtarget->y*rtarget->y)<1.0){ 
	    dist = fabs((*rverIt)->zv - (*rtarget)->z);
	    if(dist < mindist){
	      mindist = dist;
	      closest = (*rtarget);
	    }
	  }
	}
	// calculate deltaz (absolute and in sigma)
	float jdz =0.; float dz =0.; float deltaz = 0.;  float deltaz_sigma = 0.;
	if (closest != 0){
	  jdz = ((*rverIt)->cv[5]>0.0) ? sqrt((*rverIt)->cv[5]) : 1e9;
	  dz = sqrt( (jdz*jdz) + ((closest->dz/sqrt(12))*(closest->dz/sqrt(12))) );
	  deltaz = (*rverIt)->zv - closest->z;
	  deltaz_sigma = deltaz/dz;
	}

	if((*rverIt)->btra->fid==2443 || (*rverIt)->btra->fid==3443){
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,601),(*rverIt)->mass , 0.0, 1.0);
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,602),(*rverIt)->xv , 0.0, 1.0);
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,603),(*rverIt)->yv , 0.0, 1.0);
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,604),(*rverIt)->zv , 0.0, 1.0);
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,605),probchi2 , 0.0, 1.0);
	  if(probchi2 >= 0.05)
	    rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,606),(*rverIt)->zv , 0.0, 1.0);
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,607),deltaz , 0.0, 1.0);
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,608),deltaz_sigma , 0.0, 1.0);
	}
	else if((*rverIt)->btra->fid==443 || (*rverIt)->btra->fid==1443){
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,611),(*rverIt)->btra->lee , 0.0, 1.0);
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,612),(*rverIt)->xv , 0.0, 1.0);
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,613),(*rverIt)->yv , 0.0, 1.0);
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,614),(*rverIt)->zv , 0.0, 1.0);
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,615),probchi2 , 0.0, 1.0);
	  if(probchi2 >= 0.05)
	    rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,616),(*rverIt)->zv , 0.0, 1.0);
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,617),deltaz , 0.0, 1.0);
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,618),deltaz_sigma , 0.0, 1.0);
	}
      }
    } // end loop on RVER
    // Fill a few histograms only if there is  exactly 1 primary in the event
    if (nvtx == 1){
      for( ArteTable<RVER>::iterator rverIt = ArteTable<RVER>::begin();
	   rverIt != ArteTable<RVER>::end() ; ++rverIt){
	if( ((*rverIt)->btra).index() == 0 ){    // consider primaries only
	  int it = (*rverIt)->wire;
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,340+tflag), float((*rverIt)->nt), 0.0, 1.0);
	  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,310+it), float((*rverIt)->nt), 0.0, 1.0);
	}
      }
    }
  }

  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,297+tflag), float(nvtx),0.0, 1.0);
  rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,299+tflag), float(nvtx)/IR, 0.0, 1.0);
  for(int itar=1;itar<=8;itar++){
    // out, redundant
    //rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,300+itar), float(nvtx_wire[itar]), 0.0, 1.0);
    rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,309),itar,float(nvtx_wire[itar]),1.0);
  }

  ////*******************  segments    *************************////
  if( vds_rseg != 0 ){
    frac_used = float(vds_used)/float(vds_rseg);
    if (frac_used >0 && nvtx==1)
      rhp_hfill(DQ_ID(COMP_SVD,LVL_REC,326+tflag), frac_used , 0.0, 1.0);
  }

  return 0; // return error code (0 = okay)
}

