
// *****************************************************************************
//
// source:    faLine.C
//
// type:      source code
//
// created:    7 Jul 1997
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    faLine()
//
// input:     void
// output:    int  flag = 0: farm mode off (normal Arte)
//                      = 1: farm mode on  (farm I/O)
//
// called by: atinit_()
//
// purpose:   evaluate line command arguments and set globals accordingly
//
// hints:     FARM mode can be enabled or disabled by giving either:
//            DAQ file name
//            EMG partition name
//
// changes:   
//                  line command arguments to steer rpm (AG: 25-Nov-98)
//                  line command argument for delayed reading (AG: 26-Nov-98)
//                  multiple file input (AG: 27-Nov-98)
//                  daq format logging (AG: 1-Dec-98)
//                  number of emg segmens to 8 (AG: 12 Jan 99)
//                  more polishing (AG: 15 Jan 99)
//                  print version description in help (AG: 21 Jan 99)
//                  polish rpm handling (AG: 2 Feb 99)
//                  submission date of sources (AG: 3 Feb 99)
//                  increased default buffer size (AG: 11 Feb 99)
//                  introduce argument to cycle until stop (AG: 12 Feb 99)
//                  use same buffer size as everywhere else (AG: 17 Feb 99)
//                  polishing (AG: 19 Feb 99)
//                  init grr (AG: 25 Feb 99)
//                  add flag for online display (AG: 25 Mar 99)
//                  add flag to suppress log files (AG: 26 Mar 99)            
//                  add -length:<event_buffer_length> (AG: 14 May 99)
//                  change STAGE_DISK -> STAGE_PATH (AG: 26 May 99)
//                  put includes (AG: 1 Jun 99)
// 20 Jul 1999 (AG) increase default buffer size
// 22 Jul 1999 (AG) polishing
// 05 Oct 1999 (AG) polishing
// 08 Oct 1999 (AG) increase input file number to 200
// 10 Oct 1999 (AG) enable log file for online purposes
// 13 Oct 1999 (AG) improve command line argument handling
// 21 Oct 1999 (AG) polishing
// 03 Nov 1999 (AG) rename -L4Ocycle to -L4Onoblock
// 03 Nov 1999 (AG) print signal to file with -L4Ologsig:<file>
// 24 Jan 2000 (AG) introduce -L4Odummy mode to run without input files
// 23 Feb 2000 (AG) introduce Arte version number
// 27 Apr 2000 (AG) use ARTE_mark()
// 17 May 2000 (AG) add flag for printout of time info
// 14 Jun 2000 (AG) introduce new flags w/o prefix L4O
// 11 Jul 2000 (AG) introduce command -RHPpublish
// 12 Jul 2000 (AG) improved flag handling
// 26 Jul 2000 (AG) introduced second debug flag for testing
// 08 Nov 2000 (AG) added date and time of compilation
// 27 Nov 2000 (PK) added 2 global variables filled from run arguments 
//                  for CnA_stearing (KCnA) and CnA_debugging (cna_no)
// 19 Dec 2000 (AG) added FAMARK
// 17 Jan 2001 (AG) removed obsolete DAQLOG
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <string.h>

// ONLINE includes
#include "stage.h"

// Arte includes
#include "arte/ArteFlag.h"
#include "farm/fa.hh"
#include "farm/farm.hh"

// globals
char *DATE = __DATE__;
char *TIME = __TIME__;

char *LOGFILE  = 0;          // log file
char *LOGSIG   = 0;          // log file
int  FARM      = 0;          // default FARM mode
int  FAON      = 0;          // default FARM online=off
char *PROCNAME = 0;          // domain name
char *DOMNAME  = 0;          // process name
int  RPMINIT   = 0;          // init rpm
int  RHPPUB    = 0;          // rpm_publish()
int  NOSPAR    = 1;          // default sparsification
int  FADEB     = 0;          // default NO debug mode
int  FADEB2    = 0;          // default NO debug mode
int  FAMARK    = 0;          // default no printout of marks
int  FATIME    = 0;          // default NO printout of time info
int  NOBLOCK   = 0;          // no blocking of emg
int  DISPLAY   = 0;          // display
int  FAGRR     = 0;          // grr logger
char *PARNAME  = 0;          // default emg partition name
int  SEG_NUM   = 4;          // default emg number of segments
int  SEG_LEN   = 250000;     // default event buffer size
int  WAIT      = 0;          // delay for reading event
char *DBNAME   = 0;          // default data base file name
char *IN_FILE[N_IN_FILE];    // input files (NO defaults)
char *HBSTAGE  = STAGE_PATH; // stage disk
int  OLD       = 0;          // old file mode
int  nFiles    = 0;          // number of files

int  CYCLE     = 0;          // needed later
char Kcna[20]; // CnA-manager name needed to setup formatter msg interface
int cna_no = 0; // CnA-key for offline debugging

// externals
extern int  nArg;
extern char **vArg;

//
// program code
//
int
faLine()
{
  ARTE_mark( "faLine", 0 );

  // inits
  char *files = 0;
  char *ptr   = 0;

  // evaluate line command arguments arguments
  for( int i=1; i<nArg; i++ ){
    
    // help
    if( nArg - i ){
     if( !strcmp( vArg[i], "-help" ) ||
	  !strcmp( vArg[i], "-h" )    ||
	  !strcmp( vArg[i], "-H" )       ){
#ifdef _OUT_
	cout << "************************************************" << endl;
	cout << "* Arte version: " << arteflag_.artevers << " "
	     << "of " << DATE << " " << TIME << " *" << endl;
	cout << "************************************************" << endl;
	cout << "usage: " << vArg[0] << " <kumac> inter/batch"              
	     << endl;
	cout << "       [-help][-h][-H]"                                      
	     << endl;
	cout << "       PRINT and DEBUG options:"
	     << endl;
	cout << "       [-debug] debug info to stdout   (DEFAULT=" << FADEB             << ")"
	     << endl;
	cout << "       [-mark] marks to stdout         (DEFAULT=" << FAMARK            << ")"
	     << endl;
	cout << "       [-noprint] NO J*.prnt / J*.logf (DEFAULT=" << arteflag_.noprint << ")"
	     << endl;
	cout << "       [-timing] time info to stdout   (DEFAULT=" << FATIME            << ")"
	     << endl;
	cout << "       FARM offline options:"
	     << endl;
	cout << "       [-f <file1,file2,...(max 200)>  (DEFAULT=none)" 
	     << endl;
	cout << "       [-length:<event_buffer_length>] (DEFAULT=" << SEG_LEN << ")" 
	     << endl;
	cout << "       EMG options (FARM online mode):"
	     << endl;
	cout << "       [-EMGpart:<partition>]          (DEFAULT=" << PARNAME << ")" 
	     << endl; 
	cout << "       [-EMGnum:<segments>]            (DEFAULT=" << SEG_NUM << ")" 
	     << endl;
	cout << "       [-EMGseg:<segments>]            (DEFAULT=" << SEG_NUM << ")" 
	     << endl;
	cout << "       [-EMGlen:<length>]              (DEFAULT=" << SEG_LEN << ")" 
	     << endl;
	cout << "       RPM/RHP initialization:"
	     << endl;
	cout << "       [-RPMinit]                      (DEFAULT=" << RPMINIT    << ")"
	     << endl;
	cout << "       [-Process:<process>] name       (DEFAULT=" << "none"     << ")"
	     << endl;
	cout << "       [-Domain:<domain>] domain       (DEFAULT=" << "/OTHERS/" << ")"
	     << endl;
	cout << "       [-RHPpublish:<events>]          (DEFAULT=" << RHPPUB     << ")"
	     << endl;
	cout << "       Further options:"
	     << endl;
	cout << "       [-stage:<stage_disk>]           (DEFAULT=" << HBSTAGE << ")"
	     << endl;
	cout << "       [-grr] error logger             (DEFAULT=" << FAGRR   << ")"
	     << endl;
	cout << "       [-noblock] do NOT block emg     (DEFAULT=" << NOBLOCK << ")"
	     << endl;
	cout << "       [-dummy] one event w/o file     (DEFAULT=" << CYCLE   << ")"
	     << endl;
	cout << "       [-display] online display       (DEFAULT=" << DISPLAY << ")"
	     << endl;
	cout << "       [-wait:<seconds>] read-in delay (DEFAULT=" << WAIT    << ")"
	     << endl;
	cout << "       [-logfile:<logfile>]            (DEFAULT=" << LOGFILE << ")"
	     << endl;
	cout << "       [-logsig:<logsig>]              (DEFAULT=" << LOGSIG  << ")"
	     << endl;
#endif
	// terminate
	exit( ARTE_GOOD );
      }
    }

    if( nArg - i ){ 
      
      // debugging mode
      if( !strcmp( vArg[i], "-debug" ) ){
	FADEB  = 1;
	FAMARK = 1;
      }

      // debugging mode
      if( !strcmp( vArg[i], "-debug2" ) ){
	FADEB2 = 1;
	FAMARK = 1;
      }
      
      // mark printout mode
      if( !strcmp( vArg[i], "-mark" ) ){
	FAMARK = 1;
      }

      // J-file printout mode
      if( !strcmp( vArg[i], "-noprint" ) ){
	arteflag_.noprint = 1;
      }
      
      // printout mode
      if( !strcmp( vArg[i], "-timing" ) ){
	FATIME = 1;
      }

      // rpm init
      if( !strcmp( vArg[i], "-RPMinit" ) ){
	RPMINIT = 1;
      }
      
      // process
      if( !strncmp( vArg[i], "-Process:", 9 ) ){
	RPMINIT = 1;
	int len = strlen( vArg[i] ) - 9 + 1;
	PROCNAME = new char[len];
	memset( PROCNAME, '\0', len );
	strcpy( PROCNAME, vArg[i] + 9 );
      }
      
      // domain
      if( !strncmp( vArg[i], "-Domain:", 8 ) ){
	RPMINIT = 1;
	int len = strlen( vArg[i] ) - 8 + 1;
	DOMNAME = new char[len];
	memset( DOMNAME, '\0', len );
	strcpy( DOMNAME, vArg[i] + 8 );
      }

     // rhp publish
      if( !strncmp( vArg[i], "-RHPpublish:", 12 ) ){
	RHPPUB  = atoi( vArg[i] + 12 );
	RPMINIT = 1;
      }

      // error logging
      if( !strcmp( vArg[i], "-L4Ogrr" ) ){
	FAGRR = 1;
      }
      if( !strcmp( vArg[i], "-grr" ) ){
	FAGRR = 1;
      }
      
      // delayed reading 
      if( !strncmp( vArg[i], "-L4Owait:", 9 ) ){
	WAIT = atoi( vArg[i] + 9 );
      }
      if( !strncmp( vArg[i], "-wait:", 6 ) ){
	WAIT = atoi( vArg[i] + 6 );
      }
      
      // cycling until file cycle.stop appears
      if( !strcmp( vArg[i], "-L4Onoblock" ) ){
	NOBLOCK = 1;
      }
      if( !strcmp( vArg[i], "-noblock" ) ){
	NOBLOCK = 1;
      }

      // cycling without blocking emg
      if( !strcmp( vArg[i], "-L4Odisplay" ) ){
	DISPLAY = 1;
      }
      if( !strcmp( vArg[i], "-display" ) ){
	DISPLAY = 1;
      }

      // cycling without blocking emg
      if( !strcmp( vArg[i], "-L4Odummy" ) ){
	CYCLE = 1;
	FARM  = 1;
      }
      if( !strcmp( vArg[i], "-dummy" ) ){
	CYCLE = 1;
	FARM  = 1;
      }

      // event buffer size
      if( !strncmp( vArg[i], "-length:", 8 ) ){
	SEG_LEN = atoi( vArg[i] + 8 );
      }
      
      // PK: CnA name for L2FormArte
      if( !strncmp( vArg[i], "-cna_name:", 10 ) ){
	int len = strlen( vArg[i] ) - 10 + 1;
	cout << "-Cna_name:len=" << len << endl;
	strcpy(Kcna,vArg[i] + 10);
	cout << "cna_name: " << Kcna << endl;
      }
      
      // PK: CnA Key for offline L2FormArte debugging
      if( !strncmp( vArg[i], "-cna_no:", 8 ) ){
	cna_no = atoi( vArg[i] + 8 );
      }

      // emg partition name
      if( !strncmp( vArg[i], "-EMGpart:", 9 ) ){
	FARM    = 1; // farm mode
	FAON    = 1; // farm online mode
	int len = strlen( vArg[i] ) - 9 + 1;
	//cout << "-EMGpart:len=" << len << endl;
	PARNAME = new char[len];
	memset( PARNAME, '\0', len );
	strcpy( PARNAME, vArg[i] + 9 );
      }
      if( !strncmp( vArg[i], "-p:", 3 ) ){
	FARM    = 1; // farm mode
	FAON    = 1; // farm online mode
	int len = strlen( vArg[i] ) - 3 + 1;
	//cout << "-EMGpart:len=" << len << endl;
	PARNAME = new char[len];
	memset( PARNAME, '\0', len );
	strcpy( PARNAME, vArg[i] + 3 );
      }
      
      // emg partition number
      if( !strncmp( vArg[i], "-EMGnum:", 8 ) ){
	SEG_NUM = atoi( vArg[i] + 8 );
      }
      if( !strncmp( vArg[i], "-EMGseg:", 8 ) ){
	SEG_NUM = atoi( vArg[i] + 8 );
      }
      if( !strncmp( vArg[i], "-n:", 3 ) ){
	SEG_NUM = atoi( vArg[i] + 3 );
      }
      
      // emg partition siz
      if( !strncmp( vArg[i], "-EMGlen:", 8 ) ){
	SEG_LEN = atoi( vArg[i] + 8 );
      }
      if( !strncmp( vArg[i], "-l:", 3 ) ){
	SEG_LEN = atoi( vArg[i] + 3 );
      }

      // stage disk
      if( !strncmp( vArg[i], "-L4Ostage:", 10 ) ){
	int len = strlen( vArg[i] ) - 10 + 1;
	//cout << "-L4Ostage:len=" << len << endl;
	HBSTAGE = new char[len];
	memset( HBSTAGE, '\0', len );
	strcpy( HBSTAGE, vArg[i] + 10 );
      }
      if( !strncmp( vArg[i], "-stage:", 7 ) ){
	int len = strlen( vArg[i] ) - 7 + 1;
	//cout << "-L4Ostage:len=" << len << endl;
	HBSTAGE = new char[len];
	memset( HBSTAGE, '\0', len );
	strcpy( HBSTAGE, vArg[i] + 7 );
      }

      // log file
      if( !strncmp( vArg[i], "-L4Ologfile:", 12 ) ){
	int len = strlen( vArg[i] ) - 12 + 1;
	//cout << "-L4Ologfile:len=" << len << endl;
	LOGFILE = new char[len];
	memset( LOGFILE, '\0', len );
	strcpy( LOGFILE, vArg[i] + 12 );
      }
      if( !strncmp( vArg[i], "-logfile:", 9 ) ){
	int len = strlen( vArg[i] ) - 9 + 1;
	//cout << "-L4Ologfile:len=" << len << endl;
	LOGFILE = new char[len];
	memset( LOGFILE, '\0', len );
	strcpy( LOGFILE, vArg[i] + 9 );
      }

      // log file for sig
      if( !strncmp( vArg[i], "-L4Ologsig:", 11 ) ){
	int len = strlen( vArg[i] ) - 11 + 1;
	//cout << "-L4Ologsig:len=" << len << endl;
	LOGSIG = new char[len];
	memset( LOGSIG, '\0', len );
	strcpy( LOGSIG, vArg[i] + 11 );
      }
      if( !strncmp( vArg[i], "-logsig:", 8 ) ){
	int len = strlen( vArg[i] ) - 8 + 1;
	//cout << "-L4Ologsig:len=" << len << endl;
	LOGSIG = new char[len];
	memset( LOGSIG, '\0', len );
	strcpy( LOGSIG, vArg[i] + 8 );
      }
    }
    
    // input file
    if( nArg - i - 1 ){ 
      if( !strcmp( vArg[i], "-x" ) || 
	  !strcmp( vArg[i], "-f" )    ){
	FARM = 1; // farm mode
        files = new char[strlen( vArg[i+1] ) + 1];
	//cout << strlen( vArg[i+1] ) + 1 << endl;
        strcpy( files, vArg[i+1] );
        ptr = strtok( files, "," ); // find comma
        while( ptr ){
	  //cout << strlen( ptr ) + 1 << endl;
	  //cout << ptr << endl;
          IN_FILE[nFiles] = new char[strlen( ptr ) + 1];
          strcpy( IN_FILE[nFiles], ptr );
	  //cout << "n="    << nFiles + 1 << " file=" << ptr        << endl;
          nFiles++;
          ptr = strtok( NULL, "," );
	}
        delete[] files;
      }

      // old format
      if( !strcmp( vArg[i], "-x" ) ){
	OLD = 1; // old format (no logger)
      }
    }
  }
    
  // print globals
  cout << endl;
  cout << "SETTINGS: "
       << "debug="        << FADEB             << " "
       << "noprint="      << arteflag_.noprint << " "
       << "timing="       << FATIME            << endl;
  cout << "          "
       << "rpm_init="     << RPMINIT           << " "
       << "process:"      << PROCNAME          << " "
       << "domain:"       << DOMNAME           << " "
       << "rhp_publish="  << RHPPUB            << endl;
  cout << "          "
       << "delay="        << WAIT              << " "
       << "noblock="      << NOBLOCK           << endl;
  cout << "          "
       << "grr="          << FAGRR             << " "
       << "logfile="      << LOGFILE           << " "
       << "logsig="       << LOGSIG            << endl;

  // farm mode?
  if( !FARM ) return FARM;
  
  // farm online
  if( FAON ){
    cout << "          " 
	 << "partition="    << PARNAME << " "
	 << "number="       << SEG_NUM << " "
	 << "length="       << SEG_LEN << endl;
  }
  // farm offline
  else {
    cout << "          "
	 << "old="   << OLD     << " ";
    cout << "stage:" << HBSTAGE << " ";
    cout << endl;
    for(  int i=0; i<nFiles; i++ ){
      cout << "          "
	   << "file" << i + 1 << ":" << IN_FILE[i] << endl;;
    }
    cout << endl;
  }
  
  // return  
  return FARM;
}
