#ifndef GPACK_ARTE_INTERFACE_H
#define GPACK_ARTE_INTERFACE_H

/*
*************************************************************************
**
** Description: header file for GPACK - ARTE INTERFACE package "gart"
**		All functions are called from Fortran
**
** Created: 	 7 Jun 98
** Updated:	
**
** Author:  	Sergey Moukhine
**          	HERA-B Collaboration
**          	JINR-Dubna
**          	Russia
**
*************************************************************************
*/

#include <sys/param.h>

#include <arte/ArteFlag.h>

#ifdef  __cplusplus
extern "C" {
#endif


#define TABLE_NAME_LEN	  namelen*sizeof(int)
#define TABLE_DESC_LEN	2000
#define SELECT_CRIT_LEN	2000
#define ALRM_WAIT_TIME	  30	/* seconds wait each time         */
#define ALRM_WAIT_CNT	  10	/* wait CNT times by TIME seconds */


void gartopen_(int *lun, char *name, int *shm_flag, int *in_out_flag,
                         int *lrec, int *ios,
                         int  len);
/************************************************************************
**
** PARAMETERS
**	lun	 - stream number from Arte (1..99)
**	name	 - filename to be opened
**	shm_flag - 
**		   1 for Shared Memory (open / create+open)
**		   0 for other files
**	in_out_flag - 
**		   1 open for output
**		   0 open for input
**	lrec	 - record size (for Shared Memory: size of SHM)
**	ios	 - (output) see GartErrorCode and StreamErrorCode (0 - OK)
**	len	 - lenght of name (needed for FORTRAN strings)
**
** FUNCTION DEFINITION
**	Open stream on lun. 
*************************************************************************
*/

void gartclos_(int *lun, int *ios);

/************************************************************************
**
** PARAMETERS
**	lun	 - stream number from Arte (1..99)
**	ios	 - (output) see GartErrorCode and StreamErrorCode (0 - OK)
**
** FUNCTION DEFINITION
**	Close stream on lun.
*************************************************************************
*/

void gartend_(void);

/************************************************************************
**
** PARAMETERS
**	None
**
** FUNCTION DEFINITION
**	Close all streams if necessary.
*************************************************************************
*/

void gartendev_(int *lun);

/************************************************************************
**
** PARAMETERS
**	lun	 - stream number from Arte (1..99)
**
** FUNCTION DEFINITION
**	End of event for stream on lun.
*************************************************************************
*/

void gartsetgpack_(int *lun);

/************************************************************************
**
** PARAMETERS
**	lun	 - stream number from Arte (1..99)
**
** FUNCTION DEFINITION
**	Set stream on lun to be a gpack stream.
*************************************************************************
*/

void gartcheck_(int *lun, int *is_gpack);

/************************************************************************
**
** PARAMETERS
**	lun	 - stream number from Arte (1..99)
**	is_gpack - (output) =1 for gpack, =0 otherwize
**
** FUNCTION DEFINITION
**	Checks whether stream on lun is a gpack stream.
*************************************************************************
*/

void gartothers_(int *lun, int *may_not_gpack);

/************************************************************************
**
** PARAMETERS
**	lun	  - stream number from Arte (1..99)
**	may_not_gp - (output) =1 for not gpack, =0 otherwize
**
** FUNCTION DEFINITION
**	Checks whether stream on lun may be a non-gpack stream.
*************************************************************************
*/

void gartrewi_(int *lun, int *ios);

/************************************************************************
**
** PARAMETERS
**	lun	 - stream number from Arte (1..99)
**	ios	 - (output) see GartErrorCode and StreamErrorCode (0 - OK)
**
** FUNCTION DEFINITION
**	Rewind stream on lun.
*************************************************************************
*/

void gartlimi_(int *lun, float *limit);

/************************************************************************
**
** PARAMETERS
**	lun	 - stream number from Arte (1..99)
**	limit	 - file limit in Mbytes
**
** FUNCTION DEFINITION
**	Set file limit for a stream on lun.
*************************************************************************
*/

void gartwtab_(int *lun, int *tab, int *inew, int *questArr);

/************************************************************************
**
** PARAMETERS
**	lun	 - stream number from Arte (1..99)
**	tab	 - table index (data start from iq(jlink(tab)+1) )
**	inew	 - start of event flag: =1 for new event, =0 otherwize
**	questArr - (output) return code:
**	questArr(1)	 0 normal completion
**			+1 "pseudo end of tape"
**			-1 first attampt to write after end_of_data
**			-2 error return
**
** FUNCTION DEFINITION
**	Write one table to stream on lun.
*************************************************************************
*/

void gartrnext_(int *lun, int *questArr);

/************************************************************************
**
** PARAMETERS
**	lun	 - stream number from Arte (1..99)
**	questArr - (output) return code:
**	questArr(1)	 0  normal completion
**			+4  End of File
**			<0  error return (StreamErrorCode)
**
** FUNCTION DEFINITION
**	Read next event header from stream on lun.
*************************************************************************
*/

void gartrtab_(int *lun, int *name, int *questArr);

/************************************************************************
**
** PARAMETERS
**	lun	 - stream number from Arte (1..99)
**	name	 - TABLE_NAME_LEN characters of the next table name
**	questArr - (output) return code:
**	questArr(1)	 0  normal completion
**			+4  End of File
**			+5  End of Data
**			100 end of event
**			<0  error return (StreamErrorCode)
**	questArr(11)	always set to zero (no start new event)
**
** FUNCTION DEFINITION
**	Read the description of next table from stream on lun.
*************************************************************************
*/

void gartrdat_(int *lun, int *tab, int *ncols, int *questArr);

/************************************************************************
**
** PARAMETERS
**	lun	 - stream number from Arte (1..99)
**	tab	 - table index: mmbook(jlink(tab),...)
**	ncols	 - n of output_columns in the table read
**	questArr - (output) return code:
**	questArr(1)	 0  normal completion
**			<0  error return (StreamErrorCode)
**
** FUNCTION DEFINITION
**	Read data of current table from stream on lun.
*************************************************************************
*/

void gartskiptab_(int *lun, int *ios);

/************************************************************************
**
** PARAMETERS
**	lun	 - stream number from Arte (1..99)
**	ios	 - (output) return code:
**			 0  normal completion
**			<0  error return (StreamErrorCode)
**			>0  end of event
**
** FUNCTION DEFINITION
**	Skip current table from stream on lun.
*************************************************************************
*/

void gartselact_(void);

/************************************************************************
**
** PARAMETERS
**	None
** FUNCTION DEFINITION
**	Action for KUIP command "select".
*************************************************************************
*/

typedef enum {
  GART_OK		= 0,
  GART_END_OF_EVENT,
  GART_WRONG_PAR,
  GART_NO_FILE,
  GART_BAD_SELECT,
  GART_NOT_IMPLEMENTED,
  GART_ERROR_MAX
} GartErrorCode;

typedef enum {				/* The order is important!	*/
  GartStreamWrong	= -1,
  GartStreamDstOut	=  0,
  GartStreamMiniOut,
  GartStreamNoteOut,
  GartStreamEventIn,
  GartStreamEvIn1,
  GartStreamEvIn2,
  GartStreamEvIn3,
  GartStreamEvIn4,
  GartStreamDstInNote,
  GartStreamMiniInNote,
  GartStreamNoteIn,
  GartStreamEvdirIn,
  GartStreamDstInEvdir,
  GartStreamMiniInEvdir,
  GartStreamOthers,
  GartStreamMax				/* Max must be always the last	*/
} GartStreamKind;

typedef enum {
  GartNotOpen	=  0,			/* stream is not opened		*/
  GartOpened,				/* opened			*/
  GartOpenWritten,			/* something is written		*/
  GartOpenEndOfData,			/* EOD during opening (evdir)	*/
  GartOpenMax
} GartOpenKind;

typedef struct GartStreamData GartStreamData;
struct GartStreamData {
  GartStreamData *next;			/* link for GartStreamOthers	*/
  GartOpenKind	isOpen;			/* open flag			*/
  char		fileName[MAXPATHNAME];	/* file name			*/
  int		gpackLun;		/* Lun from Gpack		*/
  int		arteLun;		/* Lun from Arte		*/
  int		isShm;			/* is it Shared Memory (flag)	*/
  int		isOutput;		/* is it Output lun (flag)	*/
  int		shmKey;			/* Shared Memory key		*/
  int		eventType;		/* from gpack.h; -1 if not set	*/
  int		limit;			/* file limit in bytes		*/
  int		ncols;			/* ncols from last gettabinfo	*/
  int		nrows;			/* nrows from last gettabinfo	*/
  EvdInfo	evdInfo;		/* Evdir information		*/
};


#ifdef  __cplusplus
       }
#endif

#endif
