/************************************************************************
*
* ECAGEO
* written by Siegmund Nowak
*
* This routines makes the standard geometry installation for the ECAL
*       called from geomain
*
************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <vector.h>
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteMessage.hh"
#include "arte/ArteUtil.h"
#include "gean/zgeom.h"
#include "gean/zgvers.hh"
#include "gean/ZGESL.h"
#include "gean/ZGDETV.h"

extern "C" {  
    void ecageo_(); 
    void cggesl_(char*, char*, int*);
    void gsvolu_(char*, char*, int*, float [], int*, int*);
    void gsatt_(char*,char*,int*);
    int  mednum_(int&,const char*,int);
    void crpos_(char*,char*,int*,int*,float*,float*,float*,int*,char*,float[]);
    void gspos_(char*,int*,char*,float*,float*,float*,int*,char*);
    void gsposp_(char*,int*,char*,float*,float*,float*,int*,char*,float*,int*); 
    void gsdvn_(char*,char*,int*,int*);
    void gsdet_(char*,char*,int*,char*,int*,int*,int*,int*,int*,int*);
    int  matnum_(char*,int);
    void caugeo_();
    void calmed_();
}

int    read_dat(FILE *,int &);  
FILE  *utfilp(const char*, const char*);
vector<hgcal> *hg_cal;

void ecageo_() {

//  some initializations
    extern  int first_geo[32];
    static char filnam[80];  static FILE *GEO_inp[32];  
    zgdet ZGDET; zgmed ZGMED; zgvol ZGVOL; zgcal ZGCAL; zgvers ZGVERS; 
    int cv_ndiv[3] = {1,2,5}, nsci[3], imat[3]; 
    int i0 = 0, i1 = 1,  i2 = 2, i3=3,  ical=-1;
    float xms[3],dzrad[3], dzsci[3], dzcal[3], par[3], f0 =0.;  
    char  mot[4], only[4], slnam[4], wallname[4], namrad[12], namesv[36];
    strncpy(mot,"MOTH",4);
    strncpy(only,"ONLY",4); 
    strncpy(wallname,"CAWA",4); 
    strncpy(namrad,"CAPBCAPBCATU",12);
    strncpy(namesv,"CAL1CAX1CAY1CAL2CAX2CAY2CAL3CAX3CAY3",36);
    zgmoth_.ZSTACK = 10000.;

//  Definition of the superlayer - posit the superlayer volume in gean
    ArtePointer<NESL> sl(zgmoth_.CURRSL);   
    strncpy(slnam,sl->name,4); 
    if (!ZGVOL.Check_Gvol(slnam)) cggesl_(slnam,mot,&i0);

    int cmp = sl->cmp;
    if(zgdetv_.DETV[cmp-1] == 'U') return;      

    zgeom ZGEOM(cmp);
    float xsl =.5*(sl->x1+sl->x2);
    float ysl =.5*(sl->y1+sl->y2);
    float z0  = 0.5*(sl->z1 - sl->z2);   

    if(first_geo[cmp-1] == 0) {
//     select file name and  open the file
       strcpy(filnam,ZGVERS.Get_vers_datafile(cmp));         
       GEO_inp[cmp-1] = utfilp(filnam,"");
    
//     reserve storage for zgdet, zgvol and zgcal 
       ZGDET.ini(3);  ZGVOL.ini(32);
       hg_cal     = new vector<hgcal>;
       if(hg_cal == NULL) { arteMsg << "ecageo: hg_cal = NULL"; arteMsg.error();
       } 

       first_geo[cmp-1] = 1;         
    }

//  now the card handling from input begins

    for (;;)   {
//     read in one media, rotation or volume block
       int datagroup=0; int nr=0; 
       datagroup = read_dat(GEO_inp[cmp-1],nr); 

       if (datagroup == 0 ) continue;               //     no action 

       if (datagroup == 1) {                        //     media
           ZGMED.Create_Gmed();   continue; 
       }

       if (datagroup == 2)   continue;              //     rotations not present

//     volume -  get volname and type
       char volname[5];              int nr_vol=nr;    
       strncpy(volname,ZGVOL.Get_volname(nr_vol),5);

       if (datagroup == 3 || datagroup==4 ) {       //     components 
         int sens_flag = ZGVOL.Get_sens_flag(nr_vol);   
	 if(sens_flag == 0 ) continue;
//       check whether the volume already exists - if not fill it
         int Gvolnum = ZGVOL.Check_Gvol(volname); 
         if(Gvolnum == 0) {
	   if(sens_flag <= 2) {                    // posit later via GSPOSP
               ZGVOL.Create_Gvol(nr_vol);
               Gvolnum = ZGVOL.Check_Gvol(volname);}
	   else {                                 //  posit later via GSPOS
               int imed = 0;
               char box[5], colo[5];
               strcpy(box,"BOX ");  strcpy(colo,"COLO");
               imed = ZGVOL.Get_imed(nr_vol);
               int npar = ZGVOL.Get_par(nr_vol,par);
               gsvolu_(volname,box, &imed, par, &npar, &Gvolnum);
               int col = ZGVOL.Get_color(nr_vol);
               gsatt_(volname, colo, &col);  
	    }
         }

         if(ZGVOL.Get_sens_flag(nr_vol) > 2 ) {
            if(!strncmp(volname,&namesv[0],4)) {
	      ical = 0;   imat[ical] = matnum_("ECAL_PB:SC",10);
            }
            else if(!strncmp(volname,&namesv[12],4)) {
              ical = 1;   imat[ical] = matnum_("ECAL_PB:SC",10);
            }
            else if(!strncmp(volname,&namesv[24],4)) {
              ical = 2;   imat[ical] = matnum_("ECAL_W:SC",9);
	    }
            ArtePointer<GMAT> gm(imat[ical]);
            
            float ms = 2.*par[2]/gm->radl;
            xms[ical] =  ms*pow(0.0136*(1.+ 0.038*log(double(ms))),2.);
            int nni[4]; float ddi[4];
            ZGDET.Get_n_wire(nr_vol,nni,i2);
            ZGDET.Get_x_wire(nr_vol,ddi,i2);
            nsci[ical]  = nni[0];
            dzsci[ical] = ddi[0];     dzrad[ical] = ddi[1];
            dzcal[ical] = 2.*par[2];
         }
       }

       else if (datagroup == 5 || datagroup == -99) {

	 if(!ZGVOL.Get_sens_flag(nr_vol) || strncmp(slnam,volname,4)) 
                                                               continue;

         char colo[5];   strcpy(colo,"COLO"); 
         int color = ZGVOL.Get_color(nr_vol);
         gsatt_(slnam,colo,&color);          

         calmed_();
 
//       loop over subvolumes 
	 for (unsigned int i=0; i<hg_cal->size(); i++) { 
           vector<hgcal>::iterator cal = hg_cal->begin() + i;
           if(cal->caba_size[0]*cal->ncalx > (cal->cal_xmax-cal->cal_xmin)
                                                                   +1.e-5) 
	     {   arteMsg << "ecageo: discrepancies in x"; arteMsg.warn();
           }
           if(cal->caba_size[1]*cal->ncaly > (cal->cal_ymax-cal->cal_ymin)
                                                                   +1.e-5)
	     {   arteMsg << "ecageo: discrepancies in y"; arteMsg.warn();
           }

           float xbl = 0.5*(cal->cal_xmin + cal->cal_xmax);
           float ybl = 0.5*(cal->cal_ymin + cal->cal_ymax);

//         loop over modules - first column - left one, first row - top one  
           for(int icol=0; icol<cal->ncalx; icol++) {
	     for(int irow=0; irow<cal->ncaly; irow++) {
              char cas = cal->pt_castru[irow*cal->ncalx+icol];
              if(cas == 'H')  continue;
              else if(cas == '1') ical = 0;
              else if(cas == '2') ical = 1;
              else if(cas == '3') ical = 2;
              else { 
                 arteMsg << "ecageo: ical=-1"; arteMsg.warn(); continue; 
	      }
              mmbook_(ArtePointer<NCAL>::getLink().getRef(), 1);
              int nd =ArteTable<NCAL>::size();
              ArtePointer<NCAL> nc(nd);

              float xp = 0.5*cal->caba_size[0]*(2*icol-cal->ncalx+1) + xbl;
              float yp =-0.5*cal->caba_size[1]*(2*irow-cal->ncaly+1) + ybl;
              par[0] = 0.5*cal->caba_size[0]; 
              par[1] = 0.5*cal->caba_size[1];

//            posit the wall equivalents and the modules
              par[2] = 0.5*cal->dzwall[ical];  
              float zp = cal->zwall[ical] + par[2] + z0;
              crpos_(wallname,slnam,&i1,&i0,&xp,&yp,&zp,&i0,only,par);
              par[2] = 0.5*cal->dzwall[ical+3];
              zp = cal->zwall[ical+3] + par[2] + z0;
              crpos_(wallname,slnam,&i2,&i0,&xp,&yp,&zp,&i0,only,par);
              par[2] = 0.5*dzcal[ical];
	      zp=cal->zcal[ical] + par[2] + z0;
              gspos_(&namesv[ical*12],&nd,slnam,&xp,&yp,&zp,&i0,only);

//            fill the arte tables
              nc->xmin = xsl + xp - .5*cal->caba_size[0];
              nc->xmax = xsl + xp + .5*cal->caba_size[0];
              nc->ymin = ysl + yp - .5*cal->caba_size[1];
              nc->ymax = ysl + yp + .5*cal->caba_size[1];
              nc->zmin =  sl->z1 + cal->zcal[ical];
              nc->zmax =  nc->zmin + dzcal[ical];
              nc->cmp = cmp;
              createRelation(sl,nc,&NESL::ncal);
              if(!sl->fpl) sl->fpl = nd;   
              sl->npl++;
              createRelation(ArtePointer<GMAT>(imat[ical]),nc,&GMAT::ncal); 
              strncpy(nc->gvol, &namesv[ical*12],4);
	      nc->ncl = cv_ndiv[ical];
              nc->sec = 100*(irow+1) + icol+1;
              nc->ms  = xms[ical];
              nc->eff = 1.;
            }
           }
         }

//       divide the modules into substructures and make them sensitive
//       fill zstack in ZGMOTH
	 vector<hgcal>::iterator cal = hg_cal->begin();
         for (int k= 0; k<3; k++) {
            gsdvn_(&namesv[12*k+4],&namesv[12*k],&cv_ndiv[k],&i1);
	    gsdvn_(&namesv[12*k+8],&namesv[12*k+4],&cv_ndiv[k],&i2);
            if(zgdetv_.DETV[cmp-1] == 'L') {
      	       par[0] = 0.5*cal->caba_size[0]/cv_ndiv[k];    
               par[1] = 0.5*cal->caba_size[1]/cv_ndiv[k];
               par[2] = 0.5*dzrad[k];
               float zp = -0.5*dzcal[k]+dzsci[k];  // start position
               for (int iz = 1; iz<nsci[k]; iz++) {
	          zp = zp + par[2]; 
                  gsposp_(&namrad[4*k],&iz,&namesv[12*k+8],&f0,&f0,&zp,
                          &i0,only,par,&i3); 
                  zp = zp + par[2] + dzsci[k];
               }
             }
//           make modules sensitive
	     int nbitsv[3] ={14,3,3}; int iset,idet, i20 =20, i450=450;
	     char set[4];         strncpy(set ,"CALO",4);
             gsdet_(set,&namesv[12*k+8],&i3,&namesv[12*k],nbitsv,
	            &cmp,&i450,&i20,&iset,&idet);
	   
//           fill zstack in ZGMOTH
             if(cal->zcal[k] >0 ) { 
                if(zgmoth_.ZSTACK > sl->z1 + cal->zcal[k]) 
	           zgmoth_.ZSTACK = sl->z1 + cal->zcal[k];
	     }
	 }
       }

//     end of file
       if ( datagroup == -99 )  {                              
         ZGDET.fin();     ZGVOL.fin();     zgcal::fin();    break;  
       }           
    }
  }
/* ecageo_ */

