#include <algorithm>

#ifndef GROVER_WITHOUT_ARTE
#include "arte/arte.hh"
#include "arte/ArteUtil.h"
#include "marple/marple.hh"
#endif

#include "grover/grover.hh"

// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// ------------------------------------------------------------------------
// 01-02-26  Silvia   Add all missing VDS segments
// 01-03-13  Max      Added preprocessor directives
//

#ifndef GROVER_WITHOUT_ARTE
extern int anl_TrSelect(vector< ArtePointer<RTRA> > &rtra);
#endif

int GroverFillSmarter(GroverParticleList* theList)
{
  ////////////////////////////////////////////////////////////////////////
  //
  // 1) Fills a GroverParticleList according to Michael Schmelling's and 
  //    Patricia Conde's routine for track selection and clone removal
  // 2) Ensures all VDS segments used at least once (Silvia Masciocchi)
  //
  // Returns 0 if OK, -1 if none found.
  //
  ////////////////////////////////////////////////////////////////////////


#ifndef GROVER_WITHOUT_ARTE
  const int ARRAYSIZE = 5000;

  // this is needed for the anl_TrSelect below
  static bool first = true;
  if (first){
    first = false;
    marpleinit_();
  }

  vector < ArtePointer<RTRA> > rtralist;
  int ntr = anl_TrSelect(rtralist);

  // but then add also all VDS-only segments in RTRA and other VDS
  // segments not yet selected

  // store VDS RSEG id already kept
  int vds_id[ARRAYSIZE] = {0};
  int i = 0;
  for (vector< ArtePointer<RTRA> >::iterator irtra = rtralist.begin();
       irtra != rtralist.end(); ++irtra) {
    vds_id[i] = (*irtra)->csvd;
    ++i;
  }
  int initial_size = i;

  int vadded = 0;
  for( ArteTable<RTRA>::iterator rtraIt = ArteTable<RTRA>::begin();
       rtraIt != ArteTable<RTRA>::end() ; ++rtraIt){
    if ( (*rtraIt)->hsi <= 0 ) continue;
    bool used = false;
    for (int j=0; j<ARRAYSIZE; ++j) {
      if (vds_id[j] != 0){
	if ((*rtraIt)->csvd == vds_id[j]) {
	  used = true; break;
	}
      }
    }
    if (!used) {
      rtralist.push_back((*rtraIt));
      vds_id[ntr+vadded] = (*rtraIt)->csvd;
      ++vadded;
    }
  }

  ntr = rtralist.size();
  //cout << "Length of particle list after addition = " << ntr << endl;

  if (ntr == 0) return FAILURE;
  else {
    for (vector< ArtePointer<RTRA> >::iterator irtra = rtralist.begin();
	 irtra != rtralist.end(); ++irtra) {
      if (*irtra == 0) continue;
      theList->push_back(GroverParticle(*irtra));
    }
  }
#endif


  return (theList->size() > 0) ? SUCCESS : FAILURE;
}
