#include <iostream>
#include <fstream>
#include <string>
#include "grover/grover.hh"

// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Defines KUIP functions for use in kumacs (or interactively)
// Max Sang 26/10/00
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -------------------------------------------------------------------------
// 00-11-xx  Max      Lots of added functions for setting things in ggCuts
// 01-02-28  Max      Added functions for moving axis marker, added command
//                    SHOWGLOBALS for dumping sizes of global ggXX.. objects
// 01-03-13  Max      Added preprocessor directives
//

extern "C" void kugetr_(float *); // kuip functions for retrieving arguments
extern "C" void kugeti_(int *);


extern "C" void  kgg2enable_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.GroverDraw) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Toggling Grover 2D display (HIGZ canvas)" << endl;
    ggCuts.GroverDraw = usernum;
  }
  return;
}



extern "C" void  kgg3enable_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.GroverDraw3D) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Toggling Grover 3D display (ROOT canvas)" << endl;
    ggCuts.GroverDraw3D = usernum;
  }
  return;
}



extern "C" void  kgg3x3d_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDX3D) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Toggling X3D display" << endl;
    ggCuts.TDX3D = usernum;
  }
  return;
}


extern "C" void  kgg3vbox_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDTargetRegion) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Toggling drawing of box around target region" << endl;
    ggCuts.TDTargetRegion = usernum;
  }
  return;
}



extern "C" void  kgg3vaxes_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDAxes) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Toggling drawing of axes" << endl;
    ggCuts.TDAxes = usernum;
  }
  return;
}



extern "C" void  kgg3vwires_(void)
{
  int usernum=1;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDWires) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Toggling drawing of target wires" << endl;
    ggCuts.TDWires = usernum;
  }
  return;
}


extern "C" void  kgg3vprimaries_(void)
{
  int usernum=1;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDPrimaries) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Toggling drawing of primary vertices" << endl;
    ggCuts.TDPrimaries = usernum;
  }
  return;
}



extern "C" void  kgg3vsecondaries_(void)
{
  int usernum=1;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDSecondaries) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Toggling drawing of secondary vertices" << endl;
    ggCuts.TDSecondaries = usernum;
  }
  return;
}


extern "C" void  kgg3vmcprimaries_(void)
{
  int usernum=1;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDMCPrimaries) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Toggling drawing of MC primary vertices" << endl;
    ggCuts.TDMCPrimaries = usernum;
  }
  return;
}



extern "C" void  kgg3vmcsecondaries_(void)
{
  int usernum=1;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDMCSecondaries) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Toggling drawing of MC secondary vertices" << endl;
    ggCuts.TDMCSecondaries = usernum;
  }
  return;
}



extern "C" void  kgg3vprimtracks_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDPrimaryTracks) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Toggling drawing of tracks in primary vertices" << endl;
    ggCuts.TDPrimaryTracks = usernum;
  }
  return;
}



extern "C" void  kgg3vsectracks_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDSecondaryTracks) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Toggling drawing of tracks in secondary vertices" << endl;
    ggCuts.TDSecondaryTracks = usernum;
  }
  return;
}



extern "C" void  kgg3vunusedtracks_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDUnusedTracks) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Toggling drawing of all tracks not associated with vertices" << endl;
    ggCuts.TDUnusedTracks = usernum;
  }
  return;
}



extern "C" void  kgg3vspecialtracks_(void)
{
  int usernum=1;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDSpecialTracks) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Toggling drawing of user-defined tracks" << endl;
    ggCuts.TDSpecialTracks = usernum;
  }
  return;
}

extern "C" void kgg3exec_(void)
{
  cout <<"*** NOTICE ***" << endl
       << " For technical reasons the 3D Grover display is disabled." << endl
       << " But! it's easy do it yourself. Put the following in a .C file" << endl
       << " in your user directory:" << endl << endl
       << " #include \"grover/3d.hh\" " << endl << endl
       << " and then in the same directory put a file uskuip.cdf" << endl
       << " which contains the following:" << endl << endl
       << " >NAME USKUIP" <<endl
       << " >MENU /USER" << endl
       << " #include \"grover/3d_cdf.hh\" " << endl << endl
       << " Then recompile and type DRAW3D to run the display."
       << endl;
  return;
}

extern "C" void  kgg3cwires_(void)
{
  int usernum=5;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDWireColour) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting colour of target wires" << endl;
    ggCuts.TDWireColour = usernum;
  }
  return;
}

extern "C" void  kgg3cprimaries_(void)
{
  int usernum=2;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDPrimaryColour) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting colour of primary vertices" << endl;
    ggCuts.TDPrimaryColour = usernum;
  }
  return;
}

extern "C" void  kgg3csecondaries_(void)
{
  int usernum=4;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDSecondaryColour) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting colour of secondary vertices" << endl;
    ggCuts.TDSecondaryColour = usernum;
  }
  return;
}

extern "C" void  kgg3cmcprimaries_(void)
{
  int usernum=3;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDMCPrimaryColour) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting colour of MC primary vertices" << endl;
    ggCuts.TDMCPrimaryColour = usernum;
  }
  return;
}

extern "C" void  kgg3cmcsecondaries_(void)
{
  int usernum=6;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDMCSecondaryColour) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting colour of MC secondary vertices" << endl;
    ggCuts.TDMCSecondaryColour = usernum;
  }
  return;
}

extern "C" void  kgg3cprimtracks_(void)
{
  int usernum=15;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDPrimaryTrackColour) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting colour of tracks in primary vertices" << endl;
    ggCuts.TDPrimaryTrackColour = usernum;
  }
  return;
}

extern "C" void  kgg3csectracks_(void)
{
  int usernum=7;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDSecondaryTrackColour) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting colour of tracks in secondary vertices" << endl;
    ggCuts.TDSecondaryTrackColour = usernum;
  }
  return;
}

extern "C" void  kgg3cunusedtracks_(void)
{
  int usernum=28;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDUnusedTrackColour) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting colour of tracks which were not used in vertices" 
	 << endl;
    ggCuts.TDUnusedTrackColour = usernum;
  }
  return;
}

extern "C" void  kgg3cspecialtracks_(void)
{
  int usernum=41;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDSpecialTrackColour) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting colour of special (user-defined) tracks" << endl;
    ggCuts.TDSpecialTrackColour = usernum;
  }
  return;
}

extern "C" void  kgg3daxisxpos_(void)
{
  float usernum=1.0;
#ifndef GROVER_WITHOUT_KUIP
  kugetr_(&usernum);
#endif
  if (usernum == ggCuts.TDAxisX) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting x position of axis marker" << endl;
    ggCuts.TDAxisX = usernum;
  }
  return;
}

extern "C" void  kgg3daxisypos_(void)
{
  float usernum=1.0;
#ifndef GROVER_WITHOUT_KUIP
  kugetr_(&usernum);
#endif
  if (usernum == ggCuts.TDAxisY) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting y position of axis marker" << endl;
    ggCuts.TDAxisY = usernum;
  }
  return;
}

extern "C" void  kgg3daxiszpos_(void)
{
  float usernum=1.0;
#ifndef GROVER_WITHOUT_KUIP
  kugetr_(&usernum);
#endif
  if (usernum == ggCuts.TDAxisZ) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting z position of axis marker" << endl;
    ggCuts.TDAxisZ = usernum;
  }
  return;
}

extern "C" void  kgg3dprimtracks_(void)
{
  float usernum=1.0;
#ifndef GROVER_WITHOUT_KUIP
  kugetr_(&usernum);
#endif
  if (usernum == ggCuts.TDPrimaryTrackLength) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting drawn length of tracks in primary vertices" << endl;
    ggCuts.TDPrimaryTrackLength = usernum;
  }
  return;
}

extern "C" void  kgg3dsectracks_(void)
{
  float usernum=1.0;
#ifndef GROVER_WITHOUT_KUIP
  kugetr_(&usernum);
#endif
  if (usernum == ggCuts.TDSecondaryTrackLength) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting drawn length of tracks in secondary vertices" << endl;
    ggCuts.TDSecondaryTrackLength = usernum;
  }
  return;
}

extern "C" void  kgg3dunusedtracks_(void)
{
  float usernum=10.0;
#ifndef GROVER_WITHOUT_KUIP
  kugetr_(&usernum);
#endif
  if (usernum == ggCuts.TDUnusedTrackLength) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting drawn length of unused tracks" << endl;
    ggCuts.TDUnusedTrackLength = usernum;
  }
  return;
}

extern "C" void  kgg3dspecialtracks_(void)
{
  float usernum=10.0;
#ifndef GROVER_WITHOUT_KUIP
  kugetr_(&usernum);
#endif
  if (usernum == ggCuts.TDSpecialTrackLength) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Setting drawn length of special (user-defined) tracks" << endl;
    ggCuts.TDSpecialTrackLength = usernum;
  }
  return;
}


extern "C" void  kgstselection_(void)
{
  int usernum=1;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.GETrackChoice) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Track selection changed to "<<usernum<<endl;
    ggCuts.GETrackChoice = usernum;
  }
  return;
}

extern "C" void  kgsfindprimaries_(void)
{
  int usernum=1;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.GEFindPrimaries) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Primary vertex finding toggled" << endl;
    ggCuts.GEFindPrimaries = usernum;
  }
  return;
}

extern "C" void  kgswritertar_(void)
{
  int usernum=1;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.GEWriteRTAR) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Filling of RTAR toggled" << endl;
    ggCuts.GEWriteRTAR = usernum;
  }
  return;
}

extern "C" void  kgswriterver_(void)
{
  int usernum=1;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.GEWriteRVER) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Filling of RVER toggled" << endl;
    ggCuts.GEWriteRVER = usernum;
  }
  return;
}

extern "C" void  kgsfindsecondaries_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.GEFindSecondaries) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Unbiased secondary vertex finding toggled" << endl;
    ggCuts.GEFindSecondaries = usernum;
  }
  return;
}

extern "C" void  kgsfindjpsiplusx_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.FindJpsiPlusX) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** J/psi+X vertex finding toggled" << endl;
    ggCuts.FindJpsiPlusX = usernum;
  }
  return;
}
extern "C" void  kgswirefollowing_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.GPWireFollowing) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Primary finder wire-following toggled" << endl;
    ggCuts.GPWireFollowing = usernum;
  }
  return;
}

extern "C" void  kgsrefitrseg_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.GERefitRSEGTable) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Refitting of RSEGs with MC momentum toggled" << endl;
    ggCuts.GERefitRSEGTable = usernum;
  }
  return;
}

extern "C" void  kgsverbose_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.Verbose) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Verbose messaging toggled" << endl;
    ggCuts.Verbose = usernum;
  }
  return;
}

extern "C" void  kgswhbook_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.Paw) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** HBOOK functions toggled" << endl;
    ggCuts.Paw = usernum;
  }
  return;
}

extern "C" void  kgswroot_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.Root) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** ROOT ntuple functions toggled" << endl;
    ggCuts.Root = usernum;
  }
  return;
}

extern "C" void  kgcpnhitmin_(void)
{
  int usernum=5;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.RPTrackQualNH) 
    cout << "*** Cut unchanged" << endl;
  else {
    cout << "*** Cut changed!" << endl;
    ggCuts.RPTrackQualNH = usernum;
  }
  return;
}

extern "C" void  kgcpntrkmin_(void)
{
  int usernum=3;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.RPMinTrksInVtx) 
    cout << "*** Cut unchanged" << endl;
  else {
    cout << "*** Cut changed!" << endl;
    ggCuts.RPMinTrksInVtx = usernum;
  }
  return;
}

extern "C" void  kgcpoverlap_(void)
{
  float usernum=3.5;
#ifndef GROVER_WITHOUT_KUIP
  kugetr_(&usernum);
#endif
  if (usernum == ggCuts.RPTrkWireSigmaMax) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Track/vertex overlap changed!" << endl;
    ggCuts.RPTrkWireSigmaMax = usernum;
  }
  return;
}

extern "C" void  kgcrsmear_(void)
{
  float usernum=0.03;
#ifndef GROVER_WITHOUT_KUIP
  kugetr_(&usernum);
#endif
  if (usernum <= 0.0) {
    cout << "*** MC momentum smearing switched off" << endl;
    ggCuts.CheatSmearing = 0;
    ggCuts.CheatSmearConst = 0.0;
    ggCuts.CheatSmearVar = 0.0;
  }
  else {
    cout << "*** MC momentum smearing switched on" << endl;
    ggCuts.CheatSmearing = 1;
    ggCuts.CheatSmearConst = usernum;
    ggCuts.CheatSmearVar = 0.0;
  }
  return;
}

extern "C" void  kgsshowcuts_(void)
{
  ggCuts.Dump();
  return;
}


extern "C" void  kgsshowglobals_(void)
{
  std::cout << "*** Global object sizes: par,prim,mcprim,sec,mcsec,wires,specials= "
       << ggParticles.size() << " "
       << ggPrimaries.size() << " "
       << ggMCPrimaries.size() << " "
       << ggSecondaries.size() << " "
       << ggMCSecondaries.size() << " "
       << ggWires.size() << " "
       << ggSpecialParticles.size() << " "
       << std::endl;
  return;
}



extern "C" void kgg3dpmin_(void)
{
  float usernum=0.0;
#ifndef GROVER_WITHOUT_KUIP
  kugetr_(&usernum);
#endif
  if (usernum == ggCuts.TDTrackMinP) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Only drawing tracks with >"<< usernum << " GeV"<< endl;
    ggCuts.TDTrackMinP = usernum;
  }
  return;
}

extern "C" void kgg3dptmin_(void)
{
  float usernum=0.0;
#ifndef GROVER_WITHOUT_KUIP
  kugetr_(&usernum);
#endif
  if (usernum == ggCuts.TDTrackMinPt) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Only drawing tracks with pt >"<< usernum << " GeV"<< endl;
    ggCuts.TDTrackMinPt = usernum;
  }
  return;
}

extern "C" void  kgg3dchi2max_(void)
{
  float usernum=0.0;
#ifndef GROVER_WITHOUT_KUIP
  kugetr_(&usernum);
#endif
  if (usernum == ggCuts.TDTrackMaxChi2) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Only drawing tracks with chi2 < "<< usernum << endl;
    ggCuts.TDTrackMaxChi2 = usernum;
  }
  return;
}

extern "C" void kgg3dminhits_(void)
{
  int usernum=0;
#ifndef GROVER_WITHOUT_KUIP
  kugeti_(&usernum);
#endif
  if (usernum == ggCuts.TDTrackMinNh) 
    cout << "*** Option unchanged" << endl;
  else {
    cout << "*** Only drawing tracks with >= "<< usernum << " VDS hits"<< endl;
    ggCuts.TDTrackMinNh = usernum;
  }
  return;
}
