#include <algorithm>
#include "grover/grover.hh"

//
// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
// 01-02-28  Max      const safer
//

//
// UTILITIES - SORTING ALGORITHMS, MATRIX INVERSION ETC.
// 

//
//  1) Functions for sorting Grover objects using 
//     STL algorithms
//

// a) GroverParticle


// Dist
bool SortGPByDist(GroverParticle a,
		  GroverParticle b)
{return (a.Dist() < b.Dist()) ? true:false;}
bool RevSortGPByDist(GroverParticle a,
		     GroverParticle b)
{return (a.Dist() > b.Dist()) ? true:false;}


// Secondary Weight
bool SortGPBySecWeight(GroverParticle a, 
		       GroverParticle b)
{return (a.SecondaryWeight() > b.SecondaryWeight()) ? true:false;}
bool RevSortGPBySecWeight(GroverParticle a, 
			  GroverParticle b)
{return (a.SecondaryWeight() < b.SecondaryWeight()) ? true:false;}


// Chi2
bool SortGPByChi2(GroverParticle a, 
		  GroverParticle b)
{return (a.Chi2() > b.Chi2()) ? true:false;}
bool RevSortGPByChi2(GroverParticle a, 
		     GroverParticle b)
{return (a.Chi2() < b.Chi2()) ? true:false;}


// b) GroverVertex
// (none yet)


//////////////////////////////////////////////////////////////////////////

//
// 2) Square matrix inversion using Gaussian elimination
//

void three_dim_inv(double a[3][3])
{
  /**** Gaussian algorithm for 3x3 matrix inversion ****/
  int i,j,k,l;
  double factor;
  double temp[3];
  double b[3][3];

  // Set b to I
  for(i=0;i<3;i++) for(j=0;j<3;j++)
    if(i==j) b[i][j]=1.0; else b[i][j]=0.0;

  for(i=0;i<3;i++){
    for(j=i+1;j<3;j++)
      if (fabs(a[i][i])<fabs(a[j][i])){
        for(l=0;l<3;l++) temp[l]=a[i][l];
        for(l=0;l<3;l++) a[i][l]=a[j][l];
        for(l=0;l<3;l++) a[j][l]=temp[l];
        for(l=0;l<3;l++) temp[l]=b[i][l];
        for(l=0;l<3;l++) b[i][l]=b[j][l];
        for(l=0;l<3;l++) b[j][l]=temp[l];
      }
    factor=a[i][i];
    // cout<<"Highest element "<<a[i][i]<<endl;
    for(j=3-1;j>-1;j--) {
      b[i][j]/=factor;a[i][j]/=factor;
    }
    for(j=i+1;j<3;j++) {
      factor=-a[j][i];
      for(k=0;k<3;k++){
        a[j][k]+=a[i][k]*factor;b[j][k]+=b[i][k]*factor;
      }
    }
  } 
  for(i=3-1;i>0;i--){
    for(j=i-1;j>-1;j--){
      factor=-a[j][i];
      for(k=0;k<3;k++){
        a[j][k]+=a[i][k]*factor;b[j][k]+=b[i][k]*factor;
      }
    }
  }

  // copy b to a and return
  for(i=0;i<3;i++) for(j=0;j<3;j++) a[i][j]=b[i][j];
}      


void four_dim_inv(double a[4][4])
{
  /**** Gaussian algorithm for 4x4 matrix inversion ****/
  int i,j,k,l;
  double factor;
  double temp[4];
  double b[4][4];

  // Set b to I
  for(i=0;i<4;i++) for(j=0;j<4;j++)
    if(i==j) b[i][j]=1.0; else b[i][j]=0.0;

  for(i=0;i<4;i++){
    for(j=i+1;j<4;j++)
      if (fabs(a[i][i])<fabs(a[j][i])){
        for(l=0;l<4;l++) temp[l]=a[i][l];
        for(l=0;l<4;l++) a[i][l]=a[j][l];
        for(l=0;l<4;l++) a[j][l]=temp[l];
        for(l=0;l<4;l++) temp[l]=b[i][l];
        for(l=0;l<4;l++) b[i][l]=b[j][l];
        for(l=0;l<4;l++) b[j][l]=temp[l];
      }
    factor=a[i][i];
    for(j=4-1;j>-1;j--) {
      b[i][j]/=factor;a[i][j]/=factor;
    }
    for(j=i+1;j<4;j++) {
      factor=-a[j][i];
      for(k=0;k<4;k++){
        a[j][k]+=a[i][k]*factor;b[j][k]+=b[i][k]*factor;
      }
    }
  } 
  for(i=3;i>0;i--){
    for(j=i-1;j>-1;j--){
      factor=-a[j][i];
      for(k=0;k<4;k++){
        a[j][k]+=a[i][k]*factor;b[j][k]+=b[i][k]*factor;
      }
    }
  }

  // copy b to a and return
  for(i=0;i<4;i++) for(j=0;j<4;j++) a[i][j]=b[i][j];
}      



void five_dim_inv(double a[5][5])
{
  /**** Gaussian algorithm for 5x5 matrix inversion ****/
  int i,j,k,l;
  double factor;
  double temp[5];
  double b[5][5];

  // Set b to I
  for(i=0;i<5;i++) for(j=0;j<5;j++)
    if(i==j) b[i][j]=1.0; else b[i][j]=0.0;

  for(i=0;i<5;i++){
    for(j=i+1;j<5;j++)
      if (fabs(a[i][i])<fabs(a[j][i])){
        for(l=0;l<5;l++) temp[l]=a[i][l];
        for(l=0;l<5;l++) a[i][l]=a[j][l];
        for(l=0;l<5;l++) a[j][l]=temp[l];
        for(l=0;l<5;l++) temp[l]=b[i][l];
        for(l=0;l<5;l++) b[i][l]=b[j][l];
        for(l=0;l<5;l++) b[j][l]=temp[l];
      }
    factor=a[i][i];
    // cout<<"Highest element "<<a[i][i]<<endl;
    for(j=5-1;j>-1;j--) {
      b[i][j]/=factor;a[i][j]/=factor;
    }
    for(j=i+1;j<5;j++) {
      factor=-a[j][i];
      for(k=0;k<5;k++){
        a[j][k]+=a[i][k]*factor;b[j][k]+=b[i][k]*factor;
      }
    }
  } 
  for(i=4;i>0;i--){
    for(j=i-1;j>-1;j--){
      factor=-a[j][i];
      for(k=0;k<5;k++){
        a[j][k]+=a[i][k]*factor;b[j][k]+=b[i][k]*factor;
      }
    }
  }

  // copy b to a and return
  for(i=0;i<5;i++) for(j=0;j<5;j++) a[i][j]=b[i][j];
}      

void six_dim_inv(double a[6][6])
{
  /**** Gaussian algorithm for 6x6 matrix inversion ****/
  int i,j,k,l;
  double factor;
  double temp[6];
  double b[6][6];

  // Set b to I
  for(i=0;i<6;i++) for(j=0;j<6;j++)
    if(i==j) b[i][j]=1.0; else b[i][j]=0.0;

  for(i=0;i<6;i++){
    for(j=i+1;j<6;j++)
      if (fabs(a[i][i])<fabs(a[j][i])){
        for(l=0;l<6;l++) temp[l]=a[i][l];
        for(l=0;l<6;l++) a[i][l]=a[j][l];
        for(l=0;l<6;l++) a[j][l]=temp[l];
        for(l=0;l<6;l++) temp[l]=b[i][l];
        for(l=0;l<6;l++) b[i][l]=b[j][l];
        for(l=0;l<6;l++) b[j][l]=temp[l];
      }
    factor=a[i][i];
    // cout<<"Highest element "<<a[i][i]<<endl;
    for(j=6-1;j>-1;j--) {
      b[i][j]/=factor;a[i][j]/=factor;
    }
    for(j=i+1;j<6;j++) {
      factor=-a[j][i];
      for(k=0;k<6;k++){
        a[j][k]+=a[i][k]*factor;b[j][k]+=b[i][k]*factor;
      }
    }
  } 
  for(i=5;i>0;i--){
    for(j=i-1;j>-1;j--){
      factor=-a[j][i];
      for(k=0;k<6;k++){
        a[j][k]+=a[i][k]*factor;b[j][k]+=b[i][k]*factor;
      }
    }
  }

  // copy b to a and return
  for(i=0;i<6;i++) for(j=0;j<6;j++) a[i][j]=b[i][j];
}      


void seven_dim_inv(double a[7][7])
{
  /**** Gaussian algorithm for 7x7 matrix inversion ****/
  int i,j,k,l;
  double factor;
  double temp[7];
  double b[7][7];

  // Set b to I
  for(i=0;i<7;i++) for(j=0;j<7;j++)
    if(i==j) b[i][j]=1.0; else b[i][j]=0.0;

  for(i=0;i<7;i++){
    for(j=i+1;j<7;j++)
      if (fabs(a[i][i])<fabs(a[j][i])){
        for(l=0;l<7;l++) temp[l]=a[i][l];
        for(l=0;l<7;l++) a[i][l]=a[j][l];
        for(l=0;l<7;l++) a[j][l]=temp[l];
        for(l=0;l<7;l++) temp[l]=b[i][l];
        for(l=0;l<7;l++) b[i][l]=b[j][l];
        for(l=0;l<7;l++) b[j][l]=temp[l];
      }
    factor=a[i][i];
    // cout<<"Highest element "<<a[i][i]<<endl;
    for(j=7-1;j>-1;j--) {
      b[i][j]/=factor;a[i][j]/=factor;
    }
    for(j=i+1;j<7;j++) {
      factor=-a[j][i];
      for(k=0;k<7;k++){
        a[j][k]+=a[i][k]*factor;b[j][k]+=b[i][k]*factor;
      }
    }
  } 
  for(i=6;i>0;i--){
    for(j=i-1;j>-1;j--){
      factor=-a[j][i];
      for(k=0;k<7;k++){
        a[j][k]+=a[i][k]*factor;b[j][k]+=b[i][k]*factor;
      }
    }
  }

  // copy b to a and return
  for(i=0;i<7;i++) for(j=0;j<7;j++) a[i][j]=b[i][j];
}

////////////////////////////////////////////////////////////////////////

//
// 3) Transformations (S.Schaller 18/01/01)
//    



int MoveToLocal(GroverVertexList *gvl) {
  int err = SUCCESS;
  for (GroverVertexList::iterator igvl = gvl->begin();
       igvl != gvl->end(); igvl ++) {
    if (igvl->GlobalToLocal() != SUCCESS) err = FAILURE;
  }
  return err;
}


int MoveToLocal(GroverParticleList *gpl) {
  int err = SUCCESS;
  for (GroverParticleList::iterator igpl = gpl->begin();
       igpl != gpl->end(); igpl ++) {
    if (igpl->GlobalToLocal() != SUCCESS) err = FAILURE;
  }
  return err;
}


int MoveToLocal(GroverWireList *gwl) {
  int err = SUCCESS;
  for (GroverWireList::iterator igwl = gwl->begin();
       igwl != gwl->end(); igwl ++) {
    if (igwl->GlobalToLocal() != SUCCESS) err = FAILURE;
  }
  return err;
}


int MoveToGlobal(GroverVertexList *gvl) {
  int err = SUCCESS;
  for (GroverVertexList::iterator igvl = gvl->begin();
       igvl != gvl->end(); igvl ++) {
    if (igvl->LocalToGlobal() != SUCCESS) err = FAILURE;
  }
  return err;
}


int MoveToGlobal(GroverParticleList *gpl) {
  int err = SUCCESS;
  for (GroverParticleList::iterator igpl = gpl->begin();
       igpl != gpl->end(); igpl ++) {
    if (igpl->LocalToGlobal() != SUCCESS) err = FAILURE;
  }
  return err;
}


int MoveToGlobal(GroverWireList *gwl) {
  int err = SUCCESS;
  for (GroverWireList::iterator igwl = gwl->begin();
       igwl != gwl->end(); igwl ++) {
    if (igwl->LocalToGlobal() != SUCCESS) err = FAILURE;
  }
  return err;
}
