#ifndef GROVER_WITHOUT_ARTE

#include <map>
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "grover/grover.hh"


//
// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
// 01-02-28  M.Sang   const safer
// 01-03-13  Max      Added preprocessor directives
//

ArtePointer<MTRA> MTRAWhichCaused(const ArtePointer<RSEG> rseg)
{
  //////////////////////////////////////////////////////////////
  //
  // Function to decide which MTRA caused a given CATS VDS segment.
  // Returns null pointer if not found, error, ambiguous, etc.
  //
  // I.Kisel (hacked by MS)
  // 24/11/00
  //
  //////////////////////////////////////////////////////////////

  // CATS VDS segment?
  if (rseg == 0) return 0;
  if (rseg->cmp != (1|Rsegc::bit16)) return 0; 
  

  // Define nicer names
  typedef  map<int,int> Hitmap;
  Hitmap hitmap;
  typedef  map<int,int>::iterator HitmapIt;


  ArtePointer<MTRA> mtra = 0;
  int nhits_mtra = 0;
  bool successful = false;
  int imtra = 0;


  const float threshold = 0.70; // fraction of correct hits required
  int hitsum = distance(rseg->hitb.begin(), rseg->hitb.end());


  // collect MIMPs from the RSEG
  for (ArteRelationIterator<HITB> hitbIt = rseg->hitb.begin();
       hitbIt != rseg->hitb.end(); ++hitbIt) {
    for (ArteRelationIterator<MIMP> mimpIt = (*hitbIt)->mimp.begin();
	 mimpIt != (*hitbIt)->mimp.end(); ++mimpIt) {
      imtra = (*mimpIt)->mtra.index();
      if (hitmap.find(imtra) == hitmap.end()) 
	hitmap.insert(pair<int,int>(imtra, 1));
      else (hitmap[imtra])++;
    }
  }


  // find MTRA
  for (HitmapIt pos = hitmap.begin(); pos != hitmap.end(); ++pos) {
    if (static_cast<float>(pos->second)/static_cast<float>(hitsum) >  threshold) {
      if (pos->second > nhits_mtra) {
	mtra = ArtePointer<MTRA>(pos->first);
	successful = true;
	nhits_mtra = pos->second;
      }
    }
  }


  if (successful) return mtra;
  else return 0;
}


#endif
