//
// Now part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
//
//

class DumpFormatter {

  ////////////////////////////////////////////////////////////////////////
  //
  // Class DumpFormatter
  // Author(s): Max Sang, MPI Muenchen (Max.Sang@cern.ch)
  //            Sebastian Slama, ETHZ (Sebastian@Slama.de)
  // Written: 10 March 2000 MS Basic structure
  // Updated: 13 March 2000 SS Added lots more DumpPrint()s
  //
  // Lets you dump the private members of your class to stdout in a 
  // controlled way. If users want to see the contents of the private 
  // members, they should be able to call YourClass->Dump();
  // 
  // YourClass::Dump() should look something like this:
  //
  //    void YourClass::Dump()
  //    {
  //     Util::DumpFormatter Nice;
  //     Nice->DumpClassName("YourClass")
  //     Nice->DumpComment("Class written by John Smith (smith@foobar.net)");
  //     Nice->DumpHeader();
  //     Nice->DumpPrint("x",&x);         // x is an integer
  //     Nice->DumpPrint("y",y, 200);     // y is an array of 200 floats
  //     Nice->DumpPrint("fred",&fred);   // fred is a Hep3Vector
  //     Nice->DumpPrint("jim",&jim,9,9); // jim is a 9*9 MyMatrix
  //     Nice->DumpFooter();
  //    } 
  //
  // It's so easy you don't have an excuse not to provide it ;-)
  //
  ////////////////////////////////////////////////////////////////////////

public:
   DumpFormatter();
  ~DumpFormatter();
  static int dump_verbosity;                 // default 1
  void DumpClassName(const char classname[]);      // For ID purposes
  void DumpComment(const char comment[]);          // Print out whatever you like
  void DumpHeader();                         // Print a simple header
  void DumpFooter();                         // Print a simple footer
  void DumpPrint(const char varname[], ... );      // DEFAULT (does nothing)
  void DumpPrint(const char varname[], const int *var);// Print an integer....
  void DumpPrint(const char varname[], const int var[], int size);//overloaded ad infinitum
  void DumpPrint(const char varname[], const long *var);
  void DumpPrint(const char varname[], const long var[], const int size);
  void DumpPrint(const char varname[], const unsigned long *var);
  void DumpPrint(const char varname[], const unsigned long var[], const int size);
  void DumpPrint(const char varname[], const unsigned int var[], const int size);
  void DumpPrint(const char varname[], const unsigned int *var);
  void DumpPrint(const char varname[], const long long *var);
  void DumpPrint(const char varname[], const unsigned long long *var);
  void DumpPrint(const char varname[], const long long var[], const int size);
  void DumpPrint(const char varname[], const unsigned long long var[], const int size);
  void DumpPrint(const char varname[], const short *var);
  void DumpPrint(const char varname[], const unsigned short *var);
  void DumpPrint(const char varname[], const short var[], const int size);
  void DumpPrint(const char varname[], const unsigned short var[], const int size);
  void DumpPrint(const char varname[], const float *var);
  void DumpPrint(const char varname[], const float var[], const int size);
  void DumpPrint(const char varname[], const double *var);
  void DumpPrint(const char varname[], const long double *var);
  void DumpPrint(const char varname[], const double var[], const int size);
  void DumpPrint(const char varname[], const long double var[], const int size);
  void DumpPrint(const char varname[], const char *var);
  void DumpPrint(const char varname[], unsigned const char *var);
  void DumpPrint(const char varname[], const char var[], const int size);
  void DumpPrint(const char varname[], unsigned const char var[], const int size);
  void DumpPrint(const char varname[], const bool *var);
  void DumpPrint(const char varname[], const bool var[], const int size);
  /*
  void DumpPrint(const char varname[], const complex<double> *var);
  void DumpPrint(const char varname[], const complex<long double> *var);
  void DumpPrint(const char varname[], const complex<float> *var);
  void DumpPrint(const char varname[], const Hep3Vector *vec);
  void DumpPrint(const char varname[], const HepLorentzVector *vec);
  void DumpPrint(const char varname[], const HepRotation *rot);
  void DumpPrint(const char varname[], const HepLorentzRotation *lrot);
  void DumpPrint(const char varname[], const HepPoint3D *pony);
  void DumpPrint(const char varname[], const HepVector3D *vecty);
  void DumpPrint(const char varname[], const HepPlane3D *ivan);
  void DumpPrint(const char varname[], const HepNormal3D *sven);
  */
private:
  int errorflag; // ??
};
