//--------------------------------------------------------------------
// Created: 26.07.2000 by   P.Conde, W.Hulsbergen & O.Igonkina 
// see http://www-hera-b.desy.de/subgroup/software/arte/MARPLE/   

/* IMPORTANT :
   The default algorithm       SvdRpntMatch.matchInTheMiddle();
   does not consider TX at all=> correspondent value of flag is not used
   Z_plane is specifying middle plane
   X is cut on Z_plane;
   The Y (TY) variable is cut at Z of RPNT;
   chi2 is calculated in the middle

   All constants were optimized on the MC data ( single inelastic ).
   The choice of constants could be steared from kumac with MARPLE/SOFT command

  MARPLE/SOFT         dx       dy     eff     ghostrate
    1	    soft:    12.2     18.7    98.9%       68%
    2     middle:     6.0      8.6    97.4%       34%
    3       hard:     2.3      4.0    91.8%       11%  (default)
    4   veryhard:     1.2      2.1    80.0%       4.7%
    5  superhard:     0.6      1.1    54.4%       2.2%
   The additional factor to multiply obtained from the triggered data as :
   dX1 = dX*sigma(dX_data)/sigma(dX_MC) and sigma(dY_data)/sigma(dY_MC)
*/
//--------------------------------------------------------------------

#include "marple/marple.hh"
#include "marple/RcSegRpntBuilder.hh"

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "arte/RSEG_cmp.hh"

#ifdef MARPLEPERFORMANCES
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
Chrono * chMatchSvdRpnt = 0;
#endif

//definition of constants for SVD-OT matching ( see \Ref{RcMatchConst})
// flag, z_plane, z_min, z_max, use_pair_cut, n_pair_left, n_pair_right, 
// use_delta_cut, delta_x, delta_y, delta_tx, delta_ty, delta_pe, chi2, 


extern RcMatchConst rcSvdOtConst;
extern RcMatchConst rcSoftSvdOtConst;

float maSRNormX = 0.492/0.44; // obtained from Jpsi selection-run15127/MC
float maSRNormY = 1.122/0.794;

//default constants
RcMatchConst rcSvdRpntConst = { 
  11000,     454.6,   -30.,  800.,      0,          1,            1,
  1, maSRNormX*2.3,  maSRNormY*4.,   0.01,      .02,   1.e+10,   100000.,  
};

// MARPLE/SOFT default constants
RcMatchConst rcSoftMiddleConst = { 
11000,     454.6,   -30.,  800.,      0,          1,            1,
    1, maSRNormX*12.2, maSRNormY*18.7,   0.05,      .05,   1.e+10,   1.e+10,  
};


RcMatchConst rcSoftMid2Const = { 
11000,     454.6,   -30.,  800.,      0,          1,            1,
    1, maSRNormX*6., maSRNormY*8.6,      0.05,      .05,   1.e+10,   1.e+10,  
};

RcMatchConst rcSoftMid4Const = { 
11000,     454.6,   -30.,  800.,      0,          1,            1,
    1, maSRNormX*1.2, maSRNormY*2.1,      0.05,      .05,   1.e+10,   1.e+10,  
};

RcMatchConst rcSoftMid5Const = { 
11000,     454.6,   -30.,  800.,      0,          1,            1,
    1, maSRNormX*.6, maSRNormY*1.1,      0.05,      .05,   1.e+10,   1.e+10,  
};






extern "C"  
void FORTRAN_ROUTINE(matchsvdpc)() 
{

# ifdef MARPLEPERFORMANCES
  if(chMatchSvdRpnt == 0){
    ChronoManager CM;
    chMatchSvdRpnt = CM.createChrono("Svd-Rpnt match");
  }
  chMatchSvdRpnt->start();
# endif
  
  RcSegRpntBuilder SvdRpntMatch;
  maPresentMatch = MARPLEIDVDSPC;

  //--------------------------------------------------------------------
  // put constants to the MatchBuilder
  //--------------------------------------------------------------------
 if( MARPLE_SOFT )
   {
     if(       MARPLE_SOFT == 1 ){
       SvdRpntMatch.setConst(rcSoftMiddleConst);
     }else if( MARPLE_SOFT == 2 ){
       SvdRpntMatch.setConst(rcSoftMid2Const);
     }else if( MARPLE_SOFT == 3 ){
       SvdRpntMatch.setConst(rcSvdRpntConst);
     }else if( MARPLE_SOFT == 4 ){
       SvdRpntMatch.setConst(rcSoftMid4Const);
     }else if( MARPLE_SOFT == 5 ){
       SvdRpntMatch.setConst(rcSoftMid5Const);
     }else{
       SvdRpntMatch.setConst(rcSoftMiddleConst);
     }
   }else{
     SvdRpntMatch.setConst(rcSvdRpntConst);
   }

  //--------------------------------------------------------------------
  // create vector of SVD segments
  //--------------------------------------------------------------------
  
  int iErr;
  for( ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
        irseg != ArteTable<RSEG>::end(); ++irseg )
    {
      if ( (*irseg)->fit != 0 ) continue;
      if( ((*irseg)->cmp & Rsegc::bitsign) != 0 ) continue;
      if( (*irseg)->cmp != Rsegc::vxd && 
	  (*irseg)->cmp != 65537) continue;

      RcSegPoint * ptToPoint = new RcSegPoint( (*irseg) , 1 );
      SvdRpntMatch.addPointLeft( *ptToPoint);
      ptToPoint = 0;
	
    }

  // no candidate found --- no reason go through RPNT segments
  if( SvdRpntMatch.vectorLeft().size() == 0 ) 
    {
# ifdef MARPLEPERFORMANCES
      chMatchSvdRpnt->stop();
# endif
      maPresentMatch = MARPLENOMATCH;
      return;
    }
  //--------------------------------------------------------------------
  // create vector of PC-RPNT Outer Tracker segments 
  //--------------------------------------------------------------------


  for( ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
        irseg != ArteTable<RSEG>::end(); ++irseg )
    {
      if ( (*irseg)->fit != 0 ) continue;
      if( ((*irseg)->cmp & Rsegc::bitsign) != 0 ) continue;
      if(  !((*irseg)->cmp & Rsegc::patt)  ) continue;
      
      ArtePointer<RPNT> rpntTaken;
      for( ArteRelationIterator<RPNT> irpnt = (*irseg)->rpnt.begin();
        irpnt != (*irseg)->rpnt.end(); ++irpnt )
	{
	  if( (*irpnt)->zf< 680 || (*irpnt)->zf>860 ) continue; 
	  if( !rpntTaken.index() || (*irpnt)->zf < rpntTaken->zf )
	    rpntTaken = *irpnt;
	}
      if( !rpntTaken.index() )
	{
          static int idMesg = 2734542;
	  arteMsg << "MATCH SVD PC : No RPNT belongs to the RSEG " 
	       << *irseg << endl;
          arteMsg.allp(idMesg,10);
	  continue;
	}

      iErr = 0;
      RcRpntPoint * ptToPoint = new RcRpntPoint( rpntTaken); 
      SvdRpntMatch.addPointRight( *ptToPoint);
      ptToPoint = 0;	      

    }  
  
  //--------------------------------------------------------------------
  // create a list of matching points with chi2 
  //--------------------------------------------------------------------

  SvdRpntMatch.matchInTheMiddle();

  //--------------------------------------------------------------------
  // fill RPNT estimated momentum to matchPoint
  //--------------------------------------------------------------------
  // leave the momentum from the match instead
  /*
  unsigned int nCombinations = SvdRpntMatch.matchList().size();
  for( unsigned int iMatch = 0;  iMatch<nCombinations; ++iMatch)
    {
      RcPoint * rpnt = SvdRpntMatch.rightOrigin(iMatch);
      SvdRpntMatch.matchList()[iMatch]->setpe(  rpnt->pe() );
      SvdRpntMatch.matchList()[iMatch]->setcov(5,5,rpnt->cov()(5,5));	  
    }      
  */
  //--------------------------------------------------------------------
  // print result of matching
  //--------------------------------------------------------------------

  SvdRpntMatch.stat();
  
  if( MARPLE_DEBUG >= 20)
    {  
      SvdRpntMatch.fillDeltas(8213,8313);
      SvdRpntMatch.fillMatch( 8113);
    }else if( MARPLE_DEBUG >= 10){
      SvdRpntMatch.fillDeltas(8213);
    }
  //--------------------------------------------------------------------
  // save found relation to the table
  //--------------------------------------------------------------------
  if( MARPLE_MAKERSEG ) SvdRpntMatch.saveRSEG(iErr);
  
  SvdRpntMatch.saveRelation(iErr);
  if( iErr )
    { arteMsg << "MARPLE: some relations were not saved "; arteMsg.allp();}
  
  //--------------------------------------------------------------------
  // clean everything
  //--------------------------------------------------------------------
  SvdRpntMatch.deleteAll();  

  maPresentMatch = MARPLENOMATCH;

# ifdef MARPLEPERFORMANCES
  chMatchSvdRpnt->stop();
# endif
}

