
// Boris Fominykh  22.12.1999

// write the muon detector geometry from ARTE into Data Base
// program operates only in ARTE frame
// classes MuDbNede and MuDbNesl are used for communication with Mizzi DB
// classes MuNede and MuNesl are the containers for NEDE, NESL elements

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>

#include "murec/MuDbNesl.h"
#include "murec/MuDbNede.h"

#include "arte/arte.hh"
#include "arte/artefunc.h"
#include "arte/ArteMessage.hh"

extern "C" {
int MuNameDecode(char*,int*,int*,int*,int*,int*,int*);

int MuDbGeoWrite(int RUN, char* DBname) {
//*************************************************//

  ofstream fout("MuDbGeoWrite.out");
  if (!fout) 
  {
    arteMsg << "MuDbGeoWrite: can not open the output file MuDbGeoWrite.out" << endl;
    arteMsg.warn();
    return 1;
  }
  int ret;

  MuNesl   lnesl;  
  MuDbNesl Nesl;  
  Nesl.setDBname(DBname);
  Nesl.setKTstatus(-2);
  Nesl.setRun(RUN);
  Nesl.setMinor(0);

  MuNede   lnede;  
  MuDbNede Nede;  
  Nede.setDBname(DBname);
  Nede.setKTstatus(-2);
  Nede.setRun(RUN);
  Nede.setMinor(0);

// loops over NESL in ARTE
  int inum1 = 0;
  int inum2 = 0;
  int Nnede = 0;
  char memo[9];
  for ( ArteTableIterator<NESL> isl = ArteTable<NESL>::begin(); 
        isl != ArteTable<NESL>::end();
        ++isl ) {
    inum1 += 1;
    int   cmp  = (*isl)->cmp;
    if(cmp == 8 || cmp == 9) {
      inum2 += 1;
      int iv1 = (*isl)->fpl;
      int iv2 = iv1 + (*isl)->npl;
// muon superlayer
      int msl   = (*isl)->nsl;
      int nslkey = 10*msl+cmp;
// superlayer name
      char namsl[5] = "    ";
      strncpy(namsl,(*isl)->name,4);
      namsl[4]='\0';

      fout << inum2 << " inesl " << inum1 << " key " << nslkey;
      fout << " " << namsl << endl;

      int laym=0;
      int nxmem[2][3];
      int nx99m[2][3];
      for(int i=0; i<2; i++) {
        for(int j=0; j<3; j++) {
          nxmem[i][j]=0;
          nx99m[i][j]=0;
        }
      }

// loops over NEDE elements in ARTE
      if(iv2 >= iv1) {
        for ( int iv=iv1; iv<iv2; iv++) { 
          ArtePointer<NEDE> nede(iv);
          Nnede++;
          int NedeCmp  = nede->cmp;
          if(NedeCmp != cmp) {
            arteMsg << "MuDbGeoWrite: NedeCmp " << NedeCmp << " != cmp= " << cmp << endl;
            arteMsg.warn();
            return 2;
          }
// layer - lay
          int  lay  = nede->lay;
// 
          int type=0;
          int msln=0;
          int layn=0;
          int nx=0;
          int ud=0;
          int nc=0;
          int nedekey = 0;
          char namde[9];
// the case of tube and pad chambers
          if(cmp == 8) {
// chamber name
            strncpy(namde,nede->hwc,9);
            namde[8]='\0';
// decode the chamber number
            ret=MuNameDecode(namde,&type,&msln,&layn,&ud,&nx,&nc);
            if(ret != 0 || msln != msl || layn != lay) {
              arteMsg << "MuDbGeoWrite: NOT correct name " << namde;
              arteMsg << " msl " << msl << " lay " << lay << endl;
              arteMsg.warn();
              return 3;
            }
// superlayer parameters
            if(lay > laym) laym=lay;
            if(nx == 99) nx99m[ud][lay-1]=1;
            if(nx != 99 && nx > nxmem[ud][lay-1]) nxmem[ud][lay-1]=nx;
// side
            int sd=0;
            if(!strcmp(namde,memo)) sd = 1;
            strncpy(memo,namde,9);
// key word
            nedekey = 100000*msl+10000*lay+1000*ud+10*nx+sd;

	    fout << msl << "-" << lay << " : ";
	    fout << namsl << " - " << namde << " key " << nedekey;
	    fout << " msl " << msl << " lay " << lay << " ud " << ud;
	    fout << " nx " << nx << " sd " << sd << endl;

          }
// the case of pixel chambers
          if(cmp == 9) {
// chamber name
            strncpy(namde,nede->hwc,6);
            namde[5]=' ';
            namde[6]=' ';
            namde[7]=' ';
            namde[8]='\0';
// decode the chamber number
            int qu = 0;
            ret=MuNameDecode(namde,&type,&msln,&layn,&ud,&qu,&nc);
            if(ret != 0 || msln != msl) {
              arteMsg << "MuDbGeoWrite: NOT correct pixel name " << namde;
              arteMsg << " msl " << msl << " qu " << qu << endl;
              arteMsg.warn();
              return 4;
            }
// superlayer parameters
            if(qu > laym) laym=qu+1;
// key word
            nedekey = 1000*msl+100*qu+cmp;

	    fout << msl << "-" << lay << " : ";
	    fout << namsl << " - " << namde << " key " << nedekey;
	    fout << " msl " << msl << " lay " << lay << " qu " << qu;
	    fout << endl;

          }

// fill the NEDE container
          lnede.nedekey=nedekey;
          lnede.neslkey=nslkey;
          lnede.inede=iv;

          strncpy(lnede.hwc,namde,9);
          char gvolw[5];
          strncpy(gvolw,nede->gvol,4);
          gvolw[4]='\0';
          strncpy(lnede.gvol,gvolw,5);

          for(int i=0;i<9;i++) lnede.rot[i]=nede->rot[i];
          for(int i=0;i<4;i++) {
            lnede.xfw[i]=nede->xfw[i];
            lnede.zfw[i]=nede->zfw[i];
            lnede.nwi[i]=nede->nwi[i];
            lnede.y1w[i]=nede->y1w[i];
            lnede.y2w[i]=nede->y2w[i];
          }

          lnede.xmin=nede->xmin;
          lnede.xmax=nede->xmax;
          lnede.ymin=nede->ymin;
          lnede.ymax=nede->ymax;
          lnede.zmin=nede->zmin;
          lnede.zmax=nede->zmax;
        
          lnede.sxmn=nede->sxmn;
          lnede.sxmx=nede->sxmx;
          lnede.symn=nede->symn;
          lnede.symx=nede->symx;
       
          lnede.acc=nede->acc;
          lnede.eff=nede->eff;
          lnede.dod=nede->dod;
          lnede.pit=nede->pit;
          lnede.ms=nede->ms;
          lnede.cos=nede->cos;
          lnede.sin=nede->sin;
  
          lnede.nsl=nede->nsl;
          lnede.lay=nede->lay;
          lnede.sec=nede->sec;
          lnede.shp=nede->shp;
          lnede.cmp=nede->cmp;
          lnede.nwl=nede->nwl;
          lnede.fwl=nede->fwl;
          lnede.gcop=nede->gcop;
	  lnede.nesl=inum1;
          lnede.dedx=nede->dedx;
          lnede.xmad=nede->xmad;
          lnede.ymad=nede->ymad;
          lnede.zmad=nede->zmad;
          lnede.noise=nede->noise;
          lnede.ares=nede->ares;

// add the NEDE container to temporary space
          ret=Nede.Add(&lnede);
          if(ret != 0) {
            arteMsg << "MuDbGeoWrite: can NOT add NEDE, ret= " << ret << endl;
            arteMsg.warn();
            return 5;
          }
        }
      }

      fout << "key " << nslkey << " laym " << laym << endl;
      if(cmp == 8 && laym > 0) {
        for(int l=0; l<laym; l++) {
          fout << "lay " << l << " nx " << nxmem[0][l] << " " << nxmem[1][l];
          fout << " nx99 " << nx99m[0][l] << " " << nx99m[1][l] << endl;
        }
      }

// fill the NESL container
      lnesl.neslkey = nslkey;
      lnesl.inesl   = inum1;
      lnesl.cmp=(*isl)->cmp;
      lnesl.shp=(*isl)->shp;
      lnesl.nsl=(*isl)->nsl;
      lnesl.mfl=(*isl)->mfl;
      lnesl.x1=(*isl)->x1;
      lnesl.x2=(*isl)->x2;
      lnesl.y1=(*isl)->y1;
      lnesl.y2=(*isl)->y2;
      lnesl.z1=(*isl)->z1;
      lnesl.z2=(*isl)->z2;
      lnesl.dx=(*isl)->dx;
      lnesl.dy=(*isl)->dy;
      lnesl.npl=(*isl)->npl;
      lnesl.fpl=(*isl)->fpl;
      lnesl.nwl=(*isl)->nwl;
      lnesl.fwl=(*isl)->fwl;
      lnesl.ltyp=(*isl)->ltyp;
      lnesl.lay=laym;
// superlayer name
      strncpy(&(lnesl.name[0]),namsl,5);
// chamber structure
      for(int l=0; l<3; l++) {
        for(int i=0; i<2; i++) {
          if(l<laym) {
            lnesl.chamb[2*l+i]=nxmem[i][l];
            lnesl.nx99[2*l+i]=nx99m[i][l];
          }
          else {
            lnesl.chamb[2*l+i]=0;
            lnesl.nx99[2*l+i]=0;
          }
        }
      }
// add the NESL container to temporary space
      ret=Nesl.Add(&lnesl);
      if(ret != 0) {
        arteMsg << "MuDbGeoWrite: can NOT add NESL, ret= " << ret << endl;
        arteMsg.warn();
        return 6;
      }
    }
  }
// write NESL, NEDE to Data Base
  ret=Nesl.Write();
  if(ret == 0) ret=Nede.Write();
  if(ret != 0) {
    arteMsg << "MuDbGeoWrite: can NOT write NESL, NEDE to DB ";
    arteMsg << DBname << endl;
    arteMsg.warn();
    return 7;
  }
  return 0; 
}
}
