//
// Returns vertical coordinates of track from SLT refit
//
// Author: Alexander Lanyov              August 2000
//
#include <stdio.h>
#include <math.h>

#include <arte/arte.hh>

#include "farm/farm.hh"

extern "C" {
#include "sltinfo.h"
}

extern "C" void dipro3_(float *x, float *y, float *z,
			float *xwin, float *ywin, float *zwin);

extern "C" void disltcut_(int *trigger0, int *track0,
			  float *zc1, float *zc2,
			  float *cutval1, float *cutval2) {

  *zc1 = 0.;
  *zc2 = 1000.;
  *cutval1 = 0.;
  *cutval2 = 0.;

  //------------------------------------------------------ Basic SLT info
  int daqcode=faDDAQFCS_tcode();
  if (daqcode==9) return; /* No SLT information in non triggered events */
  if (SLTReadDSLT()!=SLT_OK) return;
  
  SLT_STATUS stat;
  SLT_TRIGGER trig;
  SLT_TRACK track;
  SLT_SEEDTRACK seed;
  SLT_SILITRACK sili;
  SLT_REFITTRACK refit;
  SLT_SILITRACKCOV cov;
  if (SLTStatus(&stat)!=SLT_OK) return;
  if (!(stat.status&SLTEVENT_ACCEPT)) return;

  for(int trigger=*trigger0;trigger<stat.ntriggers;trigger++) {
    if (SLTTrigInfo(trigger,&trig)!=SLT_OK) continue;
    if (trig.status==SLTTRIG_REJECT) continue;

    for(int tra=*track0;tra<trig.ntracks;tra++) {
      if (SLTTrackInfo(trig.trackindex[tra],&track)!=SLT_OK) continue;
      if(track.status==SLTTRACK_ERROR) continue;
      if (SLTSeedTrackInfo(trig.trackindex[tra],&seed)!=SLT_OK) continue;
      if(track.status!=SLTTRACK_ACCEPT) continue;
      if (SLTRefitTrackInfo(trig.trackindex[tra],&refit)!=SLT_OK) continue;
      if (SLTSiliTrackInfo(trig.trackindex[tra],&sili)!=SLT_OK) continue;
      if (SLTSiliTrackCovInfo(trig.trackindex[tra],&cov)!=SLT_OK) continue;

      /*
      *zc1 = seed.z1;
      *zc2 = seed.z2;
      *cutval1 = refit.by + refit.ty*(*zc1);
      *cutval2 = refit.by + refit.ty*(*zc2);
      */

      float z = seed.z1;
      float x = refit.bx + refit.tx*z;
      float y = refit.by + refit.ty*z;
      float xwin,ywin;
      dipro3_(&x,&y,&z, &xwin,&ywin,cutval1);
      *zc1 = z;

      z = seed.z2;
      x = refit.bx + refit.tx*z;
      y = refit.by + refit.ty*z;
      dipro3_(&x,&y,&z, &xwin,&ywin,cutval2);
      *zc2 = z;

      printf("slttrack: found trigger %d, track %d ==> "
	     "Y=%f at Z=%f, Y=%f at Z=%f\n",
	     *trigger0, *track0, *zc1, *cutval1, *zc2, *cutval2);
      return;
    }
  }
  return;
}
