/* see HPT_cable_maker_based_on_horizontal_rows.hh for details */

// Author V.Balagura (balagura@mail.desy.de)   (21.02.99).

#include "ptset/HPT_cable_maker_based_on_horizontal_rows.hh"
#include "ptset/global_error.hh"
#include <strstream.h>
#include "ptset/index_range.hh"
#include "ptset/skip_range.hh"

void HPT_CableMakerBasedOnHorizontalRows::operator
()(istream* is, HPT_Setup::IChamber ichamber) {
  //  cout << "HPT_CableMakerBasedOnHorizontalRows(istream* is) started\n";
  strstream line;
  char c;
  while ( is->get(c) && c != '\n' ) line.put(c);
  
  string cable_name;
  if ( !(line >> cable_name) ) {
    global_error()("can not read cable name");
    return;
  }
  Cable<HPT_Setup::IChamber>& cable = cables[cable_name];
  //  cout << "cable name: " << cable_name << endl;

  HPT_Chamber& chamber = (*ichamber).second;
  cable.ichamber = ichamber;
  //  cout << "chamber name: " << chamber_name << endl;
  
  // horizontal rows are determined "lazily", i.e. on demand
  // (not all at once)
  HorizontalRowsIn_HPT_Setup::iterator irow = rows.find(ichamber);
  if ( irow == rows.end() ) {
    irow = (rows.insert
	    (make_pair(ichamber, 
		       HorizontalRowsIn_HPT_Chamber()))).first;
    (*irow).second.create((*ichamber).second);
  }
  HorizontalRowsIn_HPT_Chamber& rows_in_chamber = (*irow).second;
  
  
  string rest;
  while ( line.get(c) ) rest += c;
  //  cout << "rest of line: " << rest << endl;
  fill(&rest, cable, rows_in_chamber, chamber.end(), true);
  // input should contain at least one index range (or skip);
  // thus true here means that the input is required for the 1st
  // call

  // the communication with the fill() is done using string "rest"
  if ( global_error() ) return;
  
  // now read input till the end
  while ( rest.size() != 0 ) {
    fill(&rest, cable, rows_in_chamber, chamber.end(), false);
    // false here means: if there is no input left => do not set 
    // global_error() /some input was accepted during previous call(s)/
    if ( global_error() ) return;
  }
}
// made as a separate function since 
void HPT_CableMakerBasedOnHorizontalRows::fill
(string* rest, Cable<HPT_Setup::IChamber>& cable,
 HorizontalRowsIn_HPT_Chamber& rows_in_chamber,
 HPT_Chamber::ISection dummy_is, bool is_first) {

  //  cout << "HPT_CableMakerBasedOnHorizontalRows::fill is called\n";
    strstream attempt1, attempt2;
    attempt1 << *rest; attempt2 << *rest;
  // attempt1 is read into  IndexRange, attempt2 -> into SkipRange
  *rest = "";
  
  IndexRange_X ir;
  char c;
  
  if ( !(attempt1 >> ir) ) { // is it possible to read IndexRange?
    SkipRange sr;
    if ( !(attempt2 >> sr) ) {
      if ( is_first ) global_error()("can not read cell range");
      // bool is_first flag marks the 1st call to fill() when the input 
      // should not be empty 
      return;
    }
    else { // SkipRange has been read
      HPT_Setup::ISection_Cell dummy;
      dummy.isection = dummy_is;
      for (unsigned int i=0; i<sr.n; ++i) cable.push_back(dummy);
      attempt2 >> ws; // do not call fill() one more time if input
      // contains only white space
      while ( attempt2.get(c) ) *rest += c; // the rest of input
      // is returned as the string
    }
  }
  else {  // IndexRange has been read
    Index ind;
    ind.iy = ir.iy;
    map<Index, HPT_Chamber::ISection_Cell, less<Index> >::iterator ic;
    for (ind.ix=ir.ix0; ind.ix<ir.ix1; ++ind.ix) {
      if ( (ic = rows_in_chamber.cell.find(ind)) == 
	   rows_in_chamber.cell.end() ) {
	string tmp;
	tmp << "there is no cell #" << ind.ix << " in the row #" 
	    << ind.iy;
	global_error()(tmp);
	return;
      }
      cable.push_back((*ic).second);
      attempt1 >> ws;
      while ( attempt1.get(c) ) *rest += c;
    }	
  }
}
