// 
//  Author:  Rainer Mankel                     Created:  07-Mar-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// class rangerDomObj
// basic ranger domain object

#include "rang/DomObj.hh"
#include "rang/SeedTableau.hh"
#include "rang/HL.hh"
#include "rang/XPro.hh"
#include "rang/ExclusionRange.hh"
#include "rang/DomWin.hh"
#include "rang/DomLay.hh"
#include "rang/LayPl.hh"


rangerDomObj::rangerDomObj() {
  updateLevel = 0;
  xPro = 0;
  xPro = new rangerXPro();
  if (! xPro) {
    cout << "rangerDomObj::rangerDomObj(): xpro allocation failed" 
	 << endl;
  }
}


rangerDomObj::~rangerDomObj() {

  for (vector<rangerDomain*>::iterator it=defaults.begin();
       it != defaults.end(); ++it) {
    delete *it;
  }

  for (vector<rangerDomain*>::iterator it=domains.begin();
       it != domains.end(); ++it) {
    delete *it;
  }
  delete xPro;
}


rangerHL* rangerDomObj::makeHL(rangerDomLay* dl) {

  if (!dl) return 0;
  rangerHL* hl = new rangerHL(dl);
  if (hl) {
    hlists.push_back(hl);
  }
  return hl;
}


vector<rangerDomain*>::iterator rangerDomObj::findDom(string dName){

  for (vector<rangerDomain*>::iterator di=domains.begin();
       di != domains.end(); ++di) {
    if ((*di)->nam() == dName) return di;
  }
  return domains.end();
}


int rangerDomObj::nSeedSets() {
  int n = 0;
  for (vector<rangerSeedTableau*>::const_iterator sti=seedTabs.begin();
       sti != seedTabs.end(); ++sti) {
    n += (*sti)->seedSets().size();
  }
  return n;
}


int rangerDomObj::addExclusionRange(ArtePointer<GEDE> ged,
				    double umin,double umax,
				    int subgl1,int subgl2) {
  int nAdded = 0;
  for (vector<rangerDomain*>::iterator di=domains.begin();
       di != domains.end(); ++di) {
    for (vector <rangerDomWin*>::iterator wIt=(*di)->win().begin();
	 wIt != (*di)->win().end(); ++wIt) {
      for (vector<rangerDomLay*>::iterator dli=(*wIt)->dls().begin();
	   dli != (*wIt)->dls().end(); ++dli) {
	for (vector<rangerLayPl*>::iterator li=(*dli)->setLPs().begin();
	     li != (*dli)->setLPs().end(); ++li) {
	  if ((*li)->ge() == ged) {  // check correct gede and sublayer range
	    if ((*li)->subgl1() <= subgl2 && (*li)->subgl2() >= subgl1) {
	      nAdded += (*li)->excludeRange(umin,umax,subgl1,subgl2);
	    }
	  }
	}
      }
    }
  }
  return nAdded;
}

int rangerDomObj::clearExclusionRanges() {
  int nDel = 0;
  for (vector<rangerDomain*>::iterator di=domains.begin();
       di != domains.end(); ++di) {
    for (vector <rangerDomWin*>::iterator wIt=(*di)->win().begin();
	 wIt != (*di)->win().end(); ++wIt) {
      for (vector<rangerDomLay*>::iterator dli=(*wIt)->dls().begin();
	   dli != (*wIt)->dls().end(); ++dli) {
	for (vector<rangerLayPl*>::iterator li=(*dli)->setLPs().begin();
	     li != (*dli)->setLPs().end(); ++li) {
	  nDel += (*li)->clearExclusionRanges();
	}
      }
    }
  }
  return nDel;
}


void rangerDomObj::dump(const int level){
  cout << "================== Domain Object Dump ========================"
       << endl
       << "     defaults " << defaults.size()
       << "  domains " << domains.size() << " updateLevel " << updateLevel
       << " seeding tableaus " << seedTabs.size()
       << endl;
  if (level > 0) {
    for (vector<rangerDomain*>::iterator di=domains.begin();
	 di != domains.end(); ++di) {
      (*di)->dump(level-1);
    }

    for (vector<rangerSeedTableau*>::iterator sti=seedTabs.begin();
	 sti != seedTabs.end(); ++sti) {
      (*sti)->dump(level-1);
    }
  }
}
