// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// Modified: 
//   new rangerFitL((ArtePointer<RTRA> rtra,..)   26/07/00   A.Spiridonov 
//   call of t.fitGlobal()                        06/08/00   A. Spiridonov
//   new int rangerFit(ArtePointer<RTRA> rtra,    23/05/01   W. Hulsbergen
//	      rangerParticleType pType,
//	      bool addecal, bool addrich) 
//
//
#include <iostream.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "rang/Node.hh"
#include "rang/Track.hh"
#include "rang/Fit.hh"
#include "rang/zfpar.hh"
#include "rang/hfill.hh"
#include "rang/FitMode.hh"
#include "rang/RMap.hh"
#include "rang/Geom.hh"
#include "rang/FLAGHist.hh"
#include "rang/rginit.hh"

int rangerFit(ArtePointer<RSEG> rseg,
	      rangerParticleType pType) {

  rginit_();
  rangerGeom::condUpdate();    // initialization

  rangerRMap rmap(rseg);                  // create base of R table mapping
  rangerTrack t(rseg,ModeP,pType,&rmap);  // create track from RSEG

  int stat = t.fit();

  if (stat < 0) {
    //    rseg->fit = 10;
    rseg->err = 10;
    return -10;
  }

#ifdef rangerHIST
  int id=6;
  float rad = t.totalRadThickness();
  float y = 0;
  float w=1;
  hfill_(id,rad,y,w);
#endif

  t.storeback(&rmap);

  if (rseg->cmp == 0) rseg->cmp = t.genCmp();

#ifdef LISTHITS
  for (ArteTableIterator<GEPT> geptIt; geptIt.valid(); geptIt.next()){
    if (geptIt->hit1 == 0) {
      if (geptIt->mim1 > 0) {
	cout << "plane " << geptIt << " has no hits but mimps " << endl;
      }
    }
  }
#endif

  return 0;
}



int rangerFit(vector<ArtePointer<RSEG> > rsegVec,
	      rangerParticleType pType) {

  rginit_();
  rangerGeom::condUpdate();    // initialization

  vector<rangerRMap> rmapVec;
  rangerTrack t(rsegVec,ModeP,pType,&rmapVec);  // create track from RSEG

  int stat = t.fit();

  if (stat < 0) return stat;

#ifdef rangerHIST
  int id=6;
  float rad = t.totalRadThickness();
  float y = 0;
  float w=1;
  hfill_(id,rad,y,w);
#endif

  t.storeback(&rmapVec);
#ifdef LISTHITS
  for (ArteTableIterator<GEPT> geptIt; geptIt.valid(); geptIt.next()){
    if (geptIt->hit1 == 0) {
      if (geptIt->mim1 > 0) {
	cout << "plane " << geptIt << " has no hits but mimps " << endl;
      }
    }
  }
#endif
  return 0;
}


int rangerFit(ArtePointer<RTRA> rtra,
              rangerParticleType pType) {

  rginit_();
  rangerGeom::condUpdate();    // initialization

  rangerRMap rmap(rtra);                  // create base of R table mapping
  rangerTrack t(rtra,ModeP,pType,&rmap);  // create track from RTRA

  int stat = t.fit();

  if (stat < 0) {
    //    rtra->fit = 10;
    rtra->fit = 100;
    return stat;
  }

  t.storeback(&rmap);
  return 0;
}


int rangerFit(ArtePointer<RTRA> rtra,
	      rangerParticleType pType,
	      bool addecal, bool addrich) {

  rginit_();
  rangerGeom::condUpdate();    // initialization

  rangerRMap rmap(rtra);                  // create base of R table mapping
  rangerTrack t(rtra,ModeP,pType,&rmap);  // create track from RTRA

  // add first ecal cluster if it exists
  bool modified=false ;
  if(addecal && rtra->rccl.begin() != rtra->rccl.end()) {
    ArtePointer<RCCL> clus=*(rtra->rccl.begin()) ;
    int charge = rtra->pf < 0 ? -1 : 1 ;
    rangerNode* w = rangerNode::makeNode( clus, charge, t.fitMode() );
    if(w) {
      t.prepend(*w);
      modified=true ;
    }
  }
  // add first rich segment if it exists
  if(addrich) {
    ArtePointer<RSEG> richseg ;
    for(ArteRelationIterator<RSEG> iseg = rtra->rseg.begin() ;
	iseg != rtra->rseg.end() && !richseg.index(); iseg++)
      if( ((*iseg)->fit==8 || (*iseg)->fit==10 || (*iseg)->fit==12 || (*iseg)->fit==13) &&
	  ( (*iseg)->cmp & Rsegc::rich ) ) richseg = *iseg ;
    if( richseg.index()>0 ) {
      rangerNode* w = rangerNode::makeNode(richseg,t.fitMode(),0) ;
      if(w) {
	t.prepend(*w);
	modified=true ;
      }
    }
  }    
  // re-order nodes if necessary
  if(modified) {
    // order
    t.orderIfNeeded() ;
    // this one is necesarry to get a good starting point for vds-ecal
    // refits. without this call the fit does not converge. (it is
    // very fast anyway.)
    t.filterDownStr() ;
  }
    
  int stat = t.fit();

  if (stat < 0) {
    //    rtra->fit = 10;
    rtra->fit = 100;
    return stat;
  }

  t.storeback(&rmap);
  return 0;
}

int rangerFitV(ArtePointer<RSEG> rseg, const int method,
	      rangerParticleType pType) {

  rginit_();
  rangerGeom::condUpdate();    // initialization

  rangerRMap rmap(rseg);                  // create base of R table mapping
  rangerTrack t(rseg,ModeL,pType,&rmap);  // create track from RSEG
  
  int stat = 0;
  switch(method){
  case 0:
    stat = t.fitV();
    break;
  case 1:
    stat = t.fitVf();
    break;
  case 2:
    stat = t.fitVs();
    break;
  default:
    cout << "Fit.C : rangerFitV : invalid method=" << method << endl;
    stat = -999;
    break;
  }

  if (stat < 0) {
    rseg->err = 10;
    //    rseg->fit = 10;
    return -10;
  }

  t.storeback(&rmap);

  if(method == 1) {

    /*
    float  c_x  = 1.;
    float  c_y  = 1.;
    float  c_tx = 1.;
    float  c_ty = 1.;
    */

    float  c_x  = 1.241;
    float  c_y  = 1.242;
    float  c_tx = 2.342;
    float  c_ty = 2.342;
    rseg->cvf[0] *= c_x*c_x; 
    rseg->cvf[1] *= c_x*c_y; 
    rseg->cvf[2] *= c_y*c_y; 
    rseg->cvf[3] *= c_x*c_tx; 
    rseg->cvf[4] *= c_y*c_tx; 
    rseg->cvf[5] *= c_tx*c_tx; 
    rseg->cvf[6] *= c_x*c_ty; 
    rseg->cvf[7] *= c_y*c_ty; 
    rseg->cvf[8] *= c_tx*c_ty; 
    rseg->cvf[9] *= (c_ty*c_ty); 
  }
  return 0;
}


// clone of rangerFit but linear model

int rangerFitL(ArtePointer<RTRA> rtra,
	      rangerParticleType pType) {

  rginit_();
  rangerGeom::condUpdate();    // initialization

  rangerRMap rmap(rtra);                  // create base of R table mapping
  rangerTrack t(rtra,ModeL,pType,&rmap);  // create track from RTRA

  //int stat = t.fit();
  int stat;
  if(zfpar_.slinemode) 
    {stat = t.fitGlobal();}
  else 
    {stat = t.fit(0,1,0);}

  if (stat < 0) {
    //    rtra->fit = 10;
    rtra->fit = 100;
    return stat;
  }

  t.storeback(&rmap);
  return 0;
}

int rangerFitL(ArtePointer<RSEG> rseg,
	      rangerParticleType pType) {
  rginit_();
  rangerGeom::condUpdate();    // initialization

  rangerRMap rmap(rseg);                  // create base of R table mapping
  rangerTrack t(rseg,ModeL,pType,&rmap);  // create track from RSEG

  //  int stat = t.fit();
  int stat;
  if(zfpar_.slinemode) 
    {stat = t.fitGlobal();}
  else 
    {stat = t.fit(0,1,0);}

  if (stat < 0) {
    //    rseg->fit = 10;
    rseg->err = 10;
    return -10;
  }

#ifdef rangerHIST
  int id=6;
  float rad = t.totalRadThickness();
  float y = 0;
  float w=1;
  hfill_(id,rad,y,w);
#endif

  t.storeback(&rmap);

  if (rseg->cmp == 0) rseg->cmp = t.genCmp();

#ifdef LISTHITS
  for (ArteTableIterator<GEPT> geptIt; geptIt.valid(); geptIt.next()){
    if (geptIt->hit1 == 0) {
      if (geptIt->mim1 > 0) {
	cout << "plane " << geptIt << " has no hits but mimps " << endl;
      }
    }
  }
#endif

  return 0;
}


int rangerFitL(vector<ArtePointer<RSEG> > rsegVec,
	      rangerParticleType pType) {

  rginit_();
  rangerGeom::condUpdate();    // initialization

  vector<rangerRMap> rmapVec;
  rangerTrack t(rsegVec,ModeL,pType,&rmapVec);  // create track from RSEG

  //  int stat = t.fit();
  int stat;
  if(zfpar_.slinemode) 
    {stat = t.fitGlobal();}
  else 
    {stat = t.fit(0,1,0);}

  if (stat < 0) return stat;

#ifdef rangerHIST
  int id=6;
  float rad = t.totalRadThickness();
  float y = 0;
  float w=1;
  hfill_(id,rad,y,w);
#endif

  t.storeback(&rmapVec);
#ifdef LISTHITS
  for (ArteTableIterator<GEPT> geptIt; geptIt.valid(); geptIt.next()){
    if (geptIt->hit1 == 0) {
      if (geptIt->mim1 > 0) {
	cout << "plane " << geptIt << " has no hits but mimps " << endl;
      }
    }
  }
#endif
  return 0;
}
