// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modified:   
// - moved many routines to State.C
//                                     14-Aug-00 Wouter Hulsbergen

#include <iostream>
#include <math.h>
#include "rang/StatePI.hh"
#include "rang/zfpar.hh"
#include "rang/rkmain.hh"
#include "rang/utfeld.hh"
#include "rang/zfconst.hh"

//rangerStatePI::rangerStatePI() : rangerState(6) {} 

int rangerStatePI::predict(const double& zDestination,
			 const double& qPExp,rangerState*& s,
			 HepMatrix*& F) const {

  int newState     = (s == 0);  // denote if memory allocations exist
  int newTransport = (F == 0);   //   .    .    .        .         .

#ifdef FULL
  cout << "rangerStatePI::predict: zDestination=" << zDestination << endl;

  cout << "rangerStatePI::predict: targetState=" << s
       << " newState= " <<  newState << endl;
#endif

  if (newState) {
    s = new rangerStatePI();
    if (!s) {
      cout << "rangerState::predict: allocation error" << endl; 
      exit(1);
    }
  }
  if (!F) {
    F = new HepMatrix(6,6);
    if (!F) {
      cout << "rangerState::predict: allocation error" << endl; 
      exit(1);
    }
  }

#ifdef FULL
  cout << "rangerStatePI::predict: after allocation: s->vec()=" << s->vec() 
       << endl << "       *F=" << *F << endl;
#endif

  double deltaZ = fabs(zDestination - zState);
  if ( (zDestination>zfpar_.zmaxMag && zState>zfpar_.zmaxMag)
       || (zDestination<zfpar_.zminMag && zState<zfpar_.zminMag)
       || deltaZ < 0.001) {
    s->setZ( zDestination );      // linear transport
    s->setX( x() + (zDestination - zState) * tx() );
    s->setY( y() + (zDestination - zState) * ty() );
    s->setTx( tx() );
    s->setTy( ty() );
    s->setQp( qP() );
    s->setI( I() );   // assume that TOF is not yet implemented
    s->setQpEst( qPEst() );

    for (int i=1; i<=6; i++) {
      for (int j=1; j<=6; j++) {
	(*F)(i,j) = 0.;
      }
    }
    (*F)(1,1) = 1;
    (*F)(2,2) = 1;
    (*F)(3,3) = 1;
    (*F)(4,4) = 1;
    (*F)(5,5) = 1;
    (*F)(6,6) = 1;
    (*F)(1,3) = zDestination - zState;
    (*F)(2,4) = zDestination - zState;
  }
  else {
    double zout = zDestination;
    double fstep = fabs(zDestination - zState);
    if (fstep<=0) fstep = 1;
    double pin[6];
    pin[0] = zState;
    pin[1] = x();
    pin[2] = y();
    pin[3] = tx();
    pin[4] = ty();
    pin[5] = qPExp;
    double prec[4];
    double dzTrunc = fabs(zDestination - zState);
    if (dzTrunc<1) dzTrunc = 1;

    prec[0] = 0.001 / dzTrunc;
    prec[1] = 0.001 / dzTrunc;
    prec[2] = 0.0001 / dzTrunc;
    prec[3] = 0.0001 / dzTrunc;

    int icovar = 1;
    //int icovar = 2;
    double pout[6];
    double rkd[25];
    int ierror;
    rkmain_(icovar,zout,fstep,pin,prec,pout,rkd,ierror);

    //cout << "state::predict: fake rkmain error " << endl;
    //ierror = 1;

    if (ierror == 0) {
      if (pout[0] != zDestination) 
	cout << "state::predict: zDest changed from "
	     << zDestination << " to " << pout[0] << endl;
      for (int i=1; i<=5; i++) {
	for (int j=1; j<=5; j++) {
	  (*F)(i,j) = rkd[5*(j-1)+i-1];
	}
	(*F)(i,6) = 0;
      }
      for (int j=1; j<=5; j++) {
	(*F)(6,j) = 0;
      }
      (*F)(6,6) = 1;

      s->setZ( zDestination );
      s->setX( pout[1] + (*F)(1,5) * (qP() - qPExp) );
      s->setY( pout[2] + (*F)(2,5) * (qP() - qPExp) );
      s->setTx( pout[3] + (*F)(3,5) * (qP() - qPExp) );
      s->setTy( pout[4] + (*F)(4,5) * (qP() - qPExp) );
      s->setQp( qP() );  // only valid in case momentum is unchanged
      s->setI( I() );    // assume that TOF is not yet implemented
      s->setQpEst( qPEst() );
    }
    else {
      cout << "state::predict: rkmain returns ierror=" << ierror 
	   << " during transport from " << zState << " to "
	   << zDestination << ", qP=" << qP() << endl;
      return -1;
    }
  }


#ifdef FULL
  cout << "rangerStatePI::predict: after calculation, *F=" << *F << endl;

  cout << "rangerStatePI::predict: the transported vector is"
       << s->vec() << endl;
#endif

  s->cov() = cov().similarity(*F);

#ifdef FULL
  cout << "rangerStatePI::predict: the transported cov is"
       << s->cov() << endl;
#endif

  if (1) {                // everything OK
    return 0;
  }
  else {
    cout << " This should not happen!!!" << endl;
    if (newState) delete s;
    if (newTransport) delete F;
    return -1000;
  }
}
