// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modified:   
// - moved many routines to State.C
// - introduced accessors to covmatrix elements
//                                     14-Aug-00 Wouter Hulsbergen

// class rangerStatePI
// state class for tracking with momentum and interaction time

#ifndef rangerStatePI_HH
#define rangerStatePI_HH

#ifndef rangerState_HH
#include "rang/State.hh"
#endif
#ifndef rangerHitBase_HH
#include "rang/HitBase.hh"
#endif

class rangerStatePI : public rangerState {

public:
  rangerStatePI() : rangerState(6) {}
  //rangerStatePI(const rangerStatePI& s);  // copy constructor
  virtual ~rangerStatePI() {}
  //   virtual rangerStatePI& operator=(const rangerState& s) { 
  //     rangerState::operator=(s) ; return *this ;}
  //   rangerStatePI& operator=(const rangerStatePI& s) { rangerState::operator=(s) ; return *this ;}

  virtual rangerState* clone() const { return new rangerStatePI(*this); } // produce copy
  virtual rangerFitMode fitMode() const { return ModePI; }
                                   
  virtual int predict(const double& zDestination,
	      const double& qPExp,rangerState*& targetState,
	      HepMatrix*& FTransport) const;  // perform Kalman filter prediction
  virtual void project(HepVector& resid,HepMatrix& HH,rangerHitBase* hit) {
    hit->projectPI(resid,HH,this); } ;
  
public:
  virtual const double qP() const { return p(5); }
  virtual const double I() const  { return p(6); }

  virtual void setQp(double qp) { p(5) = qp ; }
  virtual void setI(double qp) { p(6) = qp ; }

protected:

  virtual const double covXQp()  const { return cov().fast(5,1) ; }
  virtual const double covYQp()  const { return cov().fast(5,2) ; }
  virtual const double covTxQp() const { return cov().fast(5,3) ; }
  virtual const double covTyQp() const { return cov().fast(5,4) ; }
  virtual const double covQpQp() const { return cov().fast(5,5) ; }

  virtual const double covXI()  const { return cov().fast(6,1) ; }
  virtual const double covYI()  const { return cov().fast(6,2) ; }
  virtual const double covTxI() const { return cov().fast(6,3) ; }
  virtual const double covTyI() const { return cov().fast(6,4) ; }
  virtual const double covQpI() const { return cov().fast(6,5) ; }
  virtual const double covII()  const { return cov().fast(6,6) ; }

  virtual void setCovXQp(const double v)  { cov().fast(5,1) = v ; }
  virtual void setCovYQp(const double v)  { cov().fast(5,2) = v ; }
  virtual void setCovTxQp(const double v) { cov().fast(5,3) = v ; }
  virtual void setCovTyQp(const double v) { cov().fast(5,4) = v ; }
  virtual void setCovQpQp(const double v) { cov().fast(5,5) = v ; }

  virtual void setCovXI(const double v)  { cov().fast(6,1) = v ; }
  virtual void setCovYI(const double v)  { cov().fast(6,2) = v ; }
  virtual void setCovTxI(const double v) { cov().fast(6,3) = v ; }
  virtual void setCovTyI(const double v) { cov().fast(6,4) = v ; }
  virtual void setCovQpI(const double v) { cov().fast(6,5) = v ; }
  virtual void setCovII(const double v)  { cov().fast(6,6) = v ; }

};

#endif
