
// *****************************************************************************
//
// source:    rcRANGER.C
//
// type:      source code
//
// created:   13 Nov 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Department of Physics
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rcRANGER
//
// called by: rcStandalone()
//
// purpose:   handle tracking routines
//
// hints:     timing is done outside
//
// changes:
// 13 Nov 2000 (AG) creation
// 13 Nov 2000 (UU) change default OTr hit preparation to OtrI
// 14 Nov 2000 (AG) final polishing and approval
// 23 Nov 2000 (AG) added sterring for timing information
// 29 Nov 2000 (AG) polished printout
// 01 Dec 2000 (AG) polished printout
// 06 Dec 2000 (AG) polished printout
// 20 Dec 2000 (DG) removed TAXX stuff again
// 03 Jan 2001 (AG) removed 99A
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>

// ARTE includes
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "silarte/silarte.hh"
#include "cats/cats.h"
#include "rang/rgfit.hh"
#include "itrarte/ITR_arte.h"
#include "farm/farm.hh"
#include "Otr/OtrEvent.h"
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"

// globals
int maxhits     = 500000;
int hitprepflag = -1;

// externals
extern int FADEB;
extern int FATIME;

// prototypes
void waxevnt();
extern "C" void tfevntr_();

//
// function code
//
int
rcRANGER( int recoITR, int recoOTR )
{
  ARTE_mark( "rcRANGER 0", 0 );

  static Chrono *cron_rcITR    = 0;
  static Chrono *cron_rcOTRHIT = 0;
  static Chrono *cron_rcOTR    = 0;

  static bool first = 1;
  if( first ){
    first = 0;
    ChronoManager CM;
    CM.instance();
    
    cron_rcITR    = CM.createChrono( "rcITR"   );
    cron_rcOTRHIT = CM.createChrono( "rcOTRHIT"   );
    cron_rcOTR    = CM.createChrono( "rcOTR"   );
  }

  float time_rcITR    = 0;
  float time_rcOTRHIT = 0;
  float time_rcOTR    = 0;

  //==================================================================
  // 1. ITR create hits for data
  //==================================================================
  
  int ITRflag = 0;
  //    if( ArteTable<MTRA>::begin() == ArteTable<MTRA>::end() ){
  if( !ArteTable<MCIN>::booked() ){
    if( recoITR == 1 || recoITR == 3 ){
      ARTE_mark( "ItrEvnt", 0 );
      cron_rcITR->start();
      //
      ITRflag = ItrEvnt();
      //
      cron_rcITR->stop();
      time_rcITR = cron_rcITR->lastTime();
    }
  }
 
  //==================================================================
  // 2. use steering variables if set.
  //==================================================================
  if( hitprepflag == -1 ){
    char *f = 0;
    
    /* hit preparation for OTR  */
    if( ( f = getenv( "OtrHitPreparation") ) ) hitprepflag = atoi(f);
    else hitprepflag = 0;
    
    /* not supported anymore, just for backward compability */    
    if( ( f = getenv( "WAX_OCCUPCUT" ) ) ) maxhits = atoi(f);
    
    /* use this one instead */    
    if( ( f = getenv( "OtrOccupancyCut" ) ) ) maxhits = atoi(f);
    
    if( maxhits<500000 ){    
      cout << "rcOTR[warning]: Occupancy cut!!! Number of hits lower "
	   <<  maxhits << endl;
    }  
  }
  
  //==================================================================
  // 3. create hits:  fill HITB table with OTR hits
  //==================================================================
  int OTRflag = 0;
  if( !ArteTable<MCIN>::booked() ){
    if( recoOTR == 1 || recoOTR == 3 ){

      cron_rcOTRHIT->start();

      int notrhits = ArteTable<HITB>::size();
      switch( hitprepflag ){
      case 0:
	ARTE_mark( "OtrEventUpdate", 0 );
	OtrEventUpdate();
	break;
      case 1:
	ARTE_mark( "waxevnt", 0 );
	waxevnt();
	break;
      case 2:
	ARTE_mark( "waxevnt", 0 );
	waxevnt();
	ARTE_mark( "OtrEventUpdate", 0 );
	OtrEventUpdate();
	break;
      }
      notrhits = ArteTable<HITB>::size() - notrhits;
      if( notrhits > maxhits ) OTRflag = -1;

      cron_rcOTRHIT->stop();
      time_rcOTRHIT = cron_rcOTRHIT->lastTime();
    }
  }
  //==================================================================
  // 4. ranger tracking
  //==================================================================

  if( (( recoOTR > 1 ) && !OTRflag) || (( recoITR > 1 ) && !ITRflag) ){
    ARTE_mark( "tfevntr_", 0 );
    cron_rcOTR->start();
    //
    tfevntr_();
    //
    cron_rcOTR->stop();
    time_rcOTR = cron_rcOTR->lastTime();
  }
  
  // print
  if( FATIME ){
    printf( "\033[1m TIME: itr=%4.2f otr=%4.2f rang=%4.2f \033[0m \n",
	    time_rcITR, time_rcOTRHIT, time_rcOTR );
  }

  // return  
  ARTE_mark( "rcRANGER 1", 0 );
  return 0;
}
