
// ********************************************************************
//
//  Routine:      rcVDS.C
//
//  Description:  silicon stand alone reconstruction
//                RSEG/HITB/RHIT is stored in local coordinates 
//
//  Modification:
//   -- the global alignment transformation is done before the refit
//                                                       20/9/99 T.Oest
//   -- global alignment transformation is moved to rc99AStandAlone
//                                                       27/9/99 T.Oest
//   -- refit switched off to save time                 02/12/99 T.Oest
//   -- temporary transformation of HITB to local 
//      coordinates (required by CATS/HOLMES)           24/02/00 T.Oest
//   -- use ARTE_mark()                                 13/05/00 AG 
//   -- polished debug printout                         21/07/00 AG
//   -- moved to rcVDS                                  02/01/01 AG
//
// ********************************************************************

#include <iostream.h>
#include <stdlib.h>
#include "arte/arte.hh"
#include "silarte/silarte.hh"
#include "cats/cats.h"
#include "rang/rgfit.hh"
#include "farm/farm.hh"

int rcHITBtoLocal(const int, const int);

extern int recoVDSmodule;

int first = 0;



//*****************************************************************
// function
//*****************************************************************

int
rcVDS( int level )
{

//==================================================================
// 1. create hits for data
//==================================================================

     int flag = 0;
     if (level ==1 || level ==3 ){
       if (ArteTable<MTRA>::begin() == ArteTable<MTRA>::end()){
         flag = SvdEvnt();
       };
     };


//==================================================================
// 2. reconstruction, 
//    do a tansformation to local coordinates for HITB
//==================================================================

     if ( level > 1 ) {

     	ARTE_mark( "rcHITBtoLocal 0", 0 );
      	rcHITBtoLocal(1, 0);

      	ARTE_mark( "HolmesEvent", 0 );
      	if (recoVDSmodule & 1 ) HolmesEvent();

      	ARTE_mark( "CatsEvent", 0 );
      	if (recoVDSmodule & 2 ) CatsEvent();

      	ARTE_mark( "rcHITBtoLocal 1", 0 );
        rcHITBtoLocal(1, 1);
     };

//==================================================================
// 3. do a refit
// 4. fill number of hits in rseg bank
//==================================================================

   for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
        rseg != ArteTable<RSEG>::end(); ++rseg ) {
   

     if ((*rseg)->cmp == 1 || (*rseg)->cmp == 65537) {



// refit
       for( ArteRelationIterator<RHIT> rhit = (*rseg)->rhit.begin();
            rhit != (*rseg)->rhit.end(); ++rhit ) {
        (*rhit)->p = 0.2;
       };

        int fitflag   = (*rseg)->fit;
        int rs        = (*rseg) - ArtePointer<RSEG>(1) + 1;
	//	int fitstatus = rgfit_(rs,0);
        (*rseg)->fit  = fitflag;


// fill number of hits
//    it has to be done after the refit since time, dtim is set
//    to 0 there

        int ns=0,nN=0,nP=0;
        for (ArteRelationIterator<HITB> ihitb=(*rseg)->hitb.begin(); 
             ihitb!=(*rseg)->hitb.end(); ++ihitb) {
          ns++;
          int layer=(*ihitb)->gede->lay;
          if (layer==1 || layer==4) nP++;
          if (layer==2 || layer==3) nN++;
        };

        (*rseg)->time = nP;
        (*rseg)->dtim = nN;

     };

   };

     return flag;
}
