#include "reco/rcXkinema.h"


// Thorsten's bremsstrahlung recovery
float rc99Bremsstrahlung(ArtePointer<RTRA> rtra);
float kinema::findBremsstrahlung(ArtePointer<RTRA> t) {
  return rc99Bremsstrahlung(t);
}


HepLorentzVector kinema::FourVector(ArtePointer<RTRA> t, float m) {
  float p=1/fabs(t->pf);
  float pz=p/sqrt(t->txf*t->txf + t->tyf*t->tyf + 1);
  float px=pz*t->txf;
  float py=pz*t->tyf;
  return HepLorentzVector(px,py,pz,-sqrt(p*p+m*m));
}

HepLorentzVector kinema::FourVector(ArtePointer<RTRA> t) {
  float m=t->topp->mass;
  float p=1/fabs(t->pf);
  float pz=p/sqrt(t->txf*t->txf + t->tyf*t->tyf + 1);
  float px=pz*t->txf;
  float py=pz*t->tyf;
  return HepLorentzVector(px,py,pz,-sqrt(p*p+m*m));
}




HepLorentzVector kinema::FourVector(ArtePointer<MTRA> t) {
  float p=sqrt(t->px*t->px + t->py*t->py + t->pz*t->pz);
  float m=t->topp->mass;
  return HepLorentzVector(t->px,t->py,t->pz,-sqrt(p*p+m*m));
}

extern "C"  void rkmain_(const int    & icovar, 
              const double & zout, 
              const double & fstep, 
              const double * parameters, 
              const double * prec, 
              const double * p_track, 
              const double * derivatives, 
              const int    & ierror);

int kinema::swim(const double *source,double *dest,const double zout) {
  const int    icovar=2;
  double       fstep=50;
  double       prec[4];
  double       derivatives[5][5];
  int          ierror;

  prec[0] = 0.001/fabs(zout-source[0]);
  prec[1] = prec[0];
  prec[2] = 0.0001/fabs(zout-source[0]);
  prec[3] = prec[2];

  rkmain_(icovar,zout,fstep,source,prec,dest,&derivatives[0][0],ierror);

  return ierror;
}


int kinema::swim(ArtePointer<RTRA> rtra, double zout) {
  double pin[]={rtra->zf, rtra->xf, rtra->yf, rtra->txf, rtra->tyf, rtra->pf};
  double pou[6];
  swim(pin, pou, zout);
  RPNT rpnt;
  rpnt.xf =pou[1];
  rpnt.yf =pou[2];
  rpnt.zf =pou[0];
  rpnt.txf=pou[3];
  rpnt.tyf=pou[4];
  rpnt.pf =pou[5];
  for (int i=0; i<15; i++) rpnt.cvf[i]=rtra->cvf[i];
  ArtePointer<RPNT> prpnt=ArteTable<RPNT>::push_back(rpnt);
  createRelation( prpnt, rtra, &RPNT::rtra);
  return 0;
}


ArtePointer<TOPP> kinema::findTopp(int lund) {
  ArtePointer<TOPP> topp;
  for (ArteTable<TOPP>::iterator itopp=ArteTable<TOPP>::begin();
       itopp!=ArteTable<TOPP>::end(); ++itopp) {
    if ((*itopp)->lund==lund) { topp=(*itopp); break; }
  }
  return topp;
}


//
// Returns c.m. angle of daughter and mother direction
//
float kinema::angleCM(HepLorentzVector daughter, HepLorentzVector mother) {
  float p2m2  = 1/sqrt(mother.vect().mag2() + mother.m2());
  Hep3Vector beta = mother.vect();
  beta *= p2m2;
  daughter.boost(beta);
  return daughter.vect()*mother.vect()/(daughter.vect().mag()*mother.vect().mag());
}

float kinema::angleCM(ArtePointer<MTRA> mtra) {
  HepLorentzVector daughter=FourVector(mtra);
  HepLorentzVector mother  =FourVector(mtra->btra);
  return angleCM(daughter, mother);
}



//
// Returns Feynmann x for HERA-B conditions
//
float kinema::XF(HepLorentzVector vec) {
  // .. find beam-target boost
  static float mp    = .938272;
  static float mt    = 0.5*(mp + .93956533);
  static float betaz = 920./(sqrt(mp*mp+920*920) + mt);
  static float pzmax=sqrt(0.5*(mp*sqrt(mp*mp+920*920)-mp*mp));
  Hep3Vector beta(0, 0, betaz);
  // ... apply boost
  vec.boost(beta);
  // ... get xf
  return vec.pz()/pzmax;
}


float kinema::XF(ArtePointer<MTRA> t) {
  HepLorentzVector vec=FourVector(t);
  return XF(vec);
}


float kinema::XF(ArtePointer<RTRA> t) {
  HepLorentzVector vec=FourVector(t);
  return XF(vec);
}

