// ********************************************************************
//
//  Routine:      rckamatch.C
//  Author:       T. Oest
//
//  Description:  kuip command RECO/MATCH
//
//  Modification:
//  -- default kuip command : "MATCH SUBD1 SUBD2" which doesn't require
//     word "MARPLE" anymore. The old option is available as
//     " MATCH FIT98 SUBD1 SUBD2"                11/05/00 O.Igonkina
//  -- OTR-ECAL match added                      14/01/00 O.Igonkina
//  -- OTR match added                           06/12/99 O.Igonkina
//  -- SVD -PC RPNT added                        26/07/00 O.Igonkina
// 03 Jan 2000 (AG) removed 99A
//
// ********************************************************************

// includes

int matchVdsEcal     = 0;
int matchVdsRich     = 0;
int matchVdsEcalRich = 0;
int matchTarget      = 0;

int matchMARPLEVdsEcal  = 0;
int matchMARPLEVdsRich  = 0;
int matchMARPLEOtrRich  = 0;
int matchMARPLEEcalRich = 0;
int matchMARPLEVdsOtr   = 0;
int matchMARPLEVdsPc    = 0;
int matchMARPLEOtrEcal  = 0;
//int matchMARPLE2000     = 0;

extern short MARPLE_MAKERSEG; 

/************************************************************************
* KAMATCH
*
* called from  KUIP
***********************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
//#include "arte/Arte.hh"

extern "C" void kugetc_( char*, int *, int );


extern "C" void kamatch_() {   

//---------------------------------------------------------------
//   read in subdetector 1,2,3
//---------------------------------------------------------------

  int len1, len2, len3;
  int lenmax = 6;
  char subd1[7], subd2[7], subd3[7];
  kugetc_(subd1, &len1, lenmax);
  kugetc_(subd2, &len2, lenmax);
  kugetc_(subd3, &len3, lenmax);

  subd1[len1+1] = 0;
  subd2[len2+1] = 0;
  subd3[len3+1] = 0;

  for(int i=0; i<len1; i++) subd1[i] = toupper(subd1[i]);
  for(int i=0; i<len2; i++) subd2[i] = toupper(subd2[i]);
  for(int i=0; i<len3; i++) subd3[i] = toupper(subd3[i]);

//---------------------------------------------------------------
//   read in subdetector 1,2,3
//---------------------------------------------------------------
   int VDS  = 0; 
   int ECAL = 0;
   int RICH = 0;
   int OTR  = 0;
   int PC   = 0;

   if (!strncmp(subd1,"ALL",3) || !strncmp(subd2,"ALL",3)) {
     if( !strncmp(subd1,"OFF",3) || !strncmp(subd2,"OFF",3) ||
	 !strncmp(subd3,"OFF",3) )
       {
	 VDS  = 0;
	 ECAL = 0;
	 RICH = 0;
	 OTR  = 0;
	 PC   = 0;
	 matchMARPLEVdsEcal  = 0;
	 matchMARPLEVdsRich  = 0;
	 matchMARPLEOtrRich  = 0;
	 matchMARPLEEcalRich = 0;
	 matchMARPLEVdsOtr   = 0;
	 matchMARPLEVdsPc    = 0;
	 matchMARPLEOtrEcal  = 0;
       }else{
	 VDS  = 1;
	 ECAL = 1;
	 RICH = 1;
	 OTR  = 1;
	 PC   = 1;
       }
   } else {


   if (!strncmp(subd1,"VDS",3) || !strncmp(subd2,"VDS",3) || 
       !strncmp(subd3,"VDS",3) ) VDS = 1;

   if (!strncmp(subd1,"SVD",3) || !strncmp(subd2,"SVD",3) || 
       !strncmp(subd3,"SVD",3) ) VDS = 1;

   if (!strncmp(subd1,"ECAL",4) || !strncmp(subd2,"ECAL",4) || 
       !strncmp(subd3,"ECAL",4) ) ECAL = 1;

   if (!strncmp(subd1,"RICH",4) || !strncmp(subd2,"RICH",4) || 
       !strncmp(subd3,"RICH",4) ) RICH = 1;

   if (!strncmp(subd1,"OTR",3) || !strncmp(subd2,"OTR",3) || 
       !strncmp(subd3,"OTR",3) ) OTR = 1;

   if (!strncmp(subd1,"PC",2) || !strncmp(subd2,"PC",2) || 
       !strncmp(subd3,"PC",2) ) PC = 1;

   }
//---------------------------------------------------------------
//   marple matching
//---------------------------------------------------------------
/*
   if (!strncmp(subd2,"2000",4)|| !strncmp(subd2,"2000",4) || 
       !strncmp(subd3,"2000",4)  ){

     matchMARPLE2000 = 1; // switch on complete marple OTR based 
     MARPLE_MAKERSEG = 1;

     matchMARPLEVdsEcal  = 0;
     matchMARPLEVdsRich  = 0;
     matchMARPLEOtrRich  = 0;
     matchMARPLEEcalRich = 0;
     matchMARPLEVdsOtr   = 0;
     matchMARPLEVdsPc    = 0;
     matchMARPLEOtrEcal  = 0;

     if(  matchMARPLE2000 ) printf(" match MARPLE 2000 \n");
    
   }else 
*/
if ( strncmp(subd1,"FIT9",4)) {  // if it is not FIT98 request
     MARPLE_MAKERSEG = 1;
     if (VDS == 1  && ECAL == 1)  matchMARPLEVdsEcal  = 1;
     if (RICH == 1 && ECAL == 1)  matchMARPLEEcalRich = 1;
     if (VDS == 1  && RICH == 1)  matchMARPLEVdsRich  = 1;
     if (VDS == 1  && OTR == 1 )  matchMARPLEVdsOtr   = 1;
     if (VDS == 1  && PC == 1  )  matchMARPLEVdsPc    = 1;
     if (RICH == 1 && OTR == 1 )  matchMARPLEOtrRich  = 1;
     if (ECAL == 1 && OTR == 1 )  matchMARPLEOtrEcal  = 1;
     if (!strncmp(subd2,"TAR",3)) matchMARPLEVdsRich = 1;
     if (!strncmp(subd2,"OFF",3))
       {
	 matchMARPLEVdsEcal  = 0;
	 matchMARPLEVdsRich  = 0;
	 matchMARPLEOtrRich  = 0;
	 matchMARPLEEcalRich = 0;
	 matchMARPLEVdsOtr   = 0;
	 matchMARPLEVdsPc    = 0;
	 matchMARPLEOtrEcal   = 0;
       }
     
     //   printf(" MARPLE will call : \n");
     if( matchMARPLEVdsEcal  ) printf(" match MARPLE : VDS  ECAL \n");
     if( matchMARPLEEcalRich ) printf(" match MARPLE : RICH ECAL \n");
     if( matchMARPLEVdsRich  ) printf(" match MARPLE : VDS  RICH \n");
     if( matchMARPLEVdsOtr   ) printf(" match MARPLE : VDS  OTR\n");
     if( matchMARPLEVdsPc    ) printf(" match MARPLE : VDS  PC-RPNT\n");
     if( matchMARPLEOtrRich  ) printf(" match MARPLE : OTR  RICH\n");
     if( matchMARPLEOtrEcal  ) printf(" match MARPLE : OTR  ECAL\n");
     
     return;
     
//---------------------------------------------------------------
//   fit98 matching
//---------------------------------------------------------------
   }else{                          // FIT98

    if (VDS == 1 && ECAL == 1 && RICH == 1) matchVdsEcalRich = 1;
    if (VDS == 1 && ECAL == 1 && RICH == 0) matchVdsEcal     = 1;
    if (VDS == 1 && ECAL == 0 && RICH == 1) matchVdsRich     = 1;
    if (VDS == 0 && ECAL == 1 && RICH == 1) matchTarget      = 1;
    if (!strncmp(subd1,"TAR",3)           ) matchTarget      = 1;
    
  };

   return;
};

