*-- Author :    T. Oest 

      SUBROUTINE RCV2FIT(RTRA1, RTRA2, M1, M2, CHI, XVER, CVER,
     +                   PVER, CPVER, MASS, FLAG)

************************************************************************
*
* Description:
*
* Called from:
*
* External references:
*
* Argument in/out description
* RTRA1    in     first rtra row
* RTRA2    in     second rtra row
* M1       in     mass of first track
* M2       in     mass of second track
* chi      out    chi2 of fit
* xv(3)    out    x,y,z
* cv(3,3)  out    COV(x,y,z)
* pv(6)    out    (z,y,x,tx,ty,1/p) track momentum
* cpv      out    error on pv
* mass     out    two particle mass
* FLAG     OUT    FIT FLAG
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "auto/EV.fi"
#include "auto/CON.fi"
#include "auto/GEO.fi"
#include "vt/vtdat.fi"
#include "vt/vt_rtra.fi"

      INTEGER RTRA1, RTRA2
      REAL*4 CHI, XVER(3), CVER(3,3), MASS, M1, M2
      REAL*4 PVER(6), CPVER(5,5)
      REAL*4 E1, E2, P1, P2


      INTEGER IV, FLAG, I ,J

      INTEGER TPNAME

      REAL*8  X(3), COV(3,3), Z1 ,Z2
      REAL*8  PAR(5),  COVR(5, 5)
      DATA COV /1.0D5  , 0.0D0  , 0.0D0 ,
     +          0.0D0  , 1.0D5  , 0.0D0 ,
     +          0.0D0  , 0.0D0  , 1.0D5 /

#include "arte/Arte.ff"
#include "auto/EV.ff"
#include "auto/CON.ff"
#include "auto/GEO.ff"
C*****************************************************************
C   fit two particles without mass constraint
C*****************************************************************

      FLAG = 0
      CHI  = -1000.

      TRMULT = 0
      CALL VTZERO
      CALL VTRINI(RTRA1)
      CALL VTRINI(RTRA2)


      XMASS(1)  = DBLE(M1)
      XMASS(2)  = DBLE(M2)
      CALL VTZTR(1, 2, Z1, Z2)
      X(1) = 0.5*(P(3,2) + Z2 * P(1,2) + P(3,1) + Z1 * P(1,1))
      X(2) = 0.5*(P(4,2) + Z2 * P(2,2) + P(4,1) + Z1 * P(2,1))
      X(3) = 0.5*(Z1+Z2)

      CALL VTINIT (X,COV,IV,FLAG)
      IF (FLAG.LT.0)                              GOTO 900
      CALL VTADD  (IV,1, FLAG)
      IF (FLAG.LT.0)                              GOTO 900
      CALL VTADD  (IV,2, FLAG)
      IF (FLAG.LT.0)                              GOTO 900

      CALL VTFIT(IV, 0, FLAG)

      CALL VTSUM(IV,PAR,COVR,FLAG)
      IF (FLAG.NE.0)                              GOTO 900

      XVER(1) = SNGL(XVS(1,IV))
      XVER(2) = SNGL(XVS(2,IV))
      XVER(3) = SNGL(XVS(3,IV))

      DO I=1,3
      DO J=1,3
       CVER(I,J) = SNGL(CS(I,J,IV))
      ENDDO
      ENDDO

      CHI = SNGL(CHI2(IV,2))

C*****************************************************************
C   momentum
C*****************************************************************

      PVER(1) = XVER(3)
      PVER(2) = XVER(1)
      PVER(3) = XVER(2)
c      PVER(4) = PAR(1)
c      PVER(5) = PAR(2)
c      PVER(6) = 1. / PAR(5)
      PVER(4) = SNGL(PAR(1))
      PVER(5) = SNGL(PAR(2))
      PVER(6) = 1. / SNGL(PAR(5))

      CPVER(1,1) = sngl(COVR(3,3)) 
     +             + 2. * PVER(1)    * sngl(COVR(1,3))
     +             +      PVER(1)**2 * sngl(COVR(1,1))
      CPVER(1,2) =  sngl(COVR(3,4)) 
     +             +      PVER(1)    * sngl(COVR(1,3))
     +             +      PVER(1)    * sngl(COVR(4,2))
     +             +      PVER(1)**2 * sngl(COVR(1,2))
      CPVER(1,3) = sngl(COVR(3,1)) 
     +             +      PVER(1)    * sngl(COVR(1,1))
      CPVER(1,4) = sngl(COVR(3,2)) 
     +             +      PVER(1)    * sngl(COVR(1,2))
      CPVER(1,5) = - sngl(COVR(3,5)) * PVER(6)**2
     +             +      PVER(1)    * sngl(COVR(1,5))

      CPVER(2,1) = CPVER(1,2)
      CPVER(2,2) = sngl(COVR(4,4)) 
     +             + 2. * PVER(1)    * sngl(COVR(2,4))
     +             +      PVER(1)**2 * sngl(COVR(2,2))
      CPVER(2,3) = sngl(COVR(4,1)) 
     +             +      PVER(1)    * sngl(COVR(2,1))
      CPVER(2,4) = sngl(COVR(4,2)) 
     +             +      PVER(1)    * sngl(COVR(2,2))
      CPVER(2,5) = - sngl(COVR(4,5)) * PVER(6)**2
     +             +      PVER(1)    * sngl(COVR(2,5))

      CPVER(3,1) = CPVER(1,3)
      CPVER(3,2) = CPVER(2,3)
      CPVER(3,3) = sngl(COVR(1,1)) 
      CPVER(3,4) = sngl(COVR(1,2)) 
      CPVER(3,5) = - sngl(COVR(1,5))  * PVER(6)**2

      CPVER(4,1) = CPVER(1,4)
      CPVER(4,2) = CPVER(2,4)
      CPVER(4,3) = CPVER(3,4)
      CPVER(4,4) = sngl(COVR(2,2)) 
      CPVER(4,5) = - sngl(COVR(2,5))  * PVER(6)**2

      CPVER(5,1) = CPVER(1,5)
      CPVER(5,2) = CPVER(2,5)
      CPVER(5,3) = CPVER(3,5)
      CPVER(5,4) = CPVER(4,5)
      CPVER(5,5) = sngl(COVR(5,5))  * PVER(6)**4

C*****************************************************************
C   calculate mass
C*****************************************************************

      P1 = sngl(QVS(3,IV,1))
      P2 = sngl(QVS(3,IV,2))
      E1 = SQRT(M1**2 + P1**2)
      E2 = SQRT(M2**2 + P2**2)


      MASS = SQRT((E1+E2)**2 - 1./PVER(6)**2)

c-------------

 900  END

************************************************************************
* End 
************************************************************************
