//--*-C++-*--
//
//  Description:
//   Get list of good tracks and check RICH pid.
//   MC information on tracks and rings is also
//   available. File needs to be (re)compiled with
//   option USE_ROOT_DD
//
//  Created: Apr 04 2001
//
//  Modifications:
//
//
#include "reco/rcXdata.h"
#include "reco/rcXkinema.h"
#include "reco/rcXgeo.h"
#include "reco/rcXsegment.h"
#include "rich/rich.h"

#ifdef USE_ROOT_DD
#include "TH1.h"
TFile                       *event_file=0;
TH1F                        *event_histo;
TTree                       *event_tree;
datamass::TRACK_PARAMETER   event_track;
datamass::TRACK_PARAMETER   event_mctrack;
datamass::TRACK_PARAMETER   event_rpnt;
datamass::RICH_RING         event_ring;
datamass::RICH_RING         event_mcring;
datamass::ECAL_CLUSTER      event_ecal;
datamass::TRACKER_SEGMENT   event_muon;
datamass::HERAB_EVENT       eventE;
datamass::TYPE              event_mctype;
datamass::LIKELIHOODS       event_lr;
TTree                       *event_header;
#endif

int  trSelect( vector< ArtePointer<RTRA> > &rtra );
template<class T>  void Clear(T &c) {
  memset(&c,0x0,sizeof(T));
}



void RMonitorEvent() {
#ifdef USE_ROOT_DD
  static bool init=false;
  if (!init) {
    event_file=datamass::createRootFile("track","List of tracks");
    event_histo = new TH1F("event_histo","Events", 2,-.5,1.5);
    event_tree=new TTree("event","Track tree");
    event_track.branch (event_tree,"t");
    event_rpnt.branch  (event_tree,"rpnt");
    event_ring.branch  (event_tree,"rich");        
    event_mcring.branch(event_tree,"mc_rich");        
    event_muon.branch  (event_tree,"muon");        
    event_ecal.branch  (event_tree,"ecal");        
    eventE.branch      (event_tree,"event");
    event_lr.branch    (event_tree,"lr");
    if (ArteTable<MTRA>::size()) {
      event_mctrack.branch(event_tree,"mc_t");
      event_mctype.branch (event_tree,"mc_type");
    }
    event_header=new TTree("header","Track tree");
    eventE.branch      (event_header,"event");    
    init=true;
  }

  eventE.copy();
  event_histo->Fill(0);
  event_header->Fill();

  vector<ArtePointer<RTRA> > Rtra;
  trSelect(Rtra);
  for (vector<ArtePointer<RTRA> >::iterator irtra=Rtra.begin();
       irtra!=Rtra.end(); ++irtra) {
    // ... copy seed information
    event_track.copy(*irtra);
    event_ecal.copy (*irtra);
    event_muon.hit1=(*irtra)->hmu;
    // ... copy RICH reonstruction information
    Clear<datamass::RICH_RING>(event_ring);
    Clear<datamass::LIKELIHOODS>(event_lr);
    Clear<datamass::TRACK_PARAMETER>(event_rpnt);
    event_ring.copy (*irtra);
    segment::findSegments(*irtra);
    if (segment::richf()) event_ring.copy(*segment::irich);
    if ((*segment::irich)->rpnt.begin()!=(*segment::irich)->rpnt.end()) {
      event_rpnt.copy(*(*segment::irich)->rpnt.begin());
    }
    event_lr.copy   (*irtra);
    // ... copy MC information 
    Clear<datamass::TRACK_PARAMETER>(event_mctrack);
    Clear<datamass::TYPE>           (event_mctype);
    Clear<datamass::RICH_RING>      (event_mcring);
    if ((*irtra)->mtra.begin()!=(*irtra)->mtra.end()) {
      event_mctrack.copy(*(*irtra)->mtra.begin());
      event_mctype.copy (*(*irtra)->mtra.begin());
      mcRICH(*(*irtra)->mtra.begin(), event_mcring.tx, event_mcring.ty, event_mcring.rad, event_mcring.hri);
    }
    event_tree->Fill();
  }
#endif  
}


void RMonitorEvent_stop() {
#ifdef USE_ROOT_DD
  if (event_file) {
    cout<<"Writing track file ...";
    event_file->Write();
    cout<<"done"<<endl;
  }
#endif
}
