#include "rich/rich.h"

/**
   Shadowing of RICH photons by beam pipe, out of acceptance

   @param  track[0-2] = track coordinate after magnet
   @param  track[3-4] = track directions after magnet
   @param  track[5]   = ring radius
   @return fraction of accepted photons
*/
float RShadow(float *track) {

  int    n=100; // radiated photons
  float  z=950; // vessel coordinate
  float  x[] = { track[0]+track[3]*(z-track[2]), track[1]+track[4]*(z-track[2]), z}; 
  float  p[] = { track[3], track[4], 1}; 
  float  costhe = 1 - .5 * track[5]*track[5];

  CoordinateSystem A;
  A.setrotation(Vec3(p)); // x,y arbitrary, but perpendicular to \vec{p}.
  A.x0()=10*Vec3(x);

  Photon pp;
  int i;
  double eff=0;
  for (i=0; i<n; i++) { 
    double phi=2*M_PI*rnd_normal();

    pp.n()     = A.RotateToOld(FromPolar(1, costhe, phi));
    pp.x()     = A.x0();

    int r1=RICH_Vessel->geometry_trace(&pp);
    int r2=RICH_Detector->hit_in_acceptance(pp);
    if (!r1&&r2 ) eff++;
    }
  eff /= n;
  return eff;
}
