// standard includes
#include <iostream.h>
#include <stdio.h>
#include <time.h>
#include <math.h>

// ONLINE icludes
#include "etb.h"

// Arte includes
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "farm/fa.hh"
#include "farm/farm.hh"

#define L2IO_FED_RECORD_ID 0x1000

//
// get list of FED blocks (see ETB)
//
extern "C" {
int ARTE_get_feds_DSLT( int max, ETB_fed *feds, int *n );
void ARTE_put_feds_DSLT( ETB_fed *feds, int num,int size );
};

int
ARTE_get_feds_DSLT( int max, ETB_fed *feds, int *n )
{
  // inits
  int rc = ETB_ok;
  *n = 0;
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    return ETB_out_of_memory;
  }
  
  //
  // use DEVT
  //
  EVENT_HDR *event = ARTE_get_event();
  if( event ){

    if( ( rc = ETB_get_feds( event, COMP_ID_SLT, max, feds, n ) ) != ETB_ok ){
      cout << "ARTE_get_feds_DSLT(): ERROR: fed array too small? "
	   << "max=" << max << " "
	   << "n="   << *n  << " "
	   << "rc="  << rc << endl;
    }
    if( *n > max ) *n = max;
    return rc;
  }
  //
  // use DSLT
  //
  else if( ArteTable<DSLT>::size() ){

    // Arte pointer
    ArtePointer<DSLT> dnnn( 1 );
    
    // data words in table (first word is recid)
    int total = ( ArteTable<DSLT>::size() - 1 ) * sizeof( int );
    


    // recid
    int recid = 0;
    ESP_get_int( (int *) dnnn->bits, &recid, 1 );
    
    // SLB_SUBHDR
    int  bc   = 0;
    unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
    int j              = 0;
    
    // loop
    while( bc < (int) ( total - sizeof( SLB_SUBHDR ) ) ){
      
      // check array size
      if( j >= max ){
	return ETB_out_of_memory;
      }
      
      // update FED count
      *n = j + 1;
      
      // ETB_fed header
      SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
      ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
      feds[j].slb  = 0; // pointer to slb record (not existent here)
      feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
      


      // increase cunters
      j++; // fed counter
      bc += hdr->length; // byte counter
    }
    return ETB_ok;
  }

  // return
  return ETB_corrupted_event;
}  

void ARTE_put_feds_DSLT( ETB_fed *feds, int num,int size ){

  /* Copy from "arte/utdevt.C" */

  size=size+sizeof(SLB_SUBHDR)*num;

  // clear table
  static bool first = 1;
  if( first ){
    first = 0;
    ArteTable<DSLT>::clear();
  }
  int nrow = ArteTable<DSLT>::size();


  // book
  if( !nrow ){
    mmbook_( ArtePointer<DSLT>::getLink().getRef(), 
	     size / sizeof( int ) + 1 );
    nrow = 1; // jump to second word
  }
  else {
    mmbook_( ArtePointer<DSLT>::getLink().getRef(), 
	     size / sizeof( int ) );
  }
  // table
  ArtePointer<DSLT> t( 1 );
  int  *newID = (int *)   &t->bits;    
  char *p     = (char *)( &t->bits + nrow );
  int recid=L2IO_FED_RECORD_ID;

  ESP_put_int( &recid , newID, 1 );
  for(int i=0;i<num;i++) {
    ESP_put_slb_subhdr(&feds[i].hdr,(SLB_SUBHDR *)p);
    p+=sizeof(SLB_SUBHDR);
    memcpy(p,feds[i].data,feds[i].hdr.length-sizeof(SLB_SUBHDR));
    p+=feds[i].hdr.length-sizeof(SLB_SUBHDR);
  }


}

