// *****************************************************************************
//
// source:    L3_Track.C
//
// type:      source code
//
// created:   06 Dec 1999
//
// author:    Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// symbol:    
//
// changes:
//
// *****************************************************************************

#include "L3_Track.hh"

int L3_Track::Chips[7][4][2]; // [SL][SEC][chip1 or chip2]

void L3_Track::refit(float p) {
  sort(hitlist.begin(),hitlist.end(),H_greater());
  resetChips();
  for (vector<L3_Hitb>::const_iterator hIt=hitlist.begin(); hIt != hitlist.end(); hIt++){
    Chips[ hIt->gesl()-1 ][ hIt->sec()-1 ][ hIt->chip()-1 ]++;
  }
#if 0
  cout << hitlist.size() << " hits" << endl;
  for(int sl=0;sl<7;sl++) for(int sec=0;sec<4;sec++) for(int chip=0;chip<2;chip++) if( Chips[sl][sec][chip]>0 ){
    cout << sl+1 << ' ' << sec+1 << ' ' << chip << " : " << Chips[sl][sec][chip] << endl;
  }
#endif
  chiSquare=0.0 ;
  parTransport( (hitlist.front()).z() );
  cov.clear();
  cov(0,0) = 16.;
  cov(1,1) = 16.;
  cov(2,2) = 1.;
  cov(3,3) = 1.;
  //cov(0,0) = 32.;
  //cov(1,1) = 32.;
  //cov(2,2) = 0.04;
  //cov(3,3) = 0.04;
  //cov(0,0) = 1.;
  //cov(1,1) = 1.;
  //cov(2,2) = 0.04;
  //cov(3,3) = 0.04;
  double dchi2;
  for (vector<L3_Hitb>::const_iterator hIt=hitlist.begin(); hIt != hitlist.end(); ) {
    kalTransport(hIt->z());
    const L3_Plane &Plane = hIt->getPlane();
    //the weight is 1.0 if there are two hits on this chip
    //the weight is 2.0 if there is just one hit on this chip
    float weight = 2.0/float(Chips[hIt->gesl()-1][hIt->sec()-1][hIt->chip()-1]);
    kalMultScat(weight*Plane.ms(),p);
    kalFilter(dchi2, hIt->x(), Plane.cos(), Plane.sin());
    chiSquare += dchi2;
    ++hIt;
#if 0
    if( hIt==hitlist.end() && p<0.0 ){
      //When we processed the last hit and if the momentum is unknow we add now 
      //K*(theta)^2 to both Cov(tx) and Cov(ty) to get
      //the pulls right. 
      double pSquared = pow(L3_Parameter::meanMomentum(),2) * ( 1.0+1.0/(pow(the_mx,2)+pow(the_my,2)) );      
      double ms = Plane.ms()*L3_Parameter::multipleScattering()/pSquared;
      ms *= L3_Parameter::K();
      if( hitlist.size() > 4 ){
	ms *= chiSquare/double(hitlist.size()-4);
      } else {
	if( L3_Parameter::Debug() ) cout << "L3_Track::refit: only " << hitlist.size() << " hits" << endl;
	ms *= 100.0; //blow it up
      }
      cov(2,2) += ms;
      cov(3,3) += ms;      
    }
#endif
  }
  //cout << " Covariance Matrix at first Hit " << endl;
  //cov.print();
  kalTransport(0.0);
  //cout << " Covariance Matrix at z = 0" << endl;
  //cov.print();
  //if( L3_Parameter::Trace() ) cout << "refit STOP" << endl;
}


void L3_Track::kalTransport(const double& rz) {
  double dZ = (rz - the_refz);
  the_refz = rz;
// swim Impact Parameter
  the_x += dZ*the_mx;
  the_y += dZ*the_my;
// swim Covariance Matrix
  double Ctmp02 = cov(0,2) + dZ * cov(2,2);
  cov(0,0) += dZ *(cov(0,2) + Ctmp02);
  cov(0,2) = Ctmp02;
  double Ctmp13 = cov(1,3) + dZ * cov(3,3);
  cov(1,1) += dZ *(cov(1,3) + Ctmp13);
  cov(1,3) = Ctmp13;

  cov(0,3) += dZ * cov(2,3);  
  cov(0,1) += dZ *(cov(1,2) + cov(0,3));
  cov(1,2) += dZ * cov(2,3);

  cov(1,0) = cov(0,1);
  cov(2,0) = cov(0,2);
  cov(3,0) = cov(0,3);
  cov(2,1) = cov(1,2);
  cov(3,1) = cov(1,3);
}

void L3_Track::kalMultScat(float ms,float p) {
  // --- add ms contribution [from orig. z]
  //  MSC = 0.0000012 / (2.5 GeV * 2.5 GeV) = 0.000000192
  //  add it to cov(2,2) and cov(3,3)
  //  cov(2,2) += 0.000000192;
  //cov(3,3) += 0.000000192;
  // taking value for 1 GeV Particle from GEDE : 0.3623E-06
  // MSC = 3.623e-7 / (4.5 GeV * 4.5 GeV) = 1.78914e-8
  if( p<=0.0 ) p = L3_Parameter::meanMomentum()*sqrt( 1.0+1.0/(pow(the_mx,2)+pow(the_my,2)) );      
  ms *= L3_Parameter::multipleScattering()/pow(p,2);  
  cov(2,2) += ms;
  cov(3,3) += ms;
}

void L3_Track::kalFilter(double& dchi2,const double m, const double co, const double si) {
  // Resolution of single Measurement m
  // double sigma= 0.005 / sqrt( 12.); 
  // double sigma2 = 0.005 * 0.005 / 12.;
  // const double sigma2 = 0.0000020833333;
  //For MC, the Acc (accuracy) value from GEDE is taken , at the moment 0.0012
  const double sigma2 = 0.00000144;
  // calculate factor [V + H C H^T]^-1
  double invfactor = 1. / ( sigma2 + co * co * cov(0,0) + si * si * cov(1,1)
			    - 2.0 * co * si * cov(0,1) );
  // calculate Kalman gain matrix
  double K[4];
  K[0] = invfactor * (co * cov(0,0) - si * cov(1,0));
  K[1] = invfactor * (co * cov(0,1) - si * cov(1,1));
  K[2] = invfactor * (co * cov(0,2) - si * cov(1,2));
  K[3] = invfactor * (co * cov(0,3) - si * cov(1,3));
  //filter parameter vector
  double help = m - co * the_x + si * the_y; // calc. m - H p
  the_x  += K[0] * help;
  the_y  += K[1] * help;
  the_mx += K[2] * help;
  the_my += K[3] * help;
  //filter cov matrix
  double mhelp[4][2];
  mhelp[0][0] = 1.0 - K[0] * co;
  mhelp[0][1] =       K[0] * si;
  mhelp[1][0] =     - K[1] * co;
  mhelp[1][1] = 1.0 + K[1] * si;
  mhelp[2][0] =      -K[2] * co;
  mhelp[2][1] =       K[2] * si;
  mhelp[3][0] =      -K[3] * co;
  mhelp[3][1] =       K[3] * si;
  //We can save some double operations here since C_fil is symmetric.
  L3_CovMatrix<4> C_fil;
  C_fil(0,0) = mhelp[0][0]*cov(0,0) + mhelp[0][1]*cov(1,0);
  C_fil(1,0) = mhelp[1][0]*cov(0,0) + mhelp[1][1]*cov(1,0);
  //C_fil(2,0) = mhelp[2][0]*cov(0,0) + mhelp[2][1]*cov(1,0) + cov(2,0);
  //C_fil(3,0) = mhelp[3][0]*cov(0,0) + mhelp[3][1]*cov(1,0) + cov(3,0);
  //C_fil(0,1) = mhelp[0][0]*cov(0,1) + mhelp[0][1]*cov(1,1);
  C_fil(1,1) = mhelp[1][0]*cov(0,1) + mhelp[1][1]*cov(1,1);
  //C_fil(2,1) = mhelp[2][0]*cov(0,1) + mhelp[2][1]*cov(1,1) + cov(2,1);
  //C_fil(3,1) = mhelp[3][0]*cov(0,1) + mhelp[3][1]*cov(1,1) + cov(3,1);
  C_fil(0,2) = mhelp[0][0]*cov(0,2) + mhelp[0][1]*cov(1,2);
  C_fil(1,2) = mhelp[1][0]*cov(0,2) + mhelp[1][1]*cov(1,2);
  C_fil(2,2) = mhelp[2][0]*cov(0,2) + mhelp[2][1]*cov(1,2) + cov(2,2);
  C_fil(3,2) = mhelp[3][0]*cov(0,2) + mhelp[3][1]*cov(1,2) + cov(3,2);
  C_fil(0,3) = mhelp[0][0]*cov(0,3) + mhelp[0][1]*cov(1,3);
  C_fil(1,3) = mhelp[1][0]*cov(0,3) + mhelp[1][1]*cov(1,3);
  //C_fil(2,3) = mhelp[2][0]*cov(0,3) + mhelp[2][1]*cov(1,3) + cov(2,3);
  C_fil(3,3) = mhelp[3][0]*cov(0,3) + mhelp[3][1]*cov(1,3) + cov(3,3);
  //copy the rest
  C_fil(2,0) = C_fil(0,2);
  C_fil(3,0) = C_fil(0,3);
  C_fil(0,1) = C_fil(1,0);
  C_fil(2,1) = C_fil(1,2);
  C_fil(3,1) = C_fil(1,3);
  C_fil(2,3) = C_fil(3,2);
  //cout <<"C(2,0) = "<<C_fil(2,0)<<' '<<mhelp[2][0]*cov(0,0) + mhelp[2][1]*cov(1,0) + cov(2,0)<<endl;
  //cout <<"C(3,0) = "<<C_fil(3,0)<<' '<<mhelp[3][0]*cov(0,0) + mhelp[3][1]*cov(1,0) + cov(3,0)<<endl;
  //cout <<"C(0,1) = "<<C_fil(0,1)<<' '<<mhelp[0][0]*cov(0,1) + mhelp[0][1]*cov(1,1)<<endl;
  //cout <<"C(2,1) = "<<C_fil(2,1)<<' '<<mhelp[2][0]*cov(0,1) + mhelp[2][1]*cov(1,1) + cov(2,1)<<endl;
  //cout <<"C(3,1) = "<<C_fil(3,1)<<' '<<mhelp[3][0]*cov(0,1) + mhelp[3][1]*cov(1,1) + cov(3,1)<<endl;
  //cout <<"C(2,3) = "<<C_fil(2,3)<<' '<<mhelp[2][0]*cov(0,3) + mhelp[2][1]*cov(1,3) + cov(2,3)<<endl;
  cov = C_fil; //copy entire matrix to destination
  // calculate Chi^2 increment
  double r =  m - co * the_x + si * the_y;
  //double R = (1.0 - co * K[0] + si * K[1]) * sigma2; 
  //cout << "DEBUG 21.1.99: standard R = " << R << endl; 
  double R = sigma2 * sigma2 * invfactor;
  if ( fabs(R) > 0. ) {
    dchi2 = r*r / R;
  } else {
    dchi2 = 0.0;
  }
}

L3_Track::L3_Track(const int& nr, const float& x, const float&y,const float&mx, 
			  const float&my, const float& refz,vector<L3_Hitb> &hits,float p) : the_flag(0) {
  the_nr = nr;
  the_x  = x;
  the_y  = y;
  the_mx = mx;
  the_my = my;
  the_refz = refz;
  hitlist = hits;
  if(hitlist.size()>0) refit(p);  
}

L3_Track::L3_Track(const int& nr, const float& x, const float&y,const float&mx, 
			  const float&my, const float& refz,vector<L3_SpacePoint*> &points,float p) : the_flag(0) {
  the_nr = nr;
  the_x  = x;
  the_y  = y;
  the_mx = mx;
  the_my = my;
  the_refz = refz;
  for(vector<L3_SpacePoint*>::iterator i=points.begin();i!=points.end();i++){
    RaumPunktList.push_back( *i );
    (*i)->getCopyOfHitbs(hitlist);
  }
  if(hitlist.size()>0) refit(p);
}

int L3_Track::numPunkteSL(int sl) const {
  int n = 0;
  //cout << "L3_Track::numPunkte("<<sl<<"): ";
  for(vector<SpacePoint>::const_iterator i=RaumPunktList.begin();i!=RaumPunktList.end();i++)
    {      
      if(sl == (*i)->gesl()) n++;
      //cout << ' ' << (*i)->gesl(); 
    }
  //cout << endl;
  return n;
}

void L3_Track::fullrefit(float p) {
  //if( L3_Parameter::Trace() ) cout << "refit START " << p << endl;
  sort(hitlist.begin(),hitlist.end(),H_greater());
  resetChips();
  for (vector<L3_Hitb>::const_iterator hIt=hitlist.begin(); hIt != hitlist.end(); hIt++){
    Chips[ hIt->gesl()-1 ][ hIt->sec()-1 ][ hIt->chip()-1 ]++;
  }
  const int N = hitlist.size();
  vector< HepMatrix > Cprime(N);

  vector< HepVector > trackP(N);
  vector< HepMatrix > CtrackP(N);

  vector< HepVector > trackPredict(N);  
  vector< HepMatrix > CtrackPredict(N);

  vector< HepVector > trackPsmooth(N);
  vector< HepMatrix > CtrackPsmooth(N);

  chiSquare=0.0 ;
  parTransport( (hitlist.front()).z() );
  cov.clear();
  cov(0,0) = 16.;
  cov(1,1) = 16.;
  cov(2,2) = 1.;
  cov(3,3) = 1.;
  double dchi2;
  for(int n=0;n<N;n++){
    vector<L3_Hitb>::const_iterator hIt = hitlist.begin()+n;
    kalTransport(hIt->z());
    //------------ save the predicted track parameters
    trackPredict[n] = Parameter();
    //------------ save the cov matrix of the prediction WITHOUT MS contribution
    CtrackPredict[n] = cov.getHepMatrix();   
    //--------- calculate multiple scattering
    const L3_Plane &Plane = hIt->getPlane();
    //the weight is 1.0 if there are two hits on this chip
    //the weight is 2.0 if there is just one hit on this chip
    float weight = 2.0/float(Chips[hIt->gesl()-1][hIt->sec()-1][hIt->chip()-1]);
    kalMultScat(weight*Plane.ms(),p);
    //------------ save the cov matrix of the prediction with MS contribution
    Cprime[n] = cov.getHepMatrix();
    kalFilter(dchi2, hIt->x(), Plane.cos(), Plane.sin());
    chiSquare += dchi2;
    //---------- save the obtained track parameters
    trackP[n] = Parameter();
    //--------- get covmatrix of track parameters
    CtrackP[n] = cov.getHepMatrix();
  }
  kalTransport(0.0);
  //------ smoothing starts ----------
  vector<L3_Hitb> bestList;
  for(int n=N-1;n>=0;n--){
    const L3_Hitb& hit = hitlist[n];
    //cout << "step " << n << " sl/sec/lay " << hit.gesl() << '/' << hit.sec() << '/' << hit.lay() << " and z " << hit.z() << " ";
    if( n==N-1 ){
      trackPsmooth[n] = trackP[n];
      CtrackPsmooth[n] = CtrackP[n];
    } else {
      //calculate D
      HepMatrix D(4,4);
      for(int i=0;i<4;i++){
	D[i][i] = 1.0;
	for(int k=0;k<i;k++){
	  D[i][k] = 0.0;
	  D[k][i] = 0.0;
	}
      }
      float dz = hitlist[n+1].z() - hitlist[n].z();
      D[0][2] = dz;
      D[1][3] = dz;
      int err=0;
      HepMatrix A = CtrackPredict[n+1].inverse(err);
      if( err != 0 ) cout << "error in matrix inersion, code is " << err << endl;      
      A = CtrackP[n] * D * A;
      trackPsmooth[n] = trackP[n] + A * ( trackPsmooth[n+1] - trackPredict[n+1] );
      CtrackPsmooth[n] = CtrackP[n] + A * (CtrackPsmooth[n+1] - Cprime[n+1]) * A.T();
    }
    const L3_Plane& Plane = hitlist[n].getPlane();
    const double m = hitlist[n].x();
    double resid =  m - Plane.cos() * (trackPsmooth[n])[0] + Plane.sin() * (trackPsmooth[n])[1];
    if( fabs(resid) <= 0.005 ){
      bestList.push_back(hit);
    }
  }
  if( (int)bestList.size() != N ){
    //cout << bestList.size() << " of " << N << " hits have a residual better than 100 mu" << endl;
    //cout << "track param: " << Parameter() << endl;
    //cov.print();
    hitlist = bestList;
    refit(p);
    //cout << "refit" << endl;
    //cout << "track param: " << Parameter() << endl;
    //cov.print();
  }
  //if( L3_Parameter::Trace() ) cout << "refit STOP" << endl;
}

L3_Window L3_Track::window2(const float& at_z) {

  float xmin, xmax, ymin, ymax;

  if( numPunkte()==1 ){
    float x = RaumPunktList[0]->x();
    float y = RaumPunktList[0]->y();
    float z = RaumPunktList[0]->z();
    float sv = (at_z - L3_Geometry::TargetBox_zmax()) / (z - L3_Geometry::TargetBox_zmax());
    float sh = (at_z - L3_Geometry::TargetBox_zmin()) / (z - L3_Geometry::TargetBox_zmin());
    if (x >= L3_Geometry::TargetBox_xmax()) {
      xmin = L3_Geometry::TargetBox_xmin() + (x - L3_Geometry::TargetBox_xmin()) * sv;
      xmax = L3_Geometry::TargetBox_xmax() + (x - L3_Geometry::TargetBox_xmax()) * sh;
    } else if (x <= L3_Geometry::TargetBox_xmin()) {
      xmin = L3_Geometry::TargetBox_xmin() + (x - L3_Geometry::TargetBox_xmin()) * sh;
      xmax = L3_Geometry::TargetBox_xmax() + (x - L3_Geometry::TargetBox_xmax()) * sv;
    } else {
      xmin = L3_Geometry::TargetBox_xmin() + (x - L3_Geometry::TargetBox_xmin()) * sv;
      xmax = L3_Geometry::TargetBox_xmax() + (x - L3_Geometry::TargetBox_xmax()) * sv;
    }
    if (y >= L3_Geometry::TargetBox_ymax()) {
      ymin = L3_Geometry::TargetBox_ymin() + (y - L3_Geometry::TargetBox_ymin()) * sv;
      ymax = L3_Geometry::TargetBox_ymax() + (y - L3_Geometry::TargetBox_ymax()) * sh;
    } else if (y <= L3_Geometry::TargetBox_ymin()) {
      ymin = L3_Geometry::TargetBox_ymin() + (y - L3_Geometry::TargetBox_ymin()) * sh;
      ymax = L3_Geometry::TargetBox_ymax() + (y - L3_Geometry::TargetBox_ymax()) * sv;
    } else {
      ymin = L3_Geometry::TargetBox_ymin() + (y - L3_Geometry::TargetBox_ymin()) * sv;
      ymax = L3_Geometry::TargetBox_ymax() + (y - L3_Geometry::TargetBox_ymax()) * sv;
    }
  } else {
    kalTransport(at_z);
    float dx = 3*Cx();
    float dy = 3*Cy();
    xmin = the_x-dx;
    xmax = the_x+dx;
    ymin = the_y-dy;
    ymax = the_y+dy;
  }
  return L3_Window(xmin, xmax, ymin, ymax, at_z);
}

float L3_Track::lsmfit(float refz){
  const float sigma2 = 1.44E-6; //(12 microns)^2 but in cm
  int numP = hitlist.size();
  if( numP < 1 ) return 999;
  HepMatrix A(numP,4);
  HepVector m(numP);
  for(int i=0;i<numP;i++)
    {      
      const L3_Hitb &h = hitlist[i];
      const L3_Plane& plane = h.getPlane();
      const float dz = h.z()-refz;
      A[i][0]= plane.cos();
      A[i][1]= -plane.sin();
      A[i][2]= dz*plane.cos();
      A[i][3]= -dz*plane.sin();
      m[i] = h.x();
    }
  //prepare fit
  HepMatrix B(4,4);
  B = A.T() * A;
  HepMatrix C(4,4);
  int ierr=0;
  C = B.inverse(ierr);  
  if( L3_Parameter::Debug() && ierr!=0 ){
    cout << "L3-Track::lsmfit: error in matrix inversion " << ierr << endl;
  }
  HepVector c(4);
  c = C * A.T() * m;
  the_x  = c[0]; 
  the_y  = c[1];
  the_mx = c[2]; 
  the_my = c[3];
  the_refz = refz;
  //cov matrix
  C *= sigma2;
  for(int i=0;i<4;i++){
    cov(i,i) = C[i][i];
    for(int k=0;k<i;k++){
      cov(i,k) = C[i][k];
      cov(k,i) = C[k][i];
    }
  }
  chiSquare = 0.0;
  for(int i=0;i<numP;i++){
      const L3_Hitb &h = hitlist[i];
      const L3_Plane& plane = h.getPlane();
      const float dz = h.z()-refz;
      chiSquare += pow( h.x() - (the_x+dz*the_mx)*plane.cos() + (the_y+dz*the_my)*plane.sin(), 2.0 );
  }
  chiSquare /= sigma2;
  return chiSquare;
}

float L3_Track::flightDiff(Hep3Vector &pv,float *z1){

  Hep3Vector v1(0,0,1);
  Hep3Vector pt(x(),y(),refz());
  Hep3Vector v2 = Vector();

  Hep3Vector p = pv-pt;
  Hep3Vector w = v1.cross(v2);
      
  Hep3Vector d = (p*w)/w.mag2();
      
  if(z1!=NULL) *z1 = (v2.mag2()*(d*v1-p*v1)+(v1*v2)*(p*v2-d*v2)) / (v1.mag2()*v2.mag2()-pow(v1*v2,2));
  return d.mag();
}

float L3_Track::refitP(){
  //ksi of pt
  const double P1 = 1.0927;
  const double P2 = 0.14355;
  const double P3 = 1.5182;
  const double P3inv = 1.0/P3;
  float ksi = chisqr()/float(numHits());
  float ptp = 0.7012; //mean pt for all tracks with ksi <= P1
  if( ksi > P1 ){
    ptp = pow( P2/(ksi-P1) , P3inv );
  }
  float pp = ptp * sqrt( 1.0 + 1.0/(pow(mx(),2)+pow(my(),2)) );
  refit(pp);
  return pp;
}



