// *****************************************************************************
//
// source:    tlt.C
//
// type:      source code
//
// created:   06 Dec 1999
//
// author:    Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// symbol:    
//
// changes:
//
// *****************************************************************************
#include "L3_Defines.hh"
#include <iostream>
#include "L3_SLP.hh"
#include "L3_Trigger.hh"
L3_Trigger TLT;

extern int    nArg;
extern char **vArg;

int tlt_trig(void){
  return TLT.trig();
}

void tlt_init(void) {
  static int First = 1;
  if(First){
    First = 0;
#ifdef L3_SLP_FRAME
    TLT.init(nArg,vArg);
#endif
  }
}

void tlt_evnt(void) {
#ifdef L3_ARTE_FRAME  
  static int tlt_first = 1;
  if(tlt_first>0){
    tlt_first = 0;
    TLT.init(nArg, vArg);
  } else if( !ArteTable<MCIN>::booked()) {
    if(L3_Parameter::Trace()) cout << "TLT: Updating target information" << endl;
    L3_Geometry::updateTarget();
  }
#endif
  TLT.process();
}

void tlt_stop(void) {
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "TLT: Stopping" << endl;
#endif
  TLT.stop();
}

#ifdef L3_SLP_FRAME
extern "C" {
#include <stdio.h>
#define _BOOL
#include "tlt.h"
#undef _BOOL
  int TLT_startup(SLP_transitions which) {
    SLP_Info(GRR_OK,"TLT_startup was called");
#ifdef SLP_TRACE
    printf("TLT_startup was called\n");
#endif
    tlt_init();
    return SMC_OK;
  }
  
  int TLT_initialize(SLP_transitions which) {
    SLP_Info(GRR_OK,"TLT_initialize was called");
#ifdef SLP_TRACE
    printf("TLT_initialize was called\n");
#endif
    return SMC_OK;
}

  int TLT_zero(SLP_transitions which) {
    SLP_Info(GRR_OK,"TLT_zero was called");
#ifdef SLP_TRACE
    printf("TLT_zero was called\n");
#endif    
    return SMC_OK;
  }

  int TLT_shutdown(SLP_transitions which) {
    SLP_Info(GRR_OK,"TLT_shutdown was called");
#ifdef SLP_TRACE
    /*  printf("TLT_shutdown was called\n"); */
#endif
    tlt_stop();
    return SMC_OK;
  }

  SLP_RETURN TLT_trigger(int event, int param) {
    SLP_RETURN ret = {event, SLP_ACCEPT, SLP_OK}; 
#ifdef SLP_TRACE
    /*SLP_Info(GRR_OK,"TLT_trigger was called");*/
    /*printf("TLT_trigger was called\n");*/
#endif
    L3_SLP::setL2EventNr(event);
    tlt_evnt();
    return ret;
  }
  
  void TLT_free(int event) {
#ifdef SLP_TRACE
  /* SLP_Info(GRR_OK,"TLT_free was called");
  printf("TLT_free was called\n");*/
#endif
  }

  SLP_RETURN TLT_handle(int isRPS, int msg_class, void *header, void *data) {
    SLP_RETURN ret = {SLP_NO_EVENT, SLP_SUCCESS, SLP_OK};
#ifdef SLP_TRACE
    SLP_Info(GRR_OK,"TLT_trigger: handle was called");
    printf("TLT_handle was called\n");
#endif
    return ret;
  }

  SLP_RETURN TLT_handleTimeout() {
    /*  SLP_RETURN ret = {SLP_NO_EVENT, SLP_SUCCESS, SLP_OK}; */
    SLP_RETURN ret = {SLP_NO_EVENT, SLP_REJECT, SLP_OK}; 
    
    if( SLPdbg & SLPDBG_TRACE )
      SLP_Info(GRR_OK,"TLT_handleTimeout was called\n");
    
    SLP_logError(SLP_ERR_UNKNOWN,"TLT_handleTimeout: time out occured, rejecting event");
    return ret;
  }
}
#endif








