#include <fstream>
#include <iomanip>
#include <algorithm>
#include "wax/DecodeManager.hh"
#include "wax/OTrAsdId.h"
#include "wax/OTrRead.h"
#include "wax/EventBuilder.hh"
#ifdef USE_ARTE
#include "arte/arte.hh"
#include "OtrI.h"
#include "Otr/OtrEvent.h"
extern "C" EVENT_HDR* ARTE_get_event() ;
#endif

const waxDecodeManager* waxDecodeChannel::fgDecodeManager = 0;

waxDecodeManager* waxDecodeManager::fgMe = 0;

TdcCount_t waxDecodeChannel::fgMinTdcCount = 20 ;
TdcCount_t waxDecodeChannel::fgMaxTdcCount = 254 ;

// steering parameters for Maarten's algorithm
unsigned int waxDecodeManager::fgNoiseSuppressionLevel = 1 ;
TdcCount_t waxDecodeManager::fgTimeDiffWindow = 5 ;
unsigned char waxDecodeManager::fgMaxNHitsSL=6;
unsigned char waxDecodeManager::fgMaxNHitsDL=8;

#ifdef __OTR_OLD_TABLES__
std::string waxDecodeManager::fgListOfTablesFileName = "/OTR/db/tables/list_of_tables_hallw" ;
std::string waxDecodeManager::fgTdcFedFileName       = "/OTR/db/tables/tdc_fed_hallw" ;
#else
std::string waxDecodeManager::fgListOfTablesFileName = "/OTR/db/tables-new/list_of_tables_hallw" ;
std::string waxDecodeManager::fgTdcFedFileName       = "/OTR/db/tables-new/tdc_fed_hallw" ;
#endif

std::string waxDecodeManager::fgSwapCableFileName = "/OTR/db/swapcable/default.swap" ;

std::string waxDecodeManager::fgTZeroFileName = "" ; 
std::string waxDecodeManager::fgMaskFileName = "" ;

void waxDecodeChannel::SetTZero(DriftTime_t tz, DriftTime_t tzerr) {
  // check that tz and tzerr are within range
  int tzc = int(128 + 2*tz    + 0.5) ; 
  int tzec = int(16*tzerr + 0.5) ; 
  if(tzc>255) {
    tzc=255 ;
    tzec=255 ;
  } else if(tzc<0) {
    tzc=0 ;
    tzec=255 ;
  } else if(tzec>255) {
    tzec=255 ;
  } else if(tzec<0) 
    tzec=0 ;
  SetTZeroCount(tzc,tzec) ;
}

waxDecodeGede::waxDecodeGede()
{
  fWires[0]=fWires[1]=fWires[2]=fWires[3]=fWires[4]=0;
}

waxDecodeManager* waxDecodeManager::GetMe() 
{
  if(!fgMe) fgMe = new waxDecodeManager() ;
  return fgMe ;
}

void
waxDecodeManager::DeleteMe()
{
  delete fgMe ;
  fgMe=0 ;
}
 
waxDecodeManager::~waxDecodeManager() 
{
  //delete fDecodeTable ;
  if(fgMe==this) fgMe = 0;
}

waxDecodeManager::waxDecodeManager()
{
#ifdef __WAX_DEBUG__
  cout << "waxDecodeManager: initializing decoding." << endl ;
#endif
  if(!fgMe) fgMe = this ;
  
  int mode=0;
  fMinGedeIndex=117;
  std::string tablelistfile  = waxEventBuilder::GetHBROOTDirName() + fgListOfTablesFileName ;
  std::string tdcfedfile     = waxEventBuilder::GetHBROOTDirName() + fgTdcFedFileName ;
  fDecodeTable = OTr_Decode_init(mode,tablelistfile.c_str(),tdcfedfile.c_str()) ;
  
  if(fDecodeTable!=NULL) {
    waxDecodeChannel::SetDecodeManager(this) ;
    for(AsdIterator asdIt = GetAsds() ; *asdIt != 0; asdIt++)
      for(waxDecodeAsd::ChannelIterator chanIt = (*asdIt)->GetChannels(); 
	  chanIt !=  (*asdIt)->GetChannelsEnd(); chanIt++) {
#ifdef USE_ARTE
	if(ArteTable<MCIN>::booked()) chanIt->SetTZero(0,0) ;
	else chanIt->SetTZero(0,100) ;
#endif
	chanIt->SetStatus(0) ;
      }
    ReadStatus() ;
    if(!ArteTable<MCIN>::booked()) {
      ReadTZeros() ;
      SwapCables() ;
    }
#ifdef USE_ARTE
    CreateDecodeGedes() ;
#endif   

#ifdef __WAX_DEBUG__
    cout << "Wax DEBUGGING mode: " << endl ;
 
    AsdIterator firstAsd =  GetAsds() ;
    int asdId=0;
    for(AsdIterator asdIt = firstAsd; *asdIt != 0 ; 
	asdIt++,asdId++) { 
      int chanId=0;
      for(waxDecodeAsd::ChannelIterator chanIt = (*asdIt)->GetChannels() ;
	  chanIt != (*asdIt)->GetChannelsEnd() ; chanIt++, chanId++) {
     
	int gedeId = chanIt->GetGedeIndex() ;
	if(gedeId != 0) {
	  int wireId = chanIt->GetWireIndex() ;
	  const waxDecodeGede* gedeIt = GetGede(gedeId) ;
	  if(gedeIt==0) cout << "No such gede " << gedeId << endl ;

	  const waxDecodeWire* wireIt = gedeIt->GetWire(wireId) ;
	  if(asdId  != wireIt->GetAsdIndex()  || chanId != wireIt->GetChannelIndex() ) {
	    waxDecodeChannel* altChanIt = 
	      const_cast<waxDecodeChannel*>(GetAsd(wireIt->GetAsdIndex())->
					    GetChannel( wireIt->GetChannelIndex())) ;
	    altChanIt->SetStatus(2) ;
	    chanIt->SetStatus(2) ;
	    ArtePointer<GEDE> artegede(gedeId) ;
	    std::cout << "Problem !!!! "  
		      << artegede->hwc << " "
		      << setw(6) << gedeId 
		      << setw(4) << wireId 
	      //<< setw(6) << asdId
		      << GetAsd(asdId)->GetName() << " "
		      << setw(4) << chanId 
	      //<< setw(6) << wireIt->GetAsdIndex() 
		      << GetAsd(wireIt->GetAsdIndex())->GetName() << " "
		      << setw(4) << int(wireIt->GetChannelIndex())
		      << setw(6) << altChanIt->GetGedeIndex() 
		      << setw(4) << int(altChanIt->GetWireIndex()) 
		      << std::endl ;
	    cout << "Wires per ml: " ;
	    for(int m=0; m<artegede->nsl; m++) 
	      cout << artegede->nwi[m] << " ";
	    cout << endl ;
	  }
	}
      }
    }
    
    int gedeid=fMinGedeIndex ;
    const waxDecodeWire* wireIt;
    for(GedeContainer::const_iterator gedeIt = fGedes.begin() ;
   	gedeIt != fGedes.end() ; ++gedeIt, ++gedeid) {
      for(int wireid=0; wireid<128; wireid++) {
   	if( (wireIt = gedeIt->GetWire(wireid)) && 
   	    wireIt->IsConnected()) {
   	  const waxDecodeChannel* chanIt = wireIt->GetChannel() ;
   	  if(chanIt->GetGedeIndex() != gedeid ||
   	     chanIt->GetWireIndex() != wireid) {
   	    cout << "ProblemTwo: "
   		 << (ArtePointer<GEDE>(gedeid))->hwc << " "
   		 << wireid << " "
   		 << (ArtePointer<GEDE>(chanIt->GetGedeIndex()))->hwc << " "
   		 << chanIt->GetWireIndex() << " "
   		 << wireIt->GetAsdIndex() << " "
   		 << wireIt->GetChannelIndex() << endl ;
   	  }
   	}
      }
    }
    
    cout << "These wires are not connected: " << endl ;
    const waxDecodeGede* decgedeIt ;
    for(ArteTable<GEDE>::iterator gedeIt = ArteTable<GEDE>::begin() ;
	gedeIt != ArteTable<GEDE>::end() ; ++gedeIt) 
      if(gedeIt->cmp==2 && gedeIt->nsl>0) {
	gedeIt->hwc[12]='\0' ;
	decgedeIt = GetGede(gedeIt.index()) ;
	if(decgedeIt) {
	  int ntot(0) ;
	  for(int m=0; m<gedeIt->nsl; m++) ntot += gedeIt->nwi[m]-1 ;
	  for(int m=0; m<gedeIt->nsl; m++)
	    for(int c=0; c<gedeIt->nwi[m]; ++c) {
	      int wireid = 32*m+c;
	      wireIt = decgedeIt->GetWire(wireid) ;
	      if(!wireIt) 
		cout << "wire not found: " 
		     << gedeIt->hwc << " " << m << " " << c << endl ;
	      else if(!wireIt->IsConnected())
		cout << "wire not connected: " 
		     << gedeIt->hwc << " " 
		     << m << " " << c << " " 
		     << gedeIt->nwi[m]-1 << " " << ntot << endl ;
	    }
	} else cout << "gede not found: " << gedeIt->hwc << endl;
      }
    
    // 	int chanId = wireIt->GetChannelIndex() ;
// 	const waxDecodeAsd* asdIt = GetAsd(asdId) ;
// 	const waxDecodeChannel* chanIt = asdIt->GetChannel(chanId) ;
// 	// 	std::cout << gedeId << " " <<  wireId << " ==> "
// 	// 	     << asdId << " " << chanId << " "
// 	// 	     << chanIt->GetStatus() << std::endl ;
// 	if(gedeId != chanIt->GetGedeIndex() ||
// 	   wireId != chanIt->GetWireIndex()) {
// 	  std::cout << "Help !" << gedeId << " " << wireId << " "
// 		    << chanIt->GetGedeIndex()<< " " << chanIt->GetWireIndex()<< " "
// 		    << asdId << " " << chanIt - asdIt->GetChannels() << std::endl ;
       
// 	}
//       }
//       }
//     }

    // There seems to be a problem with the fed pointers in the asds. Let's check.
    asdId=0;
    for(AsdIterator asdIt = GetAsds(); *asdIt != 0 ; 
	asdIt++,asdId++) { 
      (*asdIt)->GetName() ;
      int fedid = (*asdIt)->GetFedIndex() ;

      if(fedid >0 && fedid <MAXFED) {
	int tdcid = fDecodeTable->FEDs[fedid] ;
	if(tdcid< MAXTDC) {
	  int tdcconnector = (*asdIt)->GetTdcConnectorIndex() ;
	  const OTrTDC* tdcIt = fDecodeTable->TDCs[tdcid] ;
	  if(tdcIt->ASD[tdcconnector-1] != asdId) {
	    cout << "ASD-TDC pointers not good: " 
		 << (*asdIt)->GetName() << " "
		 << asdId << " " << fedid << " " << tdcid << endl;
	  }
	} else {
	  cout << "Also strange:  Asd ->" << (*asdIt)->GetName() 
	       << "  fedid: " << fedid << " " << tdcid << endl ;
	}
      } else {
	cout << "Strange: Asd ->" << (*asdIt)->GetName() 
	     << "  fedid: " << fedid << endl ;
      }
    }
    cout << "last asd: " << asdId-1 << endl ;

    for(OTrTDC** tdcIt = fDecodeTable->TDCs ; *tdcIt !=0; tdcIt++) {
      for(int tdcconnid=0; tdcconnid<8; tdcconnid++) {
	int asdId = (*tdcIt)->ASD[tdcconnid] ;
	if(asdId>=0 && asdId<MAXASD && fDecodeTable->ASD8s[asdId]) {
	  const waxDecodeAsd* asdIt = GetAsd(asdId) ;
	  if((asdIt->GetFedIndex() != (*tdcIt)->FED) ||
	     (asdIt->GetTdcConnectorIndex() != tdcconnid+1)) {
	    cout << "ASD-TDC pointers not good: " 
		 << asdIt->GetName() << " "
		 << asdId << " " << (*tdcIt)->FED << " " << asdIt->GetFedIndex()
		 << " " <<asdIt->GetTdcConnectorIndex() << " " << tdcconnid+1  << endl;
	  }
	}
      }
    }
    cout << "Waiting ... press a key!"
    getchar() ;

#endif
  } else {
    std::cout << "waxDecodeManager::waxDecodeManager(): "
	 << "Could not create OTR_Decode object." << std::endl ;
  }
}

bool waxCompareGede(const unsigned short lhsgedeId, const unsigned short rhsgedeId)
{
  int lhsindex(0),rhsindex(0) ;
#ifdef USE_ARTE
  int mingedeid = waxDecodeManager::GetMe()->GetMinGedeIndex() ;
  ArtePointer<GEDE> lhsGede(lhsgedeId+mingedeid) ;
  ArtePointer<GEDE> rhsGede(rhsgedeId+mingedeid) ;
  if(lhsGede.index() && lhsGede->cmp==2)
    lhsindex = 
      atoi(lhsGede->hwc+6) +  //modid
      100*(lhsGede->sec     +
	   100*(lhsGede->lay +
		10*(lhsGede->gesl.index()))) ;
  if(rhsGede.index() && rhsGede->cmp==2)
    rhsindex = 
      atoi(rhsGede->hwc+6) +  //modid
      100*(rhsGede->sec     +
	   100*(rhsGede->lay +
		10*(rhsGede->gesl.index()))) ;
   
#endif
  return lhsindex<rhsindex ;
}

void
waxDecodeManager::CreateDecodeGedes()
{
#ifdef USE_ARTE
#ifdef __WAX_DEBUG__
  cout << "waxDecodeManager::CreateDecodeGedes()" << endl ;
#endif
  // count gedes and wires
  unsigned int mingedeid(100000),maxgedeid(0),totnwires(0) ;
  for(ArteTable<GEDE>::iterator gedeIt = ArteTable<GEDE>::begin() ;
      gedeIt != ArteTable<GEDE>::end() ; gedeIt++) 
    if(gedeIt->cmp==2 && gedeIt->hwc[0]!='n' && gedeIt->nsl>0) {
      if     (mingedeid>gedeIt.index()) mingedeid=gedeIt.index() ;
      else if(maxgedeid<gedeIt.index()) maxgedeid=gedeIt.index() ;
      for(int m=0; m<gedeIt->nsl; m++) totnwires+=gedeIt->nwi[m] ;
    }
#ifdef __WAX_DEBUG__
  cout <<"min/maxgede: " << mingedeid << " " << maxgedeid << endl ;
#endif
  // allocate gedes
  fMinGedeIndex = mingedeid;
  fGedes.resize(maxgedeid-mingedeid+1,waxDecodeGede()) ;

  // create map which contains gedes in correct order
  fGedeMap.resize(fGedes.size()) ;
  for(unsigned int i=0; i<fGedeMap.size(); i++) fGedeMap[i]=i;
  sort(fGedeMap.begin(),fGedeMap.end(),waxCompareGede) ;
  
  // allocate wires and set pointers in gedes. the wires are put in
  // the order in which they are needed for filling the HITB bank
  fWires.resize(totnwires) ;
  WireContainer::iterator wireIt = fWires.begin() ;
  
  for(waxDecodeManager::GedeMapIterator 
	gedeidIt = fGedeMap.begin() ;
      gedeidIt != fGedeMap.end(); ++gedeidIt) {
    GedeIndex_t gedeId = fMinGedeIndex + *gedeidIt ;
    ArtePointer<GEDE> gedeIt = ArtePointer<GEDE>(gedeId) ;
    if(gedeIt->cmp==2 && gedeIt->hwc[0]!='n' && gedeIt->nsl>0) {
      waxDecodeGede* decgedeIt = fGedes.begin() + *gedeidIt ;
      for(int m=0; m<5; m++) {
	decgedeIt->fWires[m] = wireIt;
	if(m<gedeIt->nsl) wireIt += gedeIt->nwi[m] ;
      }
    }
  }
#ifdef __WAX_DEBUG__
  if(wireIt != fWires.end() ) cout << "Strange, miscounted wires ? " << endl ;
#endif
  
  // connect
  for(AsdIterator asdIt = GetAsds() ; *asdIt != 0; asdIt++)
    for(waxDecodeAsd::ChannelIterator chanIt = (*asdIt)->GetChannels(); 
	chanIt !=  (*asdIt)->GetChannelsEnd(); chanIt++) 
      if(chanIt->GetGedeIndex()>0) {
	waxDecodeWire* wire = const_cast<waxDecodeWire*>
	  (fGedes[chanIt->GetGedeIndex()-fMinGedeIndex].
	   GetWire(chanIt->GetWireIndex())) ;
	if(wire) {
	  wire->fAsdId     = asdIt - GetAsds() ;
	  wire->fChannelId = chanIt - (*asdIt)->GetChannels();
	} else {
	  cout <<"Trying to connect non-existent wire !"
	       << chanIt->GetGedeIndex() << " "
	       << int(chanIt->GetWireIndex()) << " " 
	       << GetAsd(chanIt->GetAsdIndex())->GetSectorIndex() << " "
	       << endl ;
	}
      }
#endif
}

void
waxDecodeManager::ResetTdcCounts()
{
  for(AsdIterator asdIt=GetAsds() ; *asdIt != 0 ; asdIt++)
    for(waxDecodeAsd::ChannelIterator chanIt = (*asdIt)->GetChannels() ;
	chanIt != (*asdIt)->GetChannelsEnd() ; chanIt++) 
      chanIt->SetTdcCount(0) ;
}

int
waxDecodeManager::FillNewEvent()
{
  int rc=OTr_fill_event(fDecodeTable) ;
  if(rc==0 && fgNoiseSuppressionLevel>=1) ApplyEventAsdMasks() ;
  return rc;
}

int waxDecodeManager::GetNAsds() const {
  int n=0;
  for(ConstAsdIterator asdIt=GetAsds() ; *asdIt != 0 ; asdIt++) n++ ; 
  return n;
}

waxDecodeAsd* waxDecodeManager::GetAsd(string asdname) 
{
  waxDecodeAsd* asdIt=0;
  unsigned char slid,layid,secid,ftbid ;
  if(asdid2num(asdname.c_str(),&slid,&layid,&secid,&ftbid)==0 && slid<32) {
    OTrSuperLayer* slIt =  fDecodeTable->Slayers + slid ;
    unsigned char firstasd = slIt->FirstASD[layid][secid] ;
    if(firstasd<255) {
      unsigned short asdId = 
	slIt->Line[slIt->pos[layid][secid] - firstasd + ftbid] ;
      asdIt = GetAsd(asdId) ;
    }
  }
  return asdIt ;
}

string waxDecodeAsd::GetName() const
{
  static char name[12] ;
  num2asdid(GetSuperLayerIndex(),GetLayerIndex(),
	    GetSectorIndex(),GetSubAsdIndex(),name) ;
  return std::string(name) ;
}

void waxDecodeManager::ReadTZerosOTrI()
{
  std::cout << "* Reading tzeros from otrI " << std::endl ;
  // This reads the tzeros from the database
  char * f ;
  if ((f=getenv("OtrDqSwitch"))) OtrDqSwitch(atoi(f));
  else OtrDqSwitch(OTRDQSelectOccupOr);
 
  if ((f=getenv("OtrEventSwitch"))) OtrEventSwitch(atoi(f));
  
  if ((f=getenv("OtrDebugSwitch"))) OtrIDebugSwitch(atoi(f));
  else OtrIDebugSwitch(OTRIDebugOffline); 
  
 OtrISwitch(OTRIUseDb|OTRIUseEnvironment); 
 OtrIInitialize(); 
 OtrEventInfoEventGet(ARTE_get_event());
 OtrDbUpdate(OtrEventInfoDbKey());
 
 float sum(0),esum(0) ;
 int n(0) ;

 for(AsdIterator asdIt = GetAsds() ; (*asdIt)!=0; ++asdIt) {
   int slid  = (*asdIt)->GetSuperLayerIndex() ;
   int layid = (*asdIt)->GetLayerIndex() ;
   int secid = (*asdIt)->GetSectorIndex() ;
   int asdid = (*asdIt)->GetSubAsdIndex() ;
   for(int asdchid=0; asdchid<16; asdchid++) {
     waxDecodeChannel* chanIt = 
       const_cast<waxDecodeChannel*>((*asdIt)->GetChannel(asdchid)) ;
     int ci = OtrIFromAsdId(slid,layid,secid,asdid,asdchid) ;
     if (!OtrIError()) {
       float t0 = OtrITdcT0(ci) ;
       float t0e = OtrT0Sigma(ci) ;
       sum  += t0 ;
       esum += t0e ;
       n++;
       if(t0e>10) t0e=100;
       //if(chanIt->GetTZeroErrCount()==255) t0e=100;
       chanIt->SetTZero(t0,t0e) ;
     } else              chanIt->SetTZero(0,100) ;
   }
 }
 std::cout << "* Number of channels, av tzero(e): " 
	   << n << " " << sum/n << " " <<  esum/n << std::endl ;
}


void waxDecodeManager::ReadTZeros() 
{
#ifdef __WAX_DEBUG__
  std::cout << "waxDecodeManager::ReadTZeros(): " ;
#endif

  if(!strncmp(fgTZeroFileName.c_str(),"none",4)) {
    std::cout << "TZeroFile = none -->Skip reading tzeros." << std::endl ;
    return ;
  }
  
  char runname[9] ;
  //sprintf(runname,"run%5.5d",arteflag_.numrun) ;
#ifdef USE_ARTE
  sprintf(runname,"run%5.5d",ArteTable<EVHD>::begin()->run) ;
#else
  sprintf(runname,"run_noa") ;
#endif
  const std::string filenameRun = waxEventBuilder::GetHBROOTDirName() + "/OTR/db/tzero/" +
    string(runname) + ".tzero" ;
  const std::string filenameGen = waxEventBuilder::GetHBROOTDirName() + "/OTR/db/tzero/otr.tzero"  ;
  
  std::ifstream ifs(fgTZeroFileName.c_str()) ;
  std::cout << "* Reading tzeros from:                           *" 
	    << std::endl << "* " ;
  if(ifs) std::cout << fgTZeroFileName << std::endl;
  else {
    ifs.open(filenameRun.c_str()) ;
    if(ifs) std::cout << filenameRun << std::endl ;
    else {
      ifs.open(filenameGen.c_str()) ;
      if(ifs) std::cout << filenameGen << std::endl ;
      else
	std::cout << "Could not open file under any of these names: " << std::endl
	     << fgTZeroFileName  << std::endl << filenameRun << std::endl << filenameGen << std::endl ;
    }
  }
  while(ifs) {
    std::string asdname ;
    waxDecodeAsd* asdIt ;
    ifs >> asdname ;
    asdIt = GetAsd(asdname) ;
    int tzero,tzeroerr ;
    if(asdIt) {
      for(waxDecodeAsd::ChannelIterator chanIt = asdIt->GetChannels() ;
	  chanIt != asdIt->GetChannelsEnd() ; chanIt++) {
	ifs >> tzero >> tzeroerr ;
	chanIt->SetTZeroCount(tzero,tzeroerr) ;
      }
    } else for(int i=0; i<16; i++) ifs >> tzero >> tzeroerr ;
  }
  ifs.close() ;
}


void waxDecodeManager::WriteTZeros(const std::string& filename) const
{
  std::ofstream ofs(filename.c_str()) ;
  int asdId=0;
  for(ConstAsdIterator asdIt=GetAsds() ; *asdIt != 0 ; asdIt++, asdId++) {
    ofs << (*asdIt)->GetName() << " " ;
    for(waxDecodeAsd::ConstChannelIterator chanIt = (*asdIt)->GetChannels() ;
	chanIt != (*asdIt)->GetChannelsEnd() ; chanIt++) 
      ofs << setw(4) << int(chanIt->GetTZeroCount())
	  << setw(4) << int(chanIt->GetTZeroErrCount()) ;
    ofs << std::endl ;
  }
  ofs.close() ;
}

void waxDecodeManager::ReadStatus() 
{
  // The status flag has 8 bits which are now assigned as follows:
  // * bit 1:   Masked out by Sascha (dead,hot,cold)
  // * bit 2-6: Not assigned
  // * bit 7:   Bad data: Channel/asd bad this event (Maarten's stuff)
  // * bit 8:   No data:  Fed bad this event
  // (The latter is important for efficiency studies.)  Bit 1 is set
  // in this routine. Bit 7 and 8 are set on event basis.

  if(!strncmp(fgMaskFileName.c_str(),"none",4)) {
    std::cout << "MaskFile = none -->Skip reading masks." << std::endl ;
    return ;
  }

  char runname[9] ;
  //sprintf(runname,"run%5.5d",arteflag_.numrun) ;
#ifdef USE_ARTE
  sprintf(runname,"run%5.5d",ArteTable<EVHD>::begin()->run) ;
#else
  sprintf(runname,"run_noa") ;
#endif
  const std::string filenameRun = waxEventBuilder::GetHBROOTDirName() + "/OTR/db/masks/beam-" +
    string(runname) + "/otr.mask" ;
  const std::string filenameGen = waxEventBuilder::GetHBROOTDirName() + "/OTR/db/masks/beam/otr.mask"  ;
  
  std::cout << "* Reading channel status from:                   *" 
    << std::endl << "* " ;

  std::ifstream ifs(fgMaskFileName.c_str()) ;
  if(ifs) std::cout << fgMaskFileName << std::endl;
  else {
    ifs.open(filenameRun.c_str()) ;
    if(ifs) std::cout << filenameRun << std::endl ;
    else {
      ifs.open(filenameGen.c_str()) ;
      if(ifs) std::cout << filenameGen << std::endl ;
      else
	std::cout << "Could not open file under any of these names: " << std::endl
		  << fgMaskFileName  << std::endl 
		  << filenameRun << std::endl 
		  << filenameGen << std::endl ;
    }
  }
  while(ifs) {
    char line[400];
    ifs.getline(line,400) ;
    if(line[0] != '#' && strlen(line)>12) {
      char asdname[12] ;
      char statusstr[17] ;
      sscanf(line,"%s %s",asdname,statusstr) ;
      waxDecodeAsd* asdIt = GetAsd(asdname) ;
      if(asdIt) {
	char* statusIt = statusstr ;
	for(waxDecodeAsd::ChannelIterator chanIt = asdIt->GetChannels() ;
	    chanIt != asdIt->GetChannelsEnd() ; chanIt++, statusIt++) {
	  switch(*statusIt) {
	  case 'D': // dead
	  case 'W': // warm
	  case 'H': // hot
	  case 'X': // non-existent
	  case 'I': // disconnected (during installation)
	  case 'V': // no HV
	  case 'N': // noisy
	  case 'F': // missing fed
	  case 'S': // disconnected (during mass production)
	  case '-': // module not installed
	    chanIt->SetStatus(1) ;
	    break;
	  case '.': // good
	  case 'C': // not checked (during mass production)
	  default:
	    chanIt->SetStatus(0) ;
	  }
	}
      } else std::cout << "Could not find asd: " << asdname << endl ;
    }
  }
  ifs.close() ;
}


void waxDecodeManager::SwapCables()
{
  
#ifdef __WAX_DEBUG__
  std::cout << "waxDecodeManager::ReadStatus(): " ; 
#endif
  //if(!strncasecmp(fgSwapCableFileName.c_str(),"none",4)) {
  if(!strncmp(fgSwapCableFileName.c_str(),"none",4)) {
    std::cout << "* SwapCableFileName = " << fgSwapCableFileName
	      << " --> No cables swapped." << std::endl ;
    return ;
  }

  std::string file = waxEventBuilder::GetHBROOTDirName() + fgSwapCableFileName ;
  std::ifstream ifs(file.c_str()) ;
  if(!ifs) {
    std::cout << "* SwapCableFileName = " << file
              << " : Cannot find --> No cables swapped." 
              << std::endl ;
    return ;
  } 

  std::cout << "* Correcting cabling mistakes from file:         *" 
            << std::endl << "* " << file << std::endl ;
  
  int format ;
  ifs >> format ;
  if(format==1) {
    int nlines(0) ;
    int nasds ;
    ifs >> nasds ;
    while(ifs) {
      std::vector<waxDecodeAsd*> asdvec(nasds) ;
      std::string name;
      bool all_ok = true ;
      for(int i=0; i<nasds; i++) {
	ifs >> name ;
	asdvec[i] = GetAsd(name) ;
	if(asdvec[i]==0) {
	  all_ok = false ;
	  cout << "Strange name: " << name << endl ;
	}
      }
      
      if(all_ok) {
#ifdef __WAX_DEBUG__
	// let's check first
	for(int i=0; i<nasds; ++i) {
	  waxDecodeAsd* asdIt = asdvec[i];
	  waxDecodeTdc* tdcIt = GetTdc(asdIt->GetFedIndex()) ;
	  if(GetAsd(tdcIt->GetAsdIndex(asdIt->GetTdcConnectorIndex()-1)) !=
	     asdIt) cout << "Shit: already connection error." << endl ;
	}
#endif
	
	for(int i=0; i<nasds-1; ++i) {
	  waxDecodeAsd* asdIt = asdvec[i];
	  waxDecodeAsd* nextasdIt = asdvec[i+1];
	  waxDecodeTdc* tdcIt = GetTdc(asdIt->GetFedIndex()) ; 
	  waxDecodeTdc* nexttdcIt = GetTdc(nextasdIt->GetFedIndex()) ;
	  // swap info in tdc
	  swap(tdcIt->fTdc.ASD[asdIt->GetTdcConnectorIndex()-1],
	       nexttdcIt->fTdc.ASD[nextasdIt->GetTdcConnectorIndex()-1]) ;  
	  // swap info in asd
	  swap(asdIt->fAsd.TDC_FED,nextasdIt->fAsd.TDC_FED) ;
	  swap(asdIt->fAsd.TDC_Connector,nextasdIt->fAsd.TDC_Connector) ;
	}
	
#ifdef __WAX_DEBUG__
	// now check:
	for(int i=0; i<nasds; ++i) {
	  waxDecodeAsd* asdIt = asdvec[i];
	  waxDecodeTdc* tdcIt = GetTdc(asdIt->GetFedIndex()) ;
	  if(GetAsd(tdcIt->GetAsdIndex(asdIt->GetTdcConnectorIndex()-1)) !=
	     asdIt) cout << "Shit: connection error introduced." << endl ;
	}
#endif
      }
      nlines++;
      ifs >> nasds ;
    }
#ifdef __WAX_DEBUG__
    std::cout << "Read " << nlines << " lines." << endl ;
#endif
  } else std::cout << "SwapCables error: wrong format." << endl ;
  ifs.close() ;
}

std::vector<FedIndex_t> 
waxDecodeManager::GetActiveFedIds() const 
{
  // returns a vector with the ids of all outer tracker feds
  std::vector<FedIndex_t> fedids;
  for(FedIndex_t fedid = 0; fedid<MAXFED; fedid++) 
    if(fDecodeTable->FEDs[fedid]!=MAXFED)
      fedids.push_back(fedid) ;
  return fedids;
}

void
waxDecodeManager::ApplyEventAsdMasks()
{
  // Bit 7 of the status flag is assigned to this masking
  // algorithm. If an asd is determined bad in this event, all the bit
  // 7 are set.
  // this algorithm maskes out any combination of at least three hits
  // with small time differences. 
  
  ArtePointer<GEDE> gede;

  waxDecodeChannel* chanIt1(0);
  waxDecodeChannel* chanIt2(0);
  waxDecodeChannel* chanIt3(0);
  int nhits(0) ;
  //int NrMaskedHits;

  int t1(0),t2(0),t3(0);
  bool mask;

  for(AsdIterator asdIt = GetAsds(); (*asdIt) != 0; asdIt++) {
    nhits=0;
    //switch 'on' by default
    for(int chanid1=0; chanid1<16; chanid1++) {
      chanIt1= const_cast<waxDecodeChannel*>((*asdIt)->GetChannel(chanid1)) ;
      chanIt1->SetStatus(chanIt1->GetStatus() & ~(1<<6)) ;
      t1=chanIt1->GetTdcCount();
      if( chanIt1->IsHit() && chanIt1->GetStatus()==0) nhits++;
    }
    mask=false;

    if(nhits>2) {
      //first channel
      for(int chanid1=0; chanid1<14; chanid1++) {
	chanIt1= const_cast<waxDecodeChannel*>((*asdIt)->GetChannel(chanid1)) ;
	t1=chanIt1->GetTdcCount();
	if(chanIt1->IsHit() && chanIt1->GetStatus()==0) {
	  //second channel
	  for(int chanid2=chanid1+1; chanid2<15; chanid2++) {
	    chanIt2= const_cast<waxDecodeChannel*>((*asdIt)->GetChannel(chanid2)) ;
	    t2=chanIt2->GetTdcCount();
	    if(chanIt2->IsHit() && chanIt2->GetStatus()==0) {
	      if(abs(t1-t2)<fgTimeDiffWindow) {
		//third channel also in that time window ?
		for(int chanid3=chanid2+1; chanid3<16; chanid3++) {
		  chanIt3= const_cast<waxDecodeChannel*>((*asdIt)->GetChannel(chanid3)) ;
		  t3=chanIt3->GetTdcCount();
		  if(chanIt3->IsHit() && chanIt3->GetStatus()==0) {
		    if(abs(t1-t3)<fgTimeDiffWindow) {
		      chanIt1->SetStatus(chanIt1->GetStatus() |  (1<<6)) ;
		      chanIt2->SetStatus(chanIt2->GetStatus() |  (1<<6)) ;
		      chanIt3->SetStatus(chanIt3->GetStatus() |  (1<<6)) ;
		      mask=true;
		    }
		  }
		}
	      }
	    }
	  }
	}
      }
      
      // make cut on multiplicity
      gede = chanIt1->GetGedeIndex();
      if(gede->nsl==2) { //single layer 
	//mask out complete ASD when it is really bad
	if((nhits>fgMaxNHitsSL) && mask) {
	  mask=true;
	} else {
	  mask=false;
	}
      } else { // double layer
	//mask out complete ASD when it is really bad
	if((nhits>fgMaxNHitsDL) && mask) {
	  mask=true;
	} else {
	  mask=false;
	}
      }
      
      // switch off complete ASD when it is really bad
      for(chanIt1 = (*asdIt)->GetChannels() ;
	  chanIt1 < (*asdIt)->GetChannelsEnd() ; chanIt1++)
	switch(mask) 
	  {
	  case true:
	    chanIt1->SetStatus(chanIt1->GetStatus() |  (1<<6)) ;
	    break;
	  default:
	    chanIt1->SetStatus(chanIt1->GetStatus() & ~(1<<6)) ;
	  }
    }
  }
}

